/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.soap.axis;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.coordinator.CoordinatorIFace;
import it.eng.spago.dispatching.coordinator.DispatcherManager;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.navigation.Navigator;
import it.eng.spago.presentation.PresentationRendering;
import it.eng.spago.presentation.Publisher;
import it.eng.spago.tracing.TracerSingleton;

import java.util.ArrayList;

import org.apache.axis.MessageContext;
import org.apache.axis.session.Session;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           Introdotto gestione del publisher di tipo loop 
* 										    
**/

/**
 * @author 
 */
public class AdapterAxis {
    
    
	public static final String REQUEST_CONTAINER = "REQUEST_CONTAINER";
    
    public AdapterAxis() {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "AdapterSOAP::AdapterSOAP: invocato");
        
    } // public ActionWebService()
    
    public String service(String request) {
        
    	TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "AdapterSOAP::service: request\n" + request);
		
		return commonService(request, null);
     }
    
	private String serviceLoop(String request, EMFErrorHandler errorHandler) {
		TracerSingleton.log(
			Constants.NOME_MODULO,
			TracerSingleton.DEBUG,
			"AdapterSOAP::serviceLoop: request\n" + request);
		
		//permette di mantenere nell'EMFErrorHandler gli eventuali errori verificatisi durante l'esecuzione
		//della prima page o action
		return commonService(request, errorHandler);
	}    

    public String commonService(String request, EMFErrorHandler errorHandler) {
		Monitor monitor = MonitorFactory.start("controller.adapter.soap");
		try {
			SourceBean serviceRequest = SourceBean.fromXMLString(request);
			RequestContainer requestContainer = new RequestContainer();
            RequestContainer.setRequestContainer(requestContainer);
            ResponseContainer responseContainer = new ResponseContainer();
            RequestContextIFace requestContext = new DefaultRequestContext(requestContainer,
                    responseContainer);
	        
			requestContainer.setChannelType(Constants.SOAP_CHANNEL);
			requestContainer.setServiceRequest(serviceRequest);
			
			TracerSingleton.log(
				Constants.NOME_MODULO,
				TracerSingleton.DEBUG,
				"AdapterSOAP::commonService: requestContainer",
				requestContainer);
			TracerSingleton.log(
				Constants.NOME_MODULO,
				TracerSingleton.DEBUG,
				"AdapterSOAP::commonService: sessionContainer",
				requestContainer.getSessionContainer());
			
			
			EMFErrorHandler emfErrorHandler = errorHandler;
			if(errorHandler==null)
				emfErrorHandler = new EMFErrorHandler();
			responseContainer.setErrorHandler(emfErrorHandler);
			SourceBean serviceResponse =
				new SourceBean(Constants.SERVICE_RESPONSE);
			responseContainer.setServiceResponse(serviceResponse);
			Navigator.checkNavigation(requestContainer);
			serviceRequest = requestContainer.getServiceRequest();
			
			MessageContext mc = MessageContext.getCurrentContext();
	        Session wsSession =  mc.getSession();
	        
			checkAxisSession(wsSession, requestContext);
			
			CoordinatorIFace coordinator =
				DispatcherManager.getCoordinator(requestContext);
			
			Exception serviceException = null;
			if (coordinator == null) {
				TracerSingleton.log(
					Constants.NOME_MODULO,
					TracerSingleton.WARNING,
					"AdapterSOAP::commonService: coordinator nullo !");
				serviceException = new Exception("Coordinatore non trovato");
				emfErrorHandler.addError(
					new EMFInternalError(
						EMFErrorSeverity.ERROR,
						"Coordinatore non trovato !"));
			} // if (coordinator == null)
			else {
				RequestContainer.setRequestContainer(requestContainer);
				((RequestContextIFace) coordinator).setRequestContext(
					requestContext);
                responseContainer.setBusinessType(coordinator.getBusinessType());
                responseContainer.setBusinessName(coordinator.getBusinessName());
                responseContainer.setPublisherName(coordinator.getPublisherName());

                try {
					coordinator.service(serviceRequest, serviceResponse);
				} // try
				catch (Exception ex) {
					TracerSingleton.log(
						Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"AdapterSOAP::commonService:",
						ex);
					serviceException = ex;
					emfErrorHandler.addError(
						new EMFInternalError(EMFErrorSeverity.ERROR, ex));
				} // catch (Exception ex)
				 ((RequestContextIFace) coordinator).setRequestContext(null);
				RequestContainer.delRequestContainer();
			} // if (coordinator == null) else
			
			TracerSingleton.log(
				Constants.NOME_MODULO,
				TracerSingleton.DEBUG,
				"AdapterSOAP::commonService: responseContainer",
				responseContainer);
			TracerSingleton.log(
				Constants.NOME_MODULO,
				TracerSingleton.DEBUG,
				"AdapterSOAP::commonService: sessionContainer",
				requestContainer.getSessionContainer());
			String publisherType = null;
			ArrayList resources = null;
			Publisher publisher =
				Publisher.getPublisher(requestContext,
					serviceException);
			
			String response = null;
            
			if (publisher == null) {
				publisherType = Constants.NOTHING_PUBLISHER_TYPE;
				resources = null;
			} // if (publisher == null)
			else {
				publisherType = publisher.getType();
				if ((publisherType == null)
					|| (publisherType
						.equalsIgnoreCase(Constants.NOTHING_PUBLISHER_TYPE))) {
					publisherType = Constants.NOTHING_PUBLISHER_TYPE;
					resources = null;
				} // if ((publisherType == null) ||
				// (publisherType.equalsIgnoreCase(Constants.NOTHING_PUBLISHER_TYPE)))
				else if (//Gestione del publisher di tipo LOOP
					publisherType.equalsIgnoreCase(
						Constants.LOOP_PUBLISHER_TYPE)) {
					
					String requestLoop = responseContainer.getLoopbackServiceRequest().toXML(false);
					response = serviceLoop(requestLoop,responseContainer.getErrorHandler());

					SourceBean resp = SourceBean.fromXMLString(response);
					if(resp.getAttribute("SERVICE_RESPONSE.ACTION_NAME")!=null) resp.delAttribute("SERVICE_RESPONSE.ACTION_NAME");
					if(resp.getAttribute("SERVICE_RESPONSE.PAGE")!=null) resp.delAttribute("SERVICE_RESPONSE.PAGE");
					if(resp.getAttribute("SERVICE_RESPONSE.NAVIGATOR_DISABLED")!=null) resp.delAttribute("SERVICE_RESPONSE.NAVIGATOR_DISABLED");

					return resp.toXML(false);
				} // if (publisherType.equalsIgnoreCase(AF_PUBLISHER_TYPE))
				else {
					resources = publisher.getResources();
					if ((resources == null) || (resources.size() == 0)) {
						publisherType = Constants.NOTHING_PUBLISHER_TYPE;
						resources = null;
					} // if ((resources == null) || (resources.size() == 0))
				} // if ((publisherType == null) ||
				// (publisherType.equalsIgnoreCase(Constants.NOTHING_PUBLISHER_TYPE))) else
			} // if (publisher == null) else
			Monitor renderingMonitor = null;
			if (publisher == null)
				renderingMonitor =
					MonitorFactory.start(
						"view."
							+ Constants.SOAP_CHANNEL
							+ "."
							+ publisherType.toLowerCase());
			else
				renderingMonitor =
					MonitorFactory.start(
						"view."
							+ Constants.SOAP_CHANNEL
							+ "."
							+ publisherType.toLowerCase()
							+ "."
							+ publisher.getName().toLowerCase());
			try {
				if(response==null)
					response = PresentationRendering.render(responseContainer, resources);
			} // try
			catch (Exception ex) {
				throw ex;
			} // catch (Exception ex)
			finally {
				renderingMonitor.stop();
			} // finally
			//System.out.println("response Prima di return="+response);
			return response;
		} // try
		catch (Exception ex) {
			TracerSingleton.log(
				Constants.NOME_MODULO,
				TracerSingleton.CRITICAL,
				"AdapterSOAP::commonService:",
				ex);

			return "";
		} // catch (Excpetion ex) try
		finally {
			monitor.stop();
		} // finally
	} // public String commonService(String request, EMFErrorHandler errorHandler)

    private void checkAxisSession(Session axisSession, RequestContextIFace requestContext) {
       
        RequestContainer requestContainer = requestContext.getRequestContainer();
        
        synchronized (axisSession) {
            RequestContainer parentRequestContainer = (RequestContainer) axisSession
                    .get(REQUEST_CONTAINER);
            if (!Navigator.isNavigatorEnabled())
                if (parentRequestContainer == null)
                    requestContainer.setSessionContainer(new SessionContainer(true));
                else
                    requestContainer.setSessionContainer(parentRequestContainer
                            .getSessionContainer());
            else {
                if (parentRequestContainer == null)
                    requestContainer.setSessionContainer(new SessionContainer(true));
                else {
                    requestContainer.setSessionContainer(new SessionContainer(false));
                    requestContainer.setParent(parentRequestContainer);
                } // if (parentRequestContainer == null) else
            } // if (!Navigator.isNavigatorEnabled())
            axisSession.set(REQUEST_CONTAINER, requestContainer);
        } // synchronized (session)
        
    }
    
    
   
} // public class AdapterSOAP
