/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.base;

import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import it.eng.spago.tracing.TracerSingleton;

/**
 * La classe <code>SourceBeanContentHandler</code> implementa l'interfaccia
 * standard di un parser SAX. Questo parser ricostruisce un oggetto di tipo
 * <code>SourceBean</code> da uno stream XML.
 * @version 1.0, 11/03/2002
 * @author Luigi Bellio
 * @see SourceBeanAttribute
 * @see SourceBean
 */
public class SourceBeanContentHandler extends DefaultHandler {
    /**
     * Costruisce ed inizializza il parser.
     */
    public SourceBeanContentHandler() {
        _sourceBeans = new Stack();
        _sourceBean = null;
        _parentBean = null;
        _characters = new StringBuffer();
        _trim = true;
    } // public SourceBeanContentHandler()
    
    /**
     * Costruisce ed inizializza il parser.
     */
    public SourceBeanContentHandler(boolean trim) {
    	this();
        _trim = trim;
    } // public SourceBeanContentHandler()

    /**
     * Gestisce l'evento di inizio stream XML.
     */
    public void startDocument() throws SAXException {
        //        TracerSingleton.log(Constants.NOME_MODULO,
        // TracerSingleton.DEBUG, "SourceBeanContentHandler::startDocument: invocato");
    } // public void startDocument() throws SAXException

    /**
     * Gestisce l'evento di fine stream XML.
     */
    public void endDocument() throws SAXException {
        //        TracerSingleton.log(Constants.NOME_MODULO,
        // TracerSingleton.DEBUG, "SourceBeanContentHandler::endDocument: invocato");
    } // public void endDocument() throws SAXException

    /**
     * Gestisce l'evento di inizio elemento XML.
     */
    public void startElement(String namespaceURI, String localName, String rawName,
        Attributes attributes) throws SAXException {
            try {
                SourceBean currentBean = new SourceBean(localName, _trim);
                for (int i = 0; i < attributes.getLength(); i++) {
                    currentBean.setAttribute(attributes.getLocalName(i), attributes.getValue(i));
                } // for (int i = 0; i < attributes.getLength(); i++)
                if (_parentBean != null) {
                    _parentBean.setAttribute(currentBean);
                    
                    if (_characters.length() > 0) {
                        _parentBean.setCharacters(_characters.toString());
                        _characters = new StringBuffer();
                    } // if (_characters.length() > 0)
                }
                
                _sourceBeans.push(currentBean);
                _parentBean = currentBean;
            } // try
            catch (SourceBeanException ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "SourceBeanContentHandler::startElement:", ex);
                throw new SAXException(ex.getMessage());
            } // catch (SourceBeanException ex) try
    } // public void startElement(String namespaceURI, String localName,
    // String rawName, Attributes attributes) throws SAXException

    /**
     * Gestisce l'evento di fine elemento XML.
     */
    public void endElement(String namespaceURI, String localName, String rawName) throws SAXException {
        if (_characters.length() > 0) {
            _parentBean.setCharacters(_characters.toString());
            _characters = new StringBuffer();
        } // if (_characters.length() > 0)
        _sourceBean = (SourceBean)_sourceBeans.pop();
        if (_sourceBeans.empty())
            _parentBean = null;
        else
            _parentBean = (SourceBean)_sourceBeans.peek();
    } // public void endElement(String namespaceURI, String localName, String rawName) throws SAXException

    /**
     * Gestisce il testo di un elemento XML.
     */
    public void characters(char[] ch, int start, int length) throws SAXException {
        try {
            _characters.append(ch, start, length);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "SourceBeanContentHandler::characters: _characters.append(ch, start, length)", ex);
            throw new SAXException(ex.getMessage());
        } // catch (Exception ex) try
    } // public void characters(char[] ch, int start, int length) throws SAXException

    /**
     * Ritorna il <code>SourceBean</code> costruito a partire dallo stream XML.
     */
    public synchronized SourceBean getSourceBean() {
        return _sourceBean;
    } // public synchronized SourceBean getSourceBean()

    /**
     * Stack dei <code>SourceBean</code> intermedi.
     */
    private Stack _sourceBeans = null;

    /**
     * <code>SourceBean</code> costruito a partire dallo stream XML.
     */
    private SourceBean _sourceBean = null;

    /**
     * <code>SourceBean</code> padre di quello corrente.
     */
    private SourceBean _parentBean = null;
    private StringBuffer _characters = null;
    private boolean _trim = true;
} // public class SourceBeanContentHandler extends DefaultHandler
