/**

 Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.cache;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.coordinator.AbstractCoordinator;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.tracing.TracerSingleton;

/**
 * This class is reponsible to cache the response of a service if the a <REACH-SERVICE> envelope
 * is found on the configuration of the services ( under the <PAGE> or <ACTION> envelope
 *
 * @author Andrea Zoppello
 */
public class CacheResponseHandler {

	public static final String REACH_SERVICE = "REACH_SERVICE";
	public static final String CACHED_RESPONSE = "CACHED_RESPONSE";
    public static final String CACHED_RESPONSE_SB = "CACHED_RESPONSE_SB";
    public static final String CACHED_RESPONSE_ATTRIBUTE = "AF_CACHED_RESPONSE";
	
	/**
	 * 
	 * @param requestContainer Spago request container
	 * @param responseContainer Spago response container
	 * @param publisher the publieher that will be rendered after the response have been cached
	 * 
	 * This method determine the current service just executet that will be rendered 
	 * on the determinated publisher ( this method is called on the render method of the AdapterHTTP) and then
	 * if from this services we can reach other services we'll cache the service response otherwise nothing is done
	 */
	public static void service(RequestContainer requestContainer, ResponseContainer responseContainer, String publisher){
		
		EMFErrorHandler errorHandler = responseContainer.getErrorHandler();
		if ((errorHandler == null) || (errorHandler.isOK())){
			String businessName = (String)requestContainer.getAttribute(AbstractCoordinator.CURRENT_SERVICE_NAME);	
			String businessType = (String)requestContainer.getAttribute(AbstractCoordinator.CURRENT_SERVICE_TYPE);
		
			ConfigSingleton configure = ConfigSingleton.getInstance();
			SourceBean serviceSourceBean = null;
			if (businessType.equalsIgnoreCase("PAGE")){
		        serviceSourceBean =
		            (SourceBean) configure.getFilteredSourceBeanAttribute(
                        Constants.PAGE_PATH,
		                "NAME",
		                businessName);
			}else{
				serviceSourceBean = (SourceBean)configure.getFilteredSourceBeanAttribute(
                        Constants.ACTION_PATH, 
						"NAME", 
						businessName);
			}
			
			if ( serviceSourceBean.getAttribute("REACH-SERVICES") != null ){
				SessionContainer permanentContainer = requestContainer.getSessionContainer().getPermanentContainer();
			
				try{
					SourceBean cachedResponseSourceBean = getCachedResponse(permanentContainer);
                    SourceBean serviceResponse = responseContainer.getServiceResponse();
                    				
					// Cache the service Response
					cachedResponseSourceBean.updAttribute(publisher, serviceResponse);
				}catch (Exception e) {
					TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
		            "CacheResponseHandler::service: ", e);
				}
			}
			
		}
		
	}
	
	/**
	 * 
	 * @param permanentContainer Spago session permanent conatiner
	 * @return The source bean under the permanent container that contains the cached responses
	 * @throws SourceBeanException
	 * 
	 * This method return the source bean under the permanent container that will contain th cached responses
	 */
	public static SourceBean getCachedResponse(SessionContainer permanentContainer) throws SourceBeanException {
		
			if (permanentContainer.getAttribute(CACHED_RESPONSE) == null){
				SourceBean sb = new SourceBean(CACHED_RESPONSE_SB);
				permanentContainer.setAttribute(CACHED_RESPONSE, sb);
			}
		
			return (SourceBean)permanentContainer.getAttribute(CACHED_RESPONSE);
	
		
	}
}
