/**

Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

This file is part of Spago.

Spago is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
any later version.

Spago is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Spago; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.cache;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.coordinator.AbstractCoordinator;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.presentation.AbstractPublisherDispatcher;
import it.eng.spago.tracing.TracerSingleton;

import java.util.List;

/**
 * 
 * @author zoppello - This is a Java publisher that MUST BE USED we we want to retrieve 
 * the cached reponse of previous service when the current service execution fails with errors
 */
public class DynamicCacheResponsePublisher extends AbstractPublisherDispatcher {
	/**
	 *
	 */
	public DynamicCacheResponsePublisher() {
		super();

	}

	
	/**
	 * @param requestContainer The spago request container
	 * @param responseContainer The spago response container
	 * 
	 * If errors occurs during the execution of a service, the last publisher name and the cached response are retrieved
	 * and the render will be redirected on the last publisher.
	 * If  execution of the current service is ok the render will be on the publisher configured in TARGET-PUBLISHER element 
	 * of the java publisher
	 */
	public String getPublisherName(RequestContainer requestContainer,
			ResponseContainer responseContainer) {
	
		EMFErrorHandler errorHandler = responseContainer.getErrorHandler();
		String businessName = (String)requestContainer.getAttribute(AbstractCoordinator.CURRENT_SERVICE_NAME);

		String targetPublisher  = ((SourceBean)(getConfig().getAttribute("TARGET_PUBLISHER"))).getCharacters();
		String lastPublisherName = getLastPublisherName(requestContainer);
		SourceBean cachedResponseForPublisher = getCachedResponseForPublisher(requestContainer, lastPublisherName);
		if ((errorHandler != null) && (!errorHandler.isOK())){
			// Si sono verificati errori nell'esecuzione del servizio
			if ((isReachable(businessName)) && ( cachedResponseForPublisher != null)){
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
		            	"DynamicCacheResponsePublisher::getPublisherName: Retrieving cached response");

                // Allow to recognize that this is a cached response
				try {
    				cachedResponseForPublisher.updAttribute(CacheResponseHandler.CACHED_RESPONSE_ATTRIBUTE, "TRUE");
				} catch (SourceBeanException ex) {
					TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
				            "CacheResponseListener::onEvent: ", ex);
				}

				// Set cached response as THE response
				responseContainer.setServiceResponse(cachedResponseForPublisher);
			}
			return lastPublisherName;
		}else{
			
			if ((isReachable(businessName)) && ( cachedResponseForPublisher != null)){
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
		            	"DynamicCacheResponsePublisher::getPublisherName: Removing cached response from cache");
				deleteCachedResponseForPublisher(requestContainer, lastPublisherName);
			}
			return targetPublisher;
		}
	}
	
	
	public boolean isReachable(String businessName){
		
		List isReachableList = (List)ApplicationContainer.getInstance().getAttribute("REACHABLE_LIST");
		
		return isReachableList.contains(businessName);
	}
	
	
	public String getLastPublisherName(RequestContainer requestContainer){
		
		return (String)requestContainer.getSessionContainer().getPermanentContainer().getAttribute("LAST_PUBLISHER_NAME");
		
	}
	
	public SourceBean getCachedResponseForPublisher(RequestContainer requestContainer, String publisherName){
	
		SessionContainer permanentContainer = requestContainer.getSessionContainer().getPermanentContainer(); 
		
		SourceBean cachedResponses =  (SourceBean)permanentContainer.getAttribute(CacheResponseHandler.CACHED_RESPONSE);
		
		if (cachedResponses != null){
			
			SourceBean sb = (SourceBean)cachedResponses.getAttribute(publisherName);
			return (SourceBean)sb.getAttribute(Constants.SERVICE_RESPONSE);
		}else{
			
			return null;
		
		}
	}
	
	public void deleteCachedResponseForPublisher(RequestContainer requestContainer, String publisherName){
		
		SessionContainer permanentContainer = requestContainer.getSessionContainer().getPermanentContainer(); 
		
		SourceBean cachedResponses =  (SourceBean)permanentContainer.getAttribute(CacheResponseHandler.CACHED_RESPONSE);
		
		if (cachedResponses != null){
			try{
				cachedResponses.delAttribute(publisherName);
			}catch (SourceBeanException ex) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "Error in deleting cached response for publisher ["+publisherName+"]", ex);
			}
		}	
	}
	
}

