/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

/**
 * Questa classe ha il compito di recuperare la stringa rappresentante lo statement SQL da un
 * file XML.
 */
public class OptimizedSQLStatements {
	
	private static HashMap cache = null;
	
    /**
     * Questo metodo recupera lo statement SQL avente associato alla <em>key</em> name.
     * @param String nome logico dello statement SQL.
     * @return  <code>String</code> corrispondente allo statement SQL
     */
    public static String getStatement(String name) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "SQLStatements::getStatement: invocato");
        if (name == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "SQLStatements::getStatement: nome dello statement nullo");
            return null;
        } // if (name == null)
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
            "SQLStatements::getStatement: name [" + name + "]");
        
		name = name.toUpperCase();
		String statement = (String)cache.get(name);
		if (statement == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
				"SQLStatements::getStatement: statement non trovato");
		}

        return statement;
    } // public static String getStatement(String name)
    
	/**
	 * Questo metodo recupera lo statement SQL avente associato alla <em>key</em> name.
	 * @param String nome logico dello statement SQL.
	 * @return  <code>String</code> corrispondente allo statement SQL
	 */
	public static String getStatement(String name,String area) {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "SQLStatements::getStatement: invocato");
		if (name == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
				"SQLStatements::getStatement: nome dello statement nullo");
			return null;
		} // if (name == null)
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
			"SQLStatements::getStatement: name [" + name + "]");
		ConfigSingleton configure = ConfigSingleton.getInstance();
		SourceBean statementConfig = (SourceBean)configure.getFilteredSourceBeanAttribute(
			"STATEMENTS_" + area + ".STATEMENT", "NAME", name);
		if (statementConfig == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
				"SQLStatements::getStatement: statement non trovato");
			return null;
		} // if (statementConfig == null)
		String statement = (String)statementConfig.getAttribute("QUERY");
		if (statement == null)
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
				"SQLStatements::getStatement: statement non trovato");
		return statement;
	} // public static String getStatement(String name,String area)

	static {
		List statements = ConfigSingleton.getInstance().getAttributeAsList("STATEMENTS.STATEMENT");
		// Calcolo la capacit iniziale in modo da non fare rehashing durante il
		// popolamento dell'hashmap e in modo da mantenere il load factor sotto
		// al 75% (il +10 serve per evitare di mantenersi sul limite del 75%)
		int initialCapacity = (int)(1.0 * statements.size() / 0.75) + 10;
		cache = new HashMap(initialCapacity);
		for (Iterator it = statements.iterator(); it.hasNext();) {
			SourceBean sbStatement = (SourceBean)it.next();
			String statement = (String)sbStatement.getAttribute("QUERY");
			String name = (String)sbStatement.getAttribute("NAME");
			cache.put(name.toUpperCase(), statement);
		}
	}

} // public class SQLStatements
