/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess.encript;

import java.io.IOException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.SecretKeySpec;

import sun.misc.BASE64Decoder;

import it.eng.spago.base.Constants;
import it.eng.spago.dbaccess.pool.IDecriptAlgorithm;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;
/**
 * @author Bernabei Angelo
 * @version 1.0
 */
public class PasswordProvider implements IDecriptAlgorithm {
	/**
	 *		Classe che implementa l'algoritmo di cifratura	 
	 */
	private Cipher cifro= null;
	/**
	 *		Chiave utilizzata per l'algoritmo Cipher 
	 */
	private static SecretKeySpec desKey= null;


	/**
	 * Costruttore
	 */
	public PasswordProvider() throws EMFInternalError{
		init();
		
	}
	/**
	 * Metodo che inizializza alcuni attributi necessari.
	 * 	- legge la key
	 * 	- istanza la classe che esegue la decifratura.
	 *
	 */
	private void init() throws EMFInternalError {
			try {
				BASE64Decoder decoder= new BASE64Decoder();
				byte[] byteKey= null;
				byteKey= decoder.decodeBuffer(Key.KEY);
				desKey = new SecretKeySpec(byteKey, "DES");
				cifro= Cipher.getInstance("DES/ECB/PKCS5Padding");
			} catch (NoSuchAlgorithmException e) {
				TracerSingleton.log(
					Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"PasswordProvider::init",
					e);
				throw new EMFInternalError(
					EMFErrorSeverity.ERROR,
					"PasswordProvider::init::NoSuchAlgorithmException" + e.getMessage());
			} catch (NoSuchPaddingException e) {
				TracerSingleton.log(
					Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"PasswordProvider::create",
					e);
				throw new EMFInternalError(
					EMFErrorSeverity.ERROR,
					"PasswordProvider::init::NoSuchPaddingException" + e.getMessage());
			}catch (IOException e1) {
			e1.printStackTrace();
			}

	}

	/**
	 * 	Decifra la password.
	 * 	@param psw:String password cifrata
	 * 	@return String password in chiaro
	 */
	public String decipher(String psw) throws EMFInternalError {
		if (psw==null) return null;
		BASE64Decoder decoder= new BASE64Decoder();
		byte[] byteInput= null;
		byte[] cifrata= null;
		try {
			byteInput= decoder.decodeBuffer(psw);
			if (cifro==null) {
				TracerSingleton.log(
					Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"PasswordProvider::decipher::CLASSE Cipher NULLA");				
				throw new EMFInternalError(
					EMFErrorSeverity.ERROR,
					"PasswordProvider::init::CLASSE Cipher NULLA");
			}
				cifro.init(Cipher.DECRYPT_MODE, desKey);
			    cifrata= cifro.doFinal(byteInput);
		   
		} catch (InvalidKeyException e) {
			TracerSingleton.log(
				Constants.NOME_MODULO,
				TracerSingleton.CRITICAL,
				"PasswordProvider::decipher",
				e);
			throw new EMFInternalError(
				EMFErrorSeverity.ERROR,
				"PasswordProvider::init::InvalidKeyException" + e.getMessage());
		} catch (IllegalStateException e) {
			TracerSingleton.log(
				Constants.NOME_MODULO,
				TracerSingleton.CRITICAL,
				"PasswordProvider::decipher",
				e);
			throw new EMFInternalError(
				EMFErrorSeverity.ERROR,
				"PasswordProvider::init::IllegalStateException" + e.getMessage());
		} catch (IllegalBlockSizeException e) {
			TracerSingleton.log(
				Constants.NOME_MODULO,
				TracerSingleton.CRITICAL,
				"PasswordProvider::decipher",
				e);
			throw new EMFInternalError(
				EMFErrorSeverity.ERROR,
				"PasswordProvider::init::IllegalBlockSizeException" + e.getMessage());
		} catch (BadPaddingException e) {
			TracerSingleton.log(
				Constants.NOME_MODULO,
				TracerSingleton.CRITICAL,
				"PasswordProvider::decipher",
				e);
			throw new EMFInternalError(
				EMFErrorSeverity.ERROR,
				"PasswordProvider::init::BadPaddingException" + e.getMessage());
		} catch (IOException e) {
			TracerSingleton.log(
				Constants.NOME_MODULO,
				TracerSingleton.CRITICAL,
				"PasswordProvider::decipher",
				e);
			throw new EMFInternalError(
				EMFErrorSeverity.ERROR,
				"PasswordProvider::init::IOException" + e.getMessage());
		}
		return new String(cifrata);
	}
}
