/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess.hibernate;

import java.io.Serializable;
import java.sql.Connection;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import net.sf.hibernate.Criteria;
import net.sf.hibernate.FlushMode;
import net.sf.hibernate.HibernateException;
import net.sf.hibernate.LockMode;
import net.sf.hibernate.Query;
import net.sf.hibernate.ReplicationMode;
import net.sf.hibernate.Session;
import net.sf.hibernate.SessionFactory;
import net.sf.hibernate.Transaction;
import net.sf.hibernate.type.Type;

import it.eng.spago.base.RequestContainer;

public abstract class AbstractHibernateSession implements Session {
    private Session _nativeSession = null;

    public AbstractHibernateSession(Session nativeSession) {
        super();
        _nativeSession = nativeSession;
    } // public HibernateSession(Session nativeSession)

    public void flush() throws HibernateException {
        _nativeSession.flush();
    } //public void flush() throws HibernateException 

    public void setFlushMode(FlushMode flushMode) {
        _nativeSession.setFlushMode(flushMode);
    } // public void setFlushMode(FlushMode flushMode)

    public FlushMode getFlushMode() {
        return _nativeSession.getFlushMode();
    } // public FlushMode getFlushMode()

    public SessionFactory getSessionFactory() {
        return _nativeSession.getSessionFactory();
    } // public SessionFactory getSessionFactory()

    public Connection connection() throws HibernateException {
        return _nativeSession.connection();
    } // public Connection connection() throws HibernateException

    public Connection disconnect() throws HibernateException {
        return _nativeSession.disconnect();
    } // public Connection disconnect() throws HibernateException

    public void reconnect() throws HibernateException {
        _nativeSession.reconnect();
    } // public void reconnect() throws HibernateException

    public void reconnect(Connection connection) throws HibernateException {
        _nativeSession.reconnect(connection);
    } // public void reconnect(Connection connection) throws HibernateException

    public Connection close() throws HibernateException {
        return _nativeSession.close();
    } // public Connection close() throws HibernateException

    public void cancelQuery() throws HibernateException {
        _nativeSession.cancelQuery();
    } // public void cancelQuery() throws HibernateException

    public boolean isOpen() {
        return _nativeSession.isOpen();
    } // public boolean isOpen()

    public boolean isConnected() {
        return _nativeSession.isConnected();
    } // public boolean isConnected()

    public boolean isDirty() throws HibernateException {
        return _nativeSession.isDirty();
    } // public boolean isDirty() throws HibernateException

    public Serializable getIdentifier(Object object)
        throws HibernateException {
        return _nativeSession.getIdentifier(object);
    } // public Serializable getIdentifier(Object object) throws HibernateException

    public boolean contains(Object object) {
        return _nativeSession.contains(object);
    } // public boolean contains(Object object)

    public void evict(Object object) throws HibernateException {
        _nativeSession.evict(object);
    } // public void evict(Object object) throws HibernateException

    public Object load(Class theClass, Serializable id, LockMode lockMode)
        throws HibernateException {
        return _nativeSession.load(theClass, id, lockMode);
    } // public Object load(Class theClass, Serializable id, LockMode lockMode) throws HibernateException

    public Object load(Class theClass, Serializable id)
        throws HibernateException {
        return _nativeSession.load(theClass, id);
    } // public Object load(Class theClass, Serializable id) throws HibernateException

    public void load(Object object, Serializable id)
        throws HibernateException {
        _nativeSession.load(object, id);
    } // public void load(Object object, Serializable id) throws HibernateException

    public void replicate(Object object, ReplicationMode replicationMode)
        throws HibernateException {
        _nativeSession.replicate(object, replicationMode);
    } // public void replicate(Object object, ReplicationMode replicationMode) throws HibernateException

    public Serializable save(Object object) throws HibernateException {
        return _nativeSession.save(object);
    } // public Serializable save(Object object) throws HibernateException

    public void save(Object object, Serializable id)
        throws HibernateException {
        _nativeSession.save(object, id);
    } // public void save(Object object, Serializable id) throws HibernateException

    public void saveOrUpdate(Object object) throws HibernateException {
        _nativeSession.saveOrUpdate(object);
    } // public void saveOrUpdate(Object object) throws HibernateException

    public void update(Object object) throws HibernateException {
        _nativeSession.update(object);
    } // public void update(Object object) throws HibernateException

    public void update(Object object, Serializable id)
        throws HibernateException {
        _nativeSession.update(object, id);
    } // public void update(Object object, Serializable id) throws HibernateException

    public Object saveOrUpdateCopy(Object object) throws HibernateException {
        return _nativeSession.saveOrUpdateCopy(object);
    } // public Object saveOrUpdateCopy(Object object) throws HibernateException

    public Object saveOrUpdateCopy(Object object, Serializable id)
        throws HibernateException {
        return _nativeSession.saveOrUpdateCopy(object, id);
    } // public Object saveOrUpdateCopy(Object object, Serializable id) throws HibernateException

    public void delete(Object object) throws HibernateException {
        _nativeSession.delete(object);
    } // public void delete(Object object) throws HibernateException

    public List find(String query) throws HibernateException {
        return _nativeSession.find(query);
    } // public List find(String query) throws HibernateException

    public List find(String query, Object value, Type type)
        throws HibernateException {
        return _nativeSession.find(query, value, type);
    } // public List find(String query, Object value, Type type) throws HibernateException

    public List find(String query, Object[] values, Type[] types)
        throws HibernateException {
        return _nativeSession.find(query, values, types);
    } // public List find(String query, Object[] values, Type[] types) throws HibernateException

    public Iterator iterate(String query) throws HibernateException {
        return _nativeSession.iterate(query);
    } // public Iterator iterate(String query) throws HibernateException

    public Iterator iterate(String query, Object value, Type type)
        throws HibernateException {
        return _nativeSession.iterate(query, value, type);
    } // public Iterator iterate(String query, Object value, Type type) throws HibernateException

    public Iterator iterate(String query, Object[] values, Type[] types)
        throws HibernateException {
        return _nativeSession.iterate(query, values, types);
    } // public Iterator iterate(String query, Object[] values, Type[] types) throws HibernateException

    public Collection filter(Object collection, String filter)
        throws HibernateException {
        return _nativeSession.filter(collection, filter);
    } // public Collection fi lter(Object collection, String filter) throws HibernateException

    public Collection filter(
        Object collection,
        String filter,
        Object value,
        Type type)
        throws HibernateException {
        return _nativeSession.filter(collection, filter, value, type);
    } // public Collection filter(Object collection, String filter, Object value, Type type) throws HibernateException

    public Collection filter(
        Object collection,
        String filter,
        Object[] values,
        Type[] types)
        throws HibernateException {
        return _nativeSession.filter(collection, filter, values, types);
    } // public Collection filter(Object collection, String filter, Object[] values, Type[] types) throws HibernateException

    public int delete(String query) throws HibernateException {
        return _nativeSession.delete(query);
    } // public int delete(String query) throws HibernateException

    public int delete(String query, Object value, Type type)
        throws HibernateException {
        return _nativeSession.delete(query, value, type);
    } // public int delete(String query, Object value, Type type) throws HibernateException

    public int delete(String query, Object[] values, Type[] types)
        throws HibernateException {
        return _nativeSession.delete(query, values, types);
    } // public int delete(String query, Object[] values, Type[] types) throws HibernateException

    public void lock(Object object, LockMode lockMode)
        throws HibernateException {
        _nativeSession.lock(object, lockMode);
    } // public void lock(Object object, LockMode lockMode) throws HibernateException

    public void refresh(Object object) throws HibernateException {
        _nativeSession.refresh(object);
    } // public void refresh(Object object) throws HibernateException

    public void refresh(Object object, LockMode lockMode)
        throws HibernateException {
        _nativeSession.refresh(object, lockMode);
    } // public void refresh(Object object, LockMode lockMode) throws HibernateException

    public LockMode getCurrentLockMode(Object object)
        throws HibernateException {
        return _nativeSession.getCurrentLockMode(object);
    } // public LockMode getCurrentLockMode(Object object) throws HibernateException

    public Transaction beginTransaction() throws HibernateException {
        return _nativeSession.beginTransaction();
    } // public Transaction beginTransaction() throws HibernateException

    public Criteria createCriteria(Class persistentClass) {
        return _nativeSession.createCriteria(persistentClass);
    } // public Criteria createCriteria(Class persistentClass)

    public Query createQuery(String queryString) throws HibernateException {
        return _nativeSession.createQuery(queryString);
    } // public Query createQuery(String queryString) throws HibernateException

    public Query createFilter(Object collection, String queryString)
        throws HibernateException {
        return _nativeSession.createFilter(collection, queryString);
    } // public Query createFilter(Object collection, String queryString) throws HibernateException

    public Query getNamedQuery(String queryName) throws HibernateException {
        return _nativeSession.getNamedQuery(queryName);
    } // public Query getNamedQuery(String queryName) throws HibernateException

    public Query createSQLQuery(
        String sql,
        String returnAlias,
        Class returnClass) {
        return _nativeSession.createSQLQuery(sql, returnAlias, returnClass);
    } // public Query createSQLQuery(String sql, String returnAlias, Class returnClass)

    public Query createSQLQuery(
        String sql,
        String[] returnAliases,
        Class[] returnClasses) {
        return _nativeSession.createSQLQuery(sql, returnAliases, returnClasses);
    } // public Query createSQLQuery(String sql, String[] returnAliases, Class[] returnClasses)

    public void clear() {
        _nativeSession.clear();
    } // public void clear()

    public Object get(Class clazz, Serializable id) throws HibernateException {
        return _nativeSession.get(clazz, id);
    } // public Object get(Class clazz, Serializable id) throws HibernateException

    public Object get(Class clazz, Serializable id, LockMode lockMode)
        throws HibernateException {
        return _nativeSession.get(clazz, id, lockMode);
    } // public Object get(Class clazz, Serializable id, LockMode lockMode) throws HibernateException

    private boolean isRequestDistributed() {
        boolean distributedBoolean = false;
        RequestContainer requestContainer =
            RequestContainer.getRequestContainer();
        if (requestContainer != null)
            distributedBoolean = requestContainer.isRequestDistributed();
        return distributedBoolean;
    } // private boolean isRequestDistributed()
} // public abstract class AbstractHibernateSession implements Session
