/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess.pool;

import java.sql.Connection;
import java.sql.SQLException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import it.eng.spago.base.Constants;
import it.eng.spago.dbaccess.ConnectionPoolDescriptor;
import it.eng.spago.dbaccess.ConnectionPoolParameter;
import it.eng.spago.tracing.TracerSingleton;

/**
 * @author zoppello
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class AppServerManagedConnectionPool implements ConnectionPoolInterface {
    private ConnectionPoolDescriptor _connectionPoolDescriptor = null;

    public AppServerManagedConnectionPool(ConnectionPoolDescriptor connectionPoolDescriptor) {
        _connectionPoolDescriptor = connectionPoolDescriptor;
    }

	/**
	 * @see it.eng.spago.dbaccess.pool.ConnectionPoolInterface#getConnection()
	 */
    public Connection getConnection() throws SQLException {
        try {
            String jndiName = (String)_connectionPoolDescriptor.getConnectionPoolParameter("jndiName").getValue();
            
            // Utente e password non sono dati obbligatori in quanto possono essere configurati
            // direttamente nell'application server
			ConnectionPoolParameter userParam = _connectionPoolDescriptor.getConnectionPoolParameter("user");
            String user = (userParam != null) ? (String)userParam.getValue() : null;
            
			ConnectionPoolParameter passwordParam = _connectionPoolDescriptor.getConnectionPoolParameter("userPassword");
            String userPassword = (passwordParam != null) ? (String)passwordParam.getValue() : null;
            
            Context ctx = new InitialContext();
            DataSource ds = (DataSource)ctx.lookup(jndiName);
            return (user != null) ? ds.getConnection(user, userPassword) : ds.getConnection();
        }
        catch (NamingException ne) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "AppServerManagedConnectionPool::getConnection:", ne);
            throw new SQLException("Naming Exception");
        }
    }

	/**
	 * @see it.eng.spago.dbaccess.pool.ConnectionPoolInterface#release()
	 */
    public void release() throws SQLException {
    }
}
