/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess.pool;
import it.eng.spago.base.Constants;
import it.eng.spago.dbaccess.Configurator;
import it.eng.spago.dbaccess.ConnectionPoolDescriptor;
import it.eng.spago.dbaccess.ConnectionPoolParameter;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;
/**
 * @author Bernabei Angelo
 * @version 1.0
 */
public class DecriptAlgorithmFactory {
	private static ConnectionPoolDescriptor infoConnectionPool= null;
	public DecriptAlgorithmFactory() {
		super();
	}
	/**
	 * Crea il provider sulla base del pool, il nome della classe  presente nel file di configurazione data_access
	 * @param poolName
	 * @return
	 */
	public static IDecriptAlgorithm create(String poolName) throws EMFInternalError {
		infoConnectionPool= Configurator.getInstance().getConnectionPoolDescriptor(poolName);
		if (infoConnectionPool != null) {
			ConnectionPoolParameter parameter=
				infoConnectionPool.getConnectionPoolParameter("algorithm");
			if (parameter == null)	return null;
			String className= parameter.getValue();
			IDecriptAlgorithm provider= null;
			try {
				provider= (IDecriptAlgorithm) Class.forName(className).newInstance();
			} catch (InstantiationException e) {
				TracerSingleton.log(
					Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"PasswordProviderFactory::create",
					e);
				throw new EMFInternalError(
					EMFErrorSeverity.ERROR,
					"PasswordProviderFactory::create::InstantiationException" + e.getMessage());
			} catch (IllegalAccessException e) {
				TracerSingleton.log(
					Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"PasswordProviderFactory::create",
					e);
				throw new EMFInternalError(
					EMFErrorSeverity.ERROR,
					"PasswordProviderFactory::create::IllegalAccessException" + e.getMessage());
			} catch (ClassNotFoundException e) {
				TracerSingleton.log(
					Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"PasswordProviderFactory::create",
					e);
				throw new EMFInternalError(
					EMFErrorSeverity.ERROR,
					"PasswordProviderFactory::create::ClassNotFoundException" + e.getMessage());
			}
			return provider;
		} else
			return null;
	}
}
