/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
/* Generated by Together */

package it.eng.spago.dbaccess.sql;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;

/**
* Questa Classe Rappresenta un generico comando SQL nello standard SQL92
*
* @author Andrea Zoppello 
* @version 1.0
*/
public abstract class SQLCommand {
    protected DataConnection _dataConnection = null;
    protected Statement _stmt = null;
    protected String _commandString = null;
    protected List _inputParameter;
    protected TracerSingleton _tracer = null;
    protected boolean _closed = false;

    /**
    * Costruttore
    * @param <B>DataConnection</B> dataConnection - la connessione al db
    * @param <B>String</B>  commandString - La stringa contenente il comando sql
    */
    public SQLCommand(DataConnection dataConnection, String commandString) {
        _dataConnection = dataConnection;
        _commandString = commandString;
    } //public SQLCommand(DataConnection dataConnection, String commandString)

    public DataConnection getDataConnection() {
        return _dataConnection;
    } //public DataConnection getDataConnection()

    /**
    * Questo metodo server per eseguire il comnado senza parametri
    * @return un oggetto di tipo  <B>DataResult</B> contente il risultato dell'esecuzione del comando
    * @throws <B>EMFInternalError</B> - Se si verifica qualche problema durante l'esecuzione del comando
    */
    public abstract DataResult execute() throws EMFInternalError;

    /**
    * Questo metodo serve per eseguire il comando con dei parametri di input
    * @param <B>List</B> inputParameters - una lista di  <B>DataField</B> object
    * @return un oggetto di tipo  <B>DataResult</B> contente il risultato dell'esecuzione del comando
    * @throws <B>EMFInternalError</B> - Se si verifica qualche problema durante l'esecuzione del comando
    */
    public abstract DataResult execute(List inputParameters)
        throws EMFInternalError;

    /**
    * Questo metodo permette di chiudere l'oggetto SQLCommand
    * @throws <B>EMFInternalError</B> - Se qualche errore si verifica
    */
    public void close() throws EMFInternalError {
        closeInternal();
        _closed = true;
    } //public void close() throws EMFInternalError

    protected void closeInternal() throws EMFInternalError {
        try {
            if ((_stmt != null) && (!_closed)) {
                _stmt.close();
            }
            return;
        } // try
        catch (SQLException sqle) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "SQLCommand::closeInternal:",
                sqle);
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "SQLCommand :: close() :: " + sqle.getMessage());
        } // catch (SQLException sqle)
    } //protected void closeInternal() throws EMFInternalError

    protected Connection getInternalConnection() {
        return _dataConnection.getInternalConnection();
    } //protected Connection getInternalConnection()

    /**
    * Questo metodo verifica che il lo stato dell'oggetto comando.
    * @throws <B>EMFInternalError</B> - Se  l'istanza comando risulta chiusa.
    */
    public void validate() throws EMFInternalError {
        if (_closed)
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "SQLCommand :: validate() :: Cannot Use SQL Command The Command is already closed");
        return;
    } //public void validate() throws EMFInternalError

} //end Class SQLCommand
