/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
/* Generated by Together */

package it.eng.spago.dbaccess.sql.result;

import java.util.List;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;

/**
* Questa classe rappresenta un InformationDataResult, cio un DataResult che contiene informazioni quali:
*
* <li> il numero di righe coinvolte nell'esecuzione del comando
* <li> un messaggio di esecuzione
* <li> lo stato di correttezza o meno dell'esecuzione del comando
*
* @author Andrea Zoppello 
* @version 1.0
*/
public class InformationDataResult implements DataResultInterface {
    private String _executionMessage;
    private boolean _correctExecution;
    private int _affectedRows;
    //private SQLCommand _sqlCommand = null;
    public static boolean CORRECT_EXECUTION = true;
    public static boolean ERROR_EXECUTION = false;
    public static final String TAG = "INFORMATION_DATA_RESULT";

	/**
	* Costruttore
	* @param <B>SQLCommand</B> command - Il comando che ha generato come risultato l'oggetto InformationDataResult che verr creato da questa funzione
	* @param <B>List</B> inputParameters - La lista dei parametri di input del comando che ha generato come risultato l'oggetto InformationDataResult che verr creato da questa funzione
	* @param boolean correctExecution - true se l'esecuzione del comando  avvenuta correttamente
	* @param int int affectedRows - il numero di righe coinvolte nll'esecuzione del comando
	* @param String executionMessage - eventuale messaggio dell'InformationDataResult da creare
	* @throws <B>EMFInternalError</B> - Se qualche errore si verifica nella creazione dell'oggetto fi tipo InformationDataResult
	*/
    public InformationDataResult(SQLCommand sqlCommand, List inputParameters, boolean correctExecution, int affectedRows,
        String executionMessage) {
            //_sqlCommand = sqlCommand;
            _affectedRows = affectedRows;
            _executionMessage = executionMessage;
            _correctExecution = correctExecution;
    } //public InformationDataResult(SQLCommand sqlCommand, List
    // inputParameters,  boolean correctExecution, int affectedRows, String executionMessage)

	/**
	* Metodo getter per il parametro _executionMessage
	* @return un oggetto di tipo <B>String</B> con il valore di _executionMessage
	*/
    public String getExecutionMessage() {
        return _executionMessage;
    } //public String getExecutionMessage()

	/**
	* Metodo getter per il parametro _correctExecution
	* @return true se l'esecuzione  avvenuta in maniera corretta
	*/
    public boolean correctExecution() {
        return _correctExecution;
    } //public boolean correctExecution()

	/**
	* Metodo getter per il parametro _affectedRows
	* @return  il numero di righe coinvolte nell'esecuzione del comando
	*/
    public int getAffectedRows() {
        return _affectedRows;
    } //public int getAffectedRows()

	/**
	* Metodo setter per il parametro connectionPoolFactory
	* @param int affectedRows - il nuemro di righe coinvolte nell'esecuzione di un comando
	*/
    public void setAffectedRows(int affectedRows) {
        _affectedRows = affectedRows;
    } //public void setAffectedRows(int affectedRows)

	/**
	* Ritorna come valore la costante return DataResultInterface.INFORMATION_DATA_RESULT;
	* @return DataResultInterface.INFORMATION_DATA_RESULT
	*/
    public String getDataResultType() {
        return DataResultInterface.INFORMATION_DATA_RESULT;
    } //public String getDataResultType()

	/**
	* Ritorna un oggetto sourcebean rappresentante l'oggetto
	* @return la rappresentazione dell'oggetto fi tipo <B>DataResultInterface</B> come <B>SourceBean</B>
	*/
    public SourceBean getSourceBean() throws EMFInternalError {
        try {
            SourceBean sourceBean = new SourceBean(InformationDataResult.TAG);
            sourceBean.setAttribute("CORRECT_EXECUTION", String.valueOf(_correctExecution));
            sourceBean.setAttribute("AFFECTED_ROWS", String.valueOf(this.getAffectedRows()));
            return sourceBean;
        }
        catch (SourceBeanException sbe) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR,
                "InformationDataResul  :: getSourceBean() ::  " + sbe.getMessage());
        }
    } //public SourceBean getSourceBean()
} //
