/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess.sql.result;

import java.sql.ResultSet;

import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.error.EMFInternalError;

/**
*  Questa interfaccia definisce il comportamento di un oggetto <code>ScrollableDataResult</code>.
*
* @see ScrollableDataResult
* @author Andrea Zoppello 
* @version 1.0
*/
public interface ScrollableDataResult extends DataResultInterface {
    public static final String ROWS_TAG = "ROWS";

    /**
    * Questo metodo viene usato per scorrere l'oggetto in maniera sequenziale.
    * Il metodo ritorna true se esistono altre righe rispetto alla posizione corrente in cui si
    *  posizionati
    * @return true se l'oggetto ha altre righe in avanti rispetto alla posizione corrente in cui si
    *  posizionati false altrimenti
    * @throws <B>EMFInternalError</B> - Se qualche errore si verifica
    */
    public boolean hasRows() throws EMFInternalError;

    /**
    * Questo metodo ritorna la riga corrente su cui il cursore  posizionato
    * @return <B>DataRow</B>  - l'oggetto rappresentante la riga del resultset su cui il cursore  posizionato
    * @throws <B>EMFInternalError</B> - Se qualche errore si verifica
    */
    public DataRow getDataRow() throws EMFInternalError;

    /**
    * Questo metodo ritorna la riga della posizione i-esima dell'oggetto scrollableDataResult
    * @param int position - il numero della riga che si vuole ottenere
    * @return <B>DataRow</B>  - l'oggetto rappresentante la riga del resultset su cui il cursore  posizionato
    * @throws <B>EMFInternalError</B> - Se qualche errore si verifica
    */
    public DataRow getDataRow(int position) throws EMFInternalError;

    public int getColumnCount();

    /**
    * Ritorna un vettore contenente i valori sql.Types delle colonne dell'oggetto ScrollableDataResult
    * @return int[] - il vettore contenente i valori sql.Types delle colonne dell'oggetto ScrollableDataResult
    */
    public int[] getColumnTypes();

    public String[] getColumnNames();

    /**
    * Ritorna il numero di righe dell'oggetto
    * @return int - il numero di righe dell'oggetto
    */
    public int getRowsNumber() throws EMFInternalError;

    /**
    * Ritorna sempre DataResultInterface.SCROLLABLE_DATA_RESULT
    * @return <B>String</B> - Ritorna sempre DataResultInterface.SCROLLABLE_DATA_RESULT
    */
    public String getDataResultType();

    /**
    * Questo metodo permette il posizionamento del cursore sulla riga i-esima dell'oggetto ScrollableDataResult
    * @param int position - il numero di riga sulla quale ci si vuole posizionare
    * @throws <B>EMFInternalError</B> - Se qualche errore si verifica
    */
    public void moveTo(int position) throws EMFInternalError;

    /**
    * Questo metodo permette di forzare l'aggiornamento dei dati sull'oggetto ScrollableDataResult
    * @throws <B>EMFInternalError</B> - Se qualche errore si verifica
    */
    public void refresh() throws EMFInternalError;

    /**
    * Questo metodo permette di chiudere l'oggetto ScrollableDataResult
    * @throws <B>EMFInternalError</B> - Se qualche errore si verifica
    */
    public void close() throws EMFInternalError;

    public ResultSet getResultSet();
} // public interface ScrollableDataResult extends DataResultInterface
