/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.message.MessageBundle;
/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 22-11-2004		  Butano			   Aggiunto il metodo getToken() 
* 										    
**/
/**
 * @author Boccalon
 */

public class TokenHandler {

	public static String REQUEST_TOKEN_KEY = "SPAGO_TOKEN";
	public static String TRANSACTION_TOKEN_KEY = "SPAGO_TRANSACTION_TOKEN";
	
	/**
	 * Method saveToken.
	 * @param sessionContainer
	 * @param serviceRequest
	 */
	public static void saveToken(SessionContainer sessionContainer, SourceBean serviceRequest) {
		sessionContainer.getPermanentContainer().setAttribute(TRANSACTION_TOKEN_KEY, generateToken(serviceRequest));
	}

	/**
	 * Method getToken.
	 * @param requestContainer
	 */
	public static String getToken(RequestContainer requestContainer) {
		return (String)requestContainer.getSessionContainer().getPermanentContainer().getAttribute(TRANSACTION_TOKEN_KEY);
	}
		
	/**
	 * Method isTokenValid.
	 * @param serviceRequest
	 * @throws EMFInternalError
	 */
	public static void isTokenValid(SessionContainer sessionContainer, SourceBean serviceRequest) throws EMFInternalError {
		
		String saved = (String)sessionContainer.getPermanentContainer().getAttribute(TRANSACTION_TOKEN_KEY);
		if (saved == null) {
			// Non c' il token: probabilmente  la prima invocazione di servizio
			// Simulo la situazione di token presente e valido
			return;
		}
		
		// Verifico se nella richiesta  presente il token
		String token = (String)serviceRequest.getAttribute(REQUEST_TOKEN_KEY);
		if (token == null) {
			// Non c' il token: il form non necessita di controllo
			return;
		}
		
		if (!saved.equals(token))
			throw new EMFInternalError(EMFErrorSeverity.WARNING, MessageBundle.getMessage("Navigazione non permessa"));
	}
	
	/**
	 * Method generateToken. Genera il token da memorizzare in sessione per evitare
	 * il problema del submit duplicato dei form.
	 * @return String Token da memorizzare in sessione.
	 */
	private static String generateToken(SourceBean serviceRequest) {
		return String.valueOf((new java.util.Date()).getTime()) + serviceRequest.hashCode();
	}
	

}
