/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.action;

import java.io.Serializable;

import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.init.InitializerIFace;

/**
 * La classe <code>AbstractAction</code> &egrave; la superclasse di tutte quelle actions che
 * non necessitano di operare esclusivamente nel canale HTTP  accedendo direttamente agli oggetti HttpServletRequest,
 * HttpServletResponse,ServletConfig.<p>
 * Questa classe mette a disposizione i metodi per recuperare le instanze delle seguenti classi :
 *  <blockquote><pre>
 *  <code>RequestContainer</code> :Il contenitore di oggetti legati ai parametri della richiesta di un servizio.
 *  <code>ResponseContainer</code> :Il contenitore di oggetti legati alla riposta ad una richiesta di un servizio.
 *  <em>EMFErrorHandler</code> :Il gestore degli errori.
 *  </blockquote></pre>
 * Nella configurazione XML dell'action  possibile scrivere uno stream XML(Config) che verr
 * passato all'istanza dell'action dopo la sua creazione.
 *
 * @author Luigi Bellio
 * @see it.eng.spago.base.RequestContainer
 * @see it.eng.spago.base.ResponseContainer
 * @see it.eng.spago.error.EMFErrorHandler
 */
public abstract class AbstractAction extends DefaultRequestContext implements InitializerIFace, ActionIFace, Serializable {
    private SourceBean _config = null;
    private String _action = null;

    public AbstractAction() {
        super();
        _config = null;
        _action = null;
    } // public AbstractAction()

    /**
     * Rende disponbile alla action lo stream XML di configurazione. Questo  a carico dell' Application Framework
     * e viene effettuato immediatamente dopo la creazione di un'istanza.
     * <p>
     * @param name nome del <code>SourceBean</code>
     */
    public void init(SourceBean config) {
        _config = config;
    } // public void init(SourceBean config)

    /**
     * Permette di recuperare lo stream XML di configurazione.
     * <p>
     * @return <code>SourceBean</code> un'istanza  contenente lo stream XML di configurazione.
     * @see SourceBean
     */
    public SourceBean getConfig() {
        return _config;
    } // public SourceBean getConfig()

    /**
     * Permette di recuperare il nome logico dell'action.
	 * @deprecated Use getActionName instead
     * @return <code>String</code> il nome logico dell'action.
     */
    public String getAction() {
        return _action;
    } // public String getAction()

    /**
     * Permette di recuperare il nome logico dell'action.
     * @return <code>String</code> il nome logico dell'action.
     */
    public String getActionName() {
        return _action;
    } // public String getAction()

    /**
     * Rende disponbile alla action il proprio nome logico . Questo  a carico dell' Application Framework.
	 * @deprecated Use setActionName instead
     * @param action nome logico della action.
     */
    public void setAction(String action) {
        _action = action;
    } // public void setAction(String action)

    /**
     * Rende disponbile alla action il proprio nome logico . Questo  a carico dell' Application Framework.
     * @param action nome logico della action.
     */
    public void setActionName(String action) {
        _action = action;
    } // public void setAction(String action)
} // public abstract class AbstractAction extends DefaultRequestContext
// implements InitializerIFace, ActionIFace, Serializable
