/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.action;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.coordinator.CoordinatorIFace;
import it.eng.spago.dispatching.coordinator.DispatcherIFace;
import it.eng.spago.dispatching.service.RequestContextIFace;

public class ActionDispatcher implements DispatcherIFace {
    public ActionDispatcher() {
    } // public ActionDispatcher()

    public boolean acceptsURL(RequestContextIFace requestContext) {
        SourceBean serviceRequest = requestContext.getRequestContainer().getServiceRequest();
        return serviceRequest.containsAttribute(Constants.ACTION_NAME);
    } // public boolean acceptsURL(RequestContextIFace requestContext)

    public String getBusinessType(RequestContextIFace requestContext) {
        return "ACTION";
    } // public String getBusinessType(RequestContextIFace requestContext)

    public String getBusinessName(RequestContextIFace requestContext) {
        SourceBean serviceRequest = requestContext.getRequestContainer().getServiceRequest();
        String actionName = (String)serviceRequest.getAttribute(Constants.ACTION_NAME);
        return actionName.toUpperCase();
    } // public String getBusinessName(RequestContextIFace requestContext)

    public String getPublisherName(RequestContextIFace requestContext) {
        ConfigSingleton configure = ConfigSingleton.getInstance();
        SourceBean actionBean = (SourceBean)configure.getFilteredSourceBeanAttribute(Constants.ACTION_PATH, "NAME", getBusinessName(requestContext));
        if (actionBean == null) {
            return null;
        }
        String publisherName = (String)actionBean.getAttribute(Constants.PUBLISHER_NAME);
        return publisherName;
    } // public String getBusinessName(RequestContextIFace requestContext)

    public CoordinatorIFace getCoordinator(RequestContextIFace requestContext) {
        return new ActionCoordinator(getBusinessType(requestContext), getBusinessName(requestContext), getPublisherName(requestContext));
    } // public CoordinatorIFace getCoordinator(RequestContextIFace requestContext)
} // public class ActionDispatcher implements DispatcherIFace
