/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.coordinator;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.tracing.TracerSingleton;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;

public class DispatcherManager {
    private static ArrayList _dispatchers = new ArrayList();

    private DispatcherManager() {
    } // private DispatcherManager()

    public static void registerDispatcher(DispatcherIFace dispatcher) {
        if (dispatcher == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "DispatcherManager::registerDispatcher: dispatcher nullo");
            return;
        } // if (dispatcher == null)
        _dispatchers.add(dispatcher);
    } // public static void registerDispatcher(DispatcherIFace dispatcher)

    public static CoordinatorIFace getCoordinator(RequestContextIFace requestContext) {
        RequestContainer requestContainer = requestContext.getRequestContainer();
        for (int i = 0; i < _dispatchers.size(); i++) {
            DispatcherIFace dispatcher = (DispatcherIFace)_dispatchers.get(i);
            if (dispatcher.acceptsURL(requestContext)) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "DispatcherManager::getCoordinator: dispatcher trovato [" + dispatcher.getClass().getName() + "]");
                String distributeCoordinatorClass = null;
                String businessType = null;
                String businessName = null;
                
                // The method requestContainer.isRequestDistributed() can return true if the
                // distributed coordinator is already executing: a second call to this
                // method (getCoordinator) is necessary to retrieve the "classic" coordinator
                if (!requestContainer.isRequestDistributed()) {
                    businessType = dispatcher.getBusinessType(requestContext);
                    businessName = dispatcher.getBusinessName(requestContext);
                    ConfigSingleton configSingleton = ConfigSingleton.getInstance();
                    List distributions = configSingleton.getFilteredSourceBeanAttributeAsList("DISTRIBUTIONS.DISTRIBUTION",
                        "BUSINESS_NAME", businessName);
                    
                        for (int j = 0; j < distributions.size(); j++) {
                            SourceBean distribution = (SourceBean)distributions.get(j);
                            String localBusinessType = (String)distribution.getAttribute("BUSINESS_TYPE");
                            if ((localBusinessType != null) && (localBusinessType.equalsIgnoreCase(businessType))) {
                                distributeCoordinatorClass = (String)distribution.getAttribute("DISTRIBUTE_COORDINATOR_CLASS");
                                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                                    "DispatcherManager::getCoordinator: business distribuito [" +
                                    distributeCoordinatorClass + "]");
                                break;
                            } // if ((localBusinessType != null) &&
                            // (localBusinessType.equalsIgnoreCase(businessType)))
                        } // for (int i = 0; i < distributions(); i++)
                } // if (!requestContainer.isRequestDistributed())
                CoordinatorIFace coordinator = null;
                if (distributeCoordinatorClass == null) {
                    // This is not a distributed request: use "classic" coordinators
                    coordinator = dispatcher.getCoordinator(requestContext);
                }
                else {
                    // Use a distributed coordinator
                    try {
                        Class distributeCoordinator = Class.forName(distributeCoordinatorClass);
                        Class[] formalParameters = new Class[3];
                        formalParameters[0] = String.class;
                        formalParameters[1] = String.class;
                        formalParameters[2] = String.class;
                        Constructor distributeCoordinatorConstructor = distributeCoordinator.getConstructor(formalParameters);
                        Object[] actualParameters = new Object[3];
                        actualParameters[0] = businessType;
                        actualParameters[1] = businessName;
                        // Publisher name not necessary in this context
                        actualParameters[2] = null;
                        coordinator = (CoordinatorIFace)distributeCoordinatorConstructor.newInstance(actualParameters);
                    } // try
                    catch (Exception ex) {
                        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                            "DispatcherManager::getCoordinator: classe del coordinator distribuito non valida", ex);
                        return null;
                    } // catch (Exception ex) try
                } // if (distributeCoordinatorClass == null) else
                return coordinator;
            } // if (dispatcher.acceptsURL(request))
        } // for (int i = 0; i < _dispatchers.size(); i++)
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
            "DispatcherManager::getCoordinator: dispatcher non trovato");
        return null;
    } // public static CoordinatorIFace getCoordinator(RequestContextIFace requestContext)

    public static void deregisterDispatcher(DispatcherIFace dispatcher) {
        if (dispatcher == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "DispatcherManager::deregisterDispatcher: dispatcher nullo");
            return;
        } // if (dispatcher == null)
        _dispatchers.remove(dispatcher);
    } // public static void deregisterDispatcher(DispatcherIFace dispatcher)
} // public class DispatcherManager
