/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.module;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.event.EventNotifierFactory;
import it.eng.spago.event.ModuleEndEvent;
import it.eng.spago.event.ModuleExceptionEvent;
import it.eng.spago.event.ModuleStartEvent;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spago.security.SecurityConfigurationSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           SourceBean ora restituisce ArrayList
* 05-12-2005          Zoppello             Modifica per lanciare gli eventi PageStartEvent, e PageEndEvent, PageExceptionEvent								    
**/

/**
 * @author 
 */
public class DefaultPage extends AbstractPage {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private String _pageName = null;
    private HashMap _modulesInstancesToKeep = null;
    private HashMap _modulesResponsesToKeep = null;
    private IEngUserProfile _userProfile = null;

    public DefaultPage() {
        super();
        _pageName = null;
        _modulesInstancesToKeep = null;
        _modulesResponsesToKeep = null;
        _userProfile = null;
    } // public DefaultPage()

    public void init(SourceBean config) {
        super.init(config);
        _pageName = (String) config.getAttribute("NAME");
        _modulesInstancesToKeep = new HashMap();
        _modulesResponsesToKeep = new HashMap();
    } // public void init(SourceBean config)

    protected HashMap initModules() throws Exception {
        HashMap modulesInstances = new HashMap();
        SourceBean serviceResponse =
            getResponseContainer().getServiceResponse();
        SourceBean sharedData = null;
        try {
            sharedData = new SourceBean("SHARED_DATA");
        } // try
        catch (SourceBeanException sbe) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DefaultPage::initModules: new SourceBean(\"SHARED_DATA\")",
                sbe);
        } // catch(SourceBeanException sbe)
        ArrayList moduleConfigs =
            (ArrayList)getConfig().getAttributeAsList("MODULES.MODULE");
        for (int i = 0; i < moduleConfigs.size(); i++) {
            SourceBean moduleConfig = (SourceBean) moduleConfigs.get(i);
            String moduleName = ((String) moduleConfig.getAttribute("NAME")).toUpperCase();
            String keepInstanceStr =
                (String) moduleConfig.getAttribute("KEEP_INSTANCE");
            boolean keepInstance =
                ((keepInstanceStr != null)
                    && keepInstanceStr.equalsIgnoreCase("TRUE"));
            ModuleIFace module = null;
            if (_modulesInstancesToKeep
                .containsKey(moduleName)) {
                module =
                    (ModuleIFace) _modulesInstancesToKeep.get(
                        moduleName);
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.DEBUG,
                    "DefaultPage::initModules: istanza module ["
                        + moduleName
                        + "] trovata");
            } // if (_modulesInstancesToKeep.containsKey(moduleName))
            else {
                try {
                    module = ModuleFactory.getModule(moduleName);
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.DEBUG,
                        "DefaultPage::initModules: istanza module ["
                            + moduleName
                            + "] creata");
                } // try
                catch (ModuleException pte) {
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.CRITICAL,
                        "DefaultPage::initModules:",
                        pte);
                } // catch (ModuleException pte)
            } // if (_modulesInstancesToKeep.containsKey(moduleName)) else
            if (module != null) {
                ((RequestContextIFace) module).setRequestContext(this);
                module.setPage(_pageName);
                module.setSharedData(sharedData);
                modulesInstances.put(moduleName, module);
                if (keepInstance)
                    _modulesInstancesToKeep.put(
                        moduleName,
                        module);
            } // if (module != null)
            if (_modulesResponsesToKeep
                .containsKey(moduleName)) {
                serviceResponse.updAttribute(
                    (SourceBean) _modulesResponsesToKeep.get(
                        moduleName));
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.DEBUG,
                    "DefaultPage::initModules: risposta module ["
                        + moduleName
                        + "] trovata");
            } // if (_modulesResponsesToKeep.containsKey(moduleName))
            else
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.DEBUG,
                    "DefaultPage::initModules: risposta module ["
                        + moduleName
                        + "] non trovata");
        } // for (int i = 0; i < modules.size(); i++)
        _userProfile =
            (IEngUserProfile) getRequestContainer()
                .getSessionContainer()
                .getPermanentContainer()
                .getAttribute(IEngUserProfile.ENG_USER_PROFILE);
		if (_userProfile instanceof RequestContextIFace) {       
			((RequestContextIFace)_userProfile).setRequestContext(this); 
		}		                
        return modulesInstances;
    } // protected void initModules() throws Exception

    protected void releaseModules(HashMap modulesInstances) {
        if (modulesInstances == null)
            return;
        ArrayList modulesArray = new ArrayList(modulesInstances.values());
        for (int i = 0; i < modulesArray.size(); i++) {
            ModuleIFace module = (ModuleIFace) modulesArray.get(i);
            ((RequestContextIFace) module).setRequestContext(null);
            module.setSharedData(null);
        } // for (int i = 0; i < modulesArray.size(); i++)
    } // protected void releaseModules(HashMap modulesInstances)

    public void service(SourceBean serviceRequest, SourceBean serviceResponse)
        throws Exception {
        HashMap modulesInstances = null;
        try {
            modulesInstances = initModules();
            nextStep(modulesInstances, null);
        } // try
        catch (Exception ex) {
            throw ex;
        } // catch (Exception ex)
        finally {
            releaseModules(modulesInstances);
        } // finally
    } // public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception
    
    /**
     * Build the list of target modules to call, starting from the module sourceName.
     * The result is a list containing the requests (SourceBean) to use to call the target modules.
     * In each request there is an attribute that contains the target module name.
     * The method should check all conditions of the dependencies involved.
     * @param sourceName The name of the starting module.
     * @return List of requests used to call the target modules.
     */
    public List getTargetRequests(
            final String sourceName) {
        return (new ModuleDependencies()).getTargetRequests(
                this,
                sourceName,
                getConfig());
    }

    protected void nextStep(HashMap modulesInstances, String sourceName)
        throws Exception {
        List targetRequests =
                getTargetRequests(sourceName);
        SourceBean serviceResponse =
            getResponseContainer().getServiceResponse();
        for (int i = 0; i < targetRequests.size(); i++) {
            SourceBean targetRequest = (SourceBean) targetRequests.get(i);
            String targetName =
                ((String) targetRequest.getAttribute("AF_MODULE_NAME")).toUpperCase();
            if (!modulesInstances.containsKey(targetName)) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "DefaultPage::nextStep: module targetName ["
                        + targetName
                        + "] non censito nella page");
                continue;
            } // if (!(_modules.containsKey(targetName)))
            boolean isModuleAuthorized = true;
            if (SecurityConfigurationSingleton
                .getInstance()
                .isPageModuleChecked(_pageName, targetName)) {
                if (_userProfile == null) {
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.WARNING,
                        "DefaultPage::nextStep: _userProfile nullo !");
                    isModuleAuthorized = false;
                } // if (_userProfile == null)
                else {
                    try {
                        isModuleAuthorized =
                            _userProfile.isAbleToExecuteModuleInPage(
                                _pageName,
                                targetName);
                    } // try
                    catch (EMFInternalError emfie) {
                        TracerSingleton.log(
                            Constants.NOME_MODULO,
                            TracerSingleton.CRITICAL,
                            "DefaultPage::nextStep:: _userProfile.isAbleToExecuteModuleInPage("
                                + _pageName
                                + ", "
                                + targetName
                                + ")");
                        isModuleAuthorized = false;
                    } // catch (EMFInternalError emfie)
                } // if (_userProfile == null) else
            } // if (SecurityConfigurationSingleton.getInstance().isPageModuleChecked(getPage(),
            // targetName))
            SourceBean targetResponse = new SourceBean(targetName);
            if (!isModuleAuthorized)
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "DefaultPage::nextStep: autorizzazione esecuzione module ["
                        + targetName
                        + "] nella page ["
                        + _pageName
                        + "] negata !");
            else {
                ModuleIFace module =
                    (ModuleIFace) modulesInstances.get(
                        targetName);
                SourceBean serviceRequest =
                    getRequestContainer().getServiceRequest();
                Monitor moduleMonitor =
                    MonitorFactory.start(
                        "model.module." + targetName.toLowerCase());
                try {
                    getRequestContainer().setServiceRequest(targetRequest);
                    
                    EventNotifierFactory.getEventNotifier().notifyEvent(new ModuleStartEvent(module), this);
                    module.service(targetRequest, targetResponse);
                    EventNotifierFactory.getEventNotifier().notifyEvent(new ModuleEndEvent(module), this);
                } // try
                catch (Exception ex) {
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.CRITICAL,
                        "DefaultPage::nextStep:: errore nel module ["
                            + targetName
                            + "]");
                    EventNotifierFactory.getEventNotifier().notifyEvent(new ModuleExceptionEvent(module,ex), this);
                    throw ex;
                } // catch (Exception ex)
                finally {
                    getRequestContainer().setServiceRequest(serviceRequest);
                    moduleMonitor.stop();
                } // finally
            } // if (!isModuleAuthorized) else
            serviceResponse.updAttribute(targetResponse);
            SourceBean moduleConfig =
                (SourceBean) getConfig().getFilteredSourceBeanAttribute(
                    "MODULES.MODULE",
                    "NAME",
                    targetName);
            String keepResponseStr =
                (String) moduleConfig.getAttribute("KEEP_RESPONSE");
            boolean keepResponse =
                ((keepResponseStr != null)
                    && keepResponseStr.equalsIgnoreCase("TRUE"));
            if (keepResponse) {
                _modulesResponsesToKeep.put(
                    targetName,
                    targetResponse);
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.DEBUG,
                    "DefaultPage::nextStep: risposta module ["
                        + targetName
                        + "] archiviata");
            } // if (keepResponse)
            nextStep(modulesInstances, targetName);
        } // for (int i = 0; i < targetRequests.size(); i++)
    } // protected void nextStep(HashMap modulesInstances, String sourceName) throws Exception
} // public class DefaultPage extends AbstractPage
