/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.module;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.TokenHandler;
import it.eng.spago.dispatching.coordinator.AbstractCoordinator;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.event.EventNotifierFactory;
import it.eng.spago.event.IEventNotifier;
import it.eng.spago.event.PageEndEvent;
import it.eng.spago.event.PageExceptionEvent;
import it.eng.spago.event.PageStartEvent;
import it.eng.spago.event.ValidationFailedEvent;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.validation.coordinator.ValidationCoordinator;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 22-11-2004		  Butano           Gestione TOKEN per il PREVENT_RESUBMIT 
* 13-12-2004              Butano           Chiamata alla classe Validation per la validazioen lato server
* 05-12-2005          Zoppello             Modifica per lanciare gli eventi ModuleStartEvent, e ModuleEndEvent, ModuleExceptionEvent 										    
**/

/**
 * @author 
 */
public class ModuleCoordinator extends AbstractCoordinator {

    private static final long serialVersionUID = 1L;

    public ModuleCoordinator(final String businessType, final String businessName, final String publisherName) {
        super(businessType, businessName, publisherName);
    } // public ModuleCoordinator(String businessType, String businessName)

    public void service(SourceBean serviceRequest, SourceBean serviceResponse)
        throws Exception {
        Monitor monitor = MonitorFactory.start("controller.coordinator.page");
		IEventNotifier eventNotifier = EventNotifierFactory.getEventNotifier();
		try {
			SessionContainer sessionContainer = getRequestContainer().getSessionContainer();

			// Verifico se  arrivato il token e se  valido
			TokenHandler.isTokenValid(sessionContainer, serviceRequest); 
			// Genero e salvo il nuovo token in sessione
			Object tokenFlag = serviceRequest.getAttribute("SPAGO_GENERATETOKEN");
			if ((tokenFlag == null) ||
				!tokenFlag.toString().equalsIgnoreCase("FALSE"))
			{
				TokenHandler.saveToken(sessionContainer, serviceRequest);
			}            		
			
	        PageIFace page =
	            PageFactory.getPage(getRequestContainer(), getBusinessName());
	        saveCurrentServiceInformation();
	        synchronized (page) {
	            Monitor pageMonitor =
	                MonitorFactory.start(
	                    "model.page." + getBusinessName().toLowerCase());
	            try {
	                TracerSingleton.log(
	                    Constants.NOME_MODULO,
	                    TracerSingleton.DEBUG,
	                    "ModuleCoordinator::service: serviceRequest",
	                    serviceRequest);
	                ((RequestContextIFace) page).setRequestContext(this);
                    if (ValidationCoordinator.validate(getBusinessType(), getBusinessName(), this)) {
                        eventNotifier.notifyEvent(new PageStartEvent(page), this);
                    	
                    	page.service(serviceRequest, serviceResponse);
                    	
                    	eventNotifier.notifyEvent(new PageEndEvent(page), this);
                    } else {
                        eventNotifier.notifyEvent(
                                new ValidationFailedEvent(getBusinessName()),
                                this);
                    }
	            } // try
	            catch (Exception ex) {
	                TracerSingleton.log(
	                    Constants.NOME_MODULO,
	                    TracerSingleton.CRITICAL,
	                    "ModuleCoordinator::service: eccezione nella page ["
	                        + getBusinessName()
	                        + "]");
	                eventNotifier.notifyEvent(new PageExceptionEvent(page, ex), this);
	                throw ex;
	            } // catch (Exception ex)
	            finally {
	                ((RequestContextIFace) page).setRequestContext(null);
	                TracerSingleton.log(
	                    Constants.NOME_MODULO,
	                    TracerSingleton.DEBUG,
	                    "ModuleCoordinator::service: serviceResponse",
                        getResponseContainer().getServiceResponse());
	                pageMonitor.stop();
	            } // finally
	        } // synchronized (page)
		}
		finally {
            monitor.stop();
		}
    } // public void service(SourceBean request, SourceBean response)
} // public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception
