/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.module;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;

import java.util.ArrayList;
import java.util.List;

/**
 * DATE CONTRIBUTOR/DEVELOPER NOTE 13-12-2004 Butano SourceBean ora restituisce
 * ArrayList
 * 
 */

public class ModuleDependencies {
    public ModuleDependencies() {
        super();
    } // private ModuleDependencies()

    /**
     * Evaluates the conditions relative to a dependence. 
     * @param dependence The dependence to check.
     * @param requestContainer
     * @param responseContainer
     * @return true if the conditions are satisfied.
     */
    public boolean handleCondition(final SourceBean dependence,
                                   final RequestContextIFace requestContext) {
        List conditions = dependence.getAttributeAsList("CONDITIONS.PARAMETER");
        if (conditions.size() == 0)
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "ModuleDependencies::getTargetRequests: nessuna condizione trovata");
        boolean conditionVerified = true;
        for (int j = 0; j < conditions.size(); j++) {
            SourceBean condition = (SourceBean) conditions.get(j);
            String parameterValue = (String) condition.getAttribute("VALUE");
            String inParameterValue = null;
            Object parameterValueObject = ContextScooping.getScopedParameter(requestContext, condition);
            if (parameterValueObject != null)
                inParameterValue = parameterValueObject.toString();
            if (parameterValue.equalsIgnoreCase("AF_DEFINED")) {
                if (inParameterValue == null) {
                    conditionVerified = false;
                    break;
                } // if (inParameterValue == null)
                continue;
            } // if (parameterValue.equalsIgnoreCase("AF_DEFINED"))
            if (parameterValue.equalsIgnoreCase("AF_NOT_DEFINED")) {
                if (inParameterValue != null) {
                    conditionVerified = false;
                    break;
                } // if (inParameterValue != null)
                continue;
            } // if (parameterValue.equalsIgnoreCase("AF_NOT_DEFINED"))
            if (!(parameterValue.equalsIgnoreCase(inParameterValue))) {
                conditionVerified = false;
                break;
            } // if (!(parameterValue.equalsIgnoreCase(inParameterValue)))
        } // for (int j = 0; conditions.size(); j++)
        return conditionVerified;
    }

    /**
     * Evaluates the consequences relative to a dependence, and build the serviceRequest
     * for the target module, adding it to the targetRequests list.
     * @param targetRequests Resulting list. Should be already allocated.
     * @param dependence
     * @param requestContainer
     * @param responseContainer
     */
    public void handleConsequence(ArrayList targetRequests,
                                  final SourceBean dependence,
                                  final RequestContextIFace requestContext) {

        List consequences = dependence.getAttributeAsList("CONSEQUENCES.PARAMETER");
        if (consequences.size() == 0)
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "ModuleDependencies::getTargetRequests: nessuna conseguenza trovata");
        SourceBean targetRequest = (SourceBean) requestContext.getServiceRequest().cloneObject();
        String targetName = (String) dependence.getAttribute("TARGET");
        try {
            targetRequest.updAttribute("AF_MODULE_NAME", targetName);
        } // try
        catch (SourceBeanException sbe) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "ModuleDependencies::getTargetRequests: targetRequest.updAttribute(\"AF_MODULE_NAME\", "
                            + targetName + ")", sbe);
        } // catch (SourceBeanException sbe)
        for (int j = 0; j < consequences.size(); j++) {
            SourceBean consequence = (SourceBean) consequences.get(j);
            String parameterName = (String) consequence.getAttribute("NAME");
            String parameterScope = (String) consequence.getAttribute("SCOPE");
            String parameterType = (String) consequence.getAttribute("TYPE");
            String parameterValue = (String) consequence.getAttribute("VALUE");
            Object inParameterValue = null;
            if (parameterType.equalsIgnoreCase("ABSOLUTE")) {
                inParameterValue = parameterValue;
            }
            else {
                inParameterValue =
                    ContextScooping.getScopedParameter(
                        requestContext.getRequestContainer(),
                        requestContext.getResponseContainer(),
                        parameterValue,
                        parameterScope);
            }
            
            String parameterAppend = (String) consequence.getAttribute("APPEND");
            boolean append = ((parameterAppend != null) && parameterAppend.equalsIgnoreCase("true"));
            try {
                if (inParameterValue == null)
                    targetRequest.delAttribute(parameterName);
                else if (inParameterValue instanceof SourceBean)
                    if (append)
                        targetRequest.setAttribute((SourceBean) inParameterValue);
                    else
                        targetRequest.updAttribute((SourceBean) inParameterValue);
                else if (append)
                    targetRequest.setAttribute(parameterName, inParameterValue);
                else
                    targetRequest.updAttribute(parameterName, inParameterValue);
            } // try
            catch (SourceBeanException sbe) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                        "ModuleDependencies::getTargetRequests: targetRequest.updAttribute("
                                + parameterName + ", " + inParameterValue + ")", sbe);
            } // catch (SourceBeanException sbe)
        } // for (int j = 0; j < consequences.size(); j++)
        targetRequests.add(targetRequest);
    }

    public List getTargetRequests(final RequestContextIFace requestContext,
            final String sourceName,
            final SourceBean pageConfig) {
        ArrayList targetRequests = new ArrayList();

        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "ModuleDependencies::getTargetRequests: sourceName [" + sourceName + "]");

        String page = (String) requestContext.getServiceRequest().getAttribute("PAGE");
        if (pageConfig == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "ModuleDependencies::getTargetRequests: pageConfig [" + page + "] nullo");
            return targetRequests;
        } // if (pageConfig == null)

        if (sourceName != null) {
            SourceBean moduleConfig = (SourceBean) pageConfig.getFilteredSourceBeanAttribute(
                    "MODULES.MODULE", "NAME", sourceName);
            if (moduleConfig == null) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                        "ModuleDependencies::getTargetRequests: module [" + sourceName
                                + "] non censito nella pagina");
                return targetRequests;
            } // if (moduleConfig == null)
        } // if (module != null)

        List dependencies = pageConfig.getFilteredSourceBeanAttributeAsList(
                "DEPENDENCIES.DEPENDENCE", "SOURCE", (sourceName == null) ? page : sourceName);

        if (dependencies.size() == 0)
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "ModuleDependencies::getTargetRequests: nessuna dipendenza trovata");

        for (int i = 0; i < dependencies.size(); i++) {
            SourceBean dependence = (SourceBean) dependencies.get(i);

            boolean conditionVerified = handleCondition(dependence, requestContext);

            if (conditionVerified) {
                handleConsequence(targetRequests, dependence, requestContext);
            } // if (conditionVerified)
        } // for (int i = 0; i < dependencies.size(); i++)
        return targetRequests;
    } // public static Collection getTargetRequests(RequestContainer
    // requestContainer, ResponseContainer responseContainer, String
    // sourceName, SourceBean pageConfig)

} // public class ModuleDependencies
