/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.module;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;

public class ModuleFactory {
    private ModuleFactory() {
        super();
    } // private ModuleFactory()

    public static ModuleIFace getModule(String moduleName) throws ModuleException {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "ModuleFactory::getModule: moduleName [" + moduleName + "]");
        ConfigSingleton configure = ConfigSingleton.getInstance();
        SourceBean moduleBean = (SourceBean)configure.getFilteredSourceBeanAttribute("MODULES.MODULE", "NAME", moduleName);
        if (moduleBean == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "ModuleFactory::getModule: nome del module non valido !");
            throw new ModuleException("nome del module [" + moduleName + "] non valido");
        } // if (actionBean == null)
        String moduleClass = (String)moduleBean.getAttribute("CLASS");
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "ModuleFactory::getModule: moduleClass [" + moduleClass + "]");
        SourceBean moduleConfig = (SourceBean)moduleBean.getAttribute("CONFIG");
        if (moduleConfig == null)
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
                "ModuleFactory::getModule: config del module non censito");
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "ModuleFactory::getModule: moduleConfig", moduleConfig);
        ModuleIFace module = null;
        try {
            module = (ModuleIFace)Class.forName(moduleClass).newInstance();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "ModuleFactory::getModule: classe del module non valida", ex);
            throw new ModuleException("Classe del module [" + moduleName + "] non valida");
        } // catch (Exception ex) try
        module.setModule(moduleName);
        ((InitializerIFace)module).init(moduleConfig);
        return module;
    } // public static ModuleIFace getModule(String moduleName) throws ModuleException
} // public class ModuleFactory
