/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.module;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;

public class PageFactory {
    private PageFactory() {
        super();
    } // private PageFactory()

    public static PageIFace getPage(
        RequestContainer requestContainer,
        String pageName)
        throws ModuleException {
        pageName = pageName.toUpperCase();
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "PageFactory::getPage: pageName [" + pageName + "]");
        ConfigSingleton configure = ConfigSingleton.getInstance();
        SourceBean pageBean =
            (SourceBean) configure.getFilteredSourceBeanAttribute(
                Constants.PAGE_PATH,
                "NAME",
                pageName);
        if (pageBean == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "PageFactory::getPage: nome della page non valido !");
            throw new ModuleException(
                "Nome della page [" + pageName + "] non valido");
        } // if (pageBean == null)
        String pageClass = (String) pageBean.getAttribute("CLASS");
        if (pageClass == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "PageFactory::getPage: classe della page non censita, default [it.eng.spago.dispatching.module.DefaultPage]");
            pageClass = "it.eng.spago.dispatching.module.DefaultPage";
        } // if (pageScope == null)
        String pageScope = (String) pageBean.getAttribute("SCOPE");
        if (pageScope == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "PageFactory::getPage: scope della page non censito, default [REQUEST]");
            pageScope = "REQUEST";
        } // if (pageScope == null)
        if ((pageScope == null)
            || (!pageScope.equalsIgnoreCase("REQUEST")
                && !pageScope.equalsIgnoreCase("SESSION")
                && !pageScope.equalsIgnoreCase("APPLICATION"))) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "PageFactory::getPage: scope della page non valido ["
                    + pageScope
                    + "]");
            pageScope = "REQUEST";
        } // if ((pageScope == null) || ...
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "PageFactory::getPage: pageScope [" + pageScope + "]");
        PageIFace page = null;
        if (pageScope.equalsIgnoreCase("REQUEST")) {
            page = getInstance(pageClass, pageBean);
            return page;
        } // if (pageScope.equalsIgnoreCase("REQUEST"))
        else if (pageScope.equalsIgnoreCase("SESSION")) {
            SessionContainer sessionContainer =
                requestContainer.getSessionContainer().getPermanentContainer();
            page =
                (PageIFace) sessionContainer.getAttribute(
                    "AF_PAGE_" + pageName);
            if (page == null) {
                synchronized (sessionContainer) {
                    page =
                        (PageIFace) sessionContainer.getAttribute(
                            "AF_PAGE_" + pageName);
                    if (page == null) {
                        TracerSingleton.log(
                            Constants.NOME_MODULO,
                            TracerSingleton.DEBUG,
                            "PageFactory::getPage: page non trovata in SESSION !");
                        page = getInstance(pageClass, pageBean);
                        sessionContainer.setAttribute(
                            "AF_PAGE_" + pageName,
                            page);
                    } // if (page == null)
                } // synchronized (sessionContainer)
            } // if (page == null)
            else
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.DEBUG,
                    "PageFactory::getPage: page trovata in SESSION");
            return page;
        } // if (pageScope.equalsIgnoreCase("SESSION"))
        else {
            ApplicationContainer applicationContainer =
                ApplicationContainer.getInstance();
            page =
                (PageIFace) applicationContainer.getAttribute(
                    "AF_PAGE_" + pageName);
            if (page == null) {
                synchronized (applicationContainer) {
                    page =
                        (PageIFace) applicationContainer.getAttribute(
                            "AF_PAGE_" + pageName);
                    if (page == null) {
                        TracerSingleton.log(
                            Constants.NOME_MODULO,
                            TracerSingleton.DEBUG,
                            "PageFactory::getPage: page non trovata in APPLICATION !");
                        page = getInstance(pageClass, pageBean);
                        applicationContainer.setAttribute(
                            "AF_PAGE_" + pageName,
                            page);
                    } // if (page == null)
                } // synchronized (applicationContainer)
            } // if (page == null)
            else
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.DEBUG,
                    "PageFactory::getPage: page trovata in APPLICATION");
            return page;
        } // if (pageScope.equalsIgnoreCase("SESSION")) else
    } // public static PageIFace getPage(RequestContainer requestContainer, String pageName) throws ModuleException

    private static PageIFace getInstance(String pageClass, SourceBean config)
        throws ModuleException {
        PageIFace page = null;
        try {
            page = (PageIFace) Class.forName(pageClass).newInstance();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "PageFactory::getInstance: ",
                ex);
            throw new ModuleException(ex.getMessage());
        } // catch (Exception ex)
        if (page instanceof InitializerIFace)
             ((InitializerIFace) page).init(config);
        return page;
    } // private static PageIFace getInstance(String pageClass, SourceBean config) throws ModuleException
} // public class PageFactory
