/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.service;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorHandler;

/**
 * Questa interfaccia definisce  tutti i metodi necessari per gestire il contesto di
 * una richiesta di servizio.
 */
public interface RequestContextIFace {
    /**
     * Imposta lo stato della classe partendo da un 'istanza di <code>RequestContextIFace</code>
     * @param requestContext l'istanza da cui impostare lo stato.
     * @see RequestContainer
     * @see ResponseContainer
     */
    void setRequestContext(RequestContextIFace requestContext);

    /**
     * Ritorna il <code>RequestContainer</code> attributo di classe.
     * <p>
     * @see RequestContainer
     * @return il <code>RequestContainer</code> attributo di classe.
     */
    RequestContainer getRequestContainer();

    /**
     * Ritorna il <code>ResponseContainer</code> attributo di classe.
     * @return il <code>ResponseContainer</code> attributo di classe.
     * @see ResponseContainer
     */
    ResponseContainer getResponseContainer();

    /**
     * Ritorna il <code>SourceBean</code> contenente tutti i parametri della richiesta di servizio.
     * @return il <code>SourceBean</code> contenente i parametri della richiesta.
     */
    SourceBean getServiceRequest();

    /**
     * Ritorna il <code>SourceBean</code> contenente gli oggetti istanziati nella business logic in risposta
     * ad una richiesta di servizio.
     * @return il <code>SourceBean</code> contenente gli oggetti di risposta.
     */
    SourceBean getServiceResponse();

    /**
     * Ritorna il <code>EMFErrorHandler</code> gestore degli errori per una richiesta di servizio.
     * <p>
     * @return il <code>EMFErrorHandler</code> gestore degli errori.
     */
    EMFErrorHandler getErrorHandler();
} // public interface RequestContextIFace
