/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.service.detail.impl;

import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.dispatching.service.ServiceIFace;
import it.eng.spago.dispatching.service.detail.IFaceDetailService;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.QueryExecutor;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           SourceBean ora restituisce ArrayList
* 										    
**/

/**
 * @author 
 */
public class DelegatedDetailService {
    public static final String DETAIL_NEW = "DETAIL_NEW";
    public static final String DETAIL_INSERT = "DETAIL_INSERT";
    public static final String DETAIL_SELECT = "DETAIL_SELECT";
    public static final String DETAIL_UPDATE = "DETAIL_UPDATE";
    public static final String SERVICE_MODE = "SERVICE_MODE";
    public static final String SERVICE_MODE_INSERT = "INSERT";
    public static final String SERVICE_MODE_UPDATE = "UPDATE";

    private DelegatedDetailService() {
        super();
    } // public DelegatedDetailService()

    public static void service(ServiceIFace service, SourceBean request, SourceBean response) throws Exception {
        if ((service == null) || (request == null) || (response == null)) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "DelegatedDetailService::service: parametri non validi");
            return;
        } // if ((service == null) || (request == null) || (response == null))
        IFaceDetailService detailService = (IFaceDetailService)service;
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "DelegatedDetailService::service: request", request);
        String message = getMessage(request);
        if ((message == null) || message.equalsIgnoreCase("BEGIN"))
            message = DETAIL_NEW;
        try {
            if (message.equalsIgnoreCase(DETAIL_INSERT)) {
                response.setAttribute(SERVICE_MODE, SERVICE_MODE_INSERT);
                detailService.insert(request, response);
            } // if (message.equalsIgnoreCase(DETAIL_INSERT))
            else if (message.equalsIgnoreCase(DETAIL_SELECT)) {
                response.setAttribute(SERVICE_MODE, SERVICE_MODE_UPDATE);
                detailService.select(request, response);
            } // if (message.equalsIgnoreCase(DETAIL_SELECT))
            else if (message.equalsIgnoreCase(DETAIL_UPDATE)) {
                response.setAttribute(SERVICE_MODE, SERVICE_MODE_UPDATE);
                detailService.update(request, response);
            } // if (message.equalsIgnoreCase(DETAIL_UPDATE))
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DelegatedDetailService::service:", ex);
        } // catch (Exception ex)
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "DelegatedDetailService::service: response", response);
    } // public static void service(ServiceIFace service, SourceBean request,
    // SourceBean response) throws Exception

    /**
         * Questo metodo viene invocato quando viene eseguita una richiesta di inserimento di un nuovo
         * dettaglio.
         * @param request sourceBean contenente i parametri della richiesta.
         * @param response sourceBean di risposta.
         */
    public static boolean insert(ServiceIFace service, SourceBean request, SourceBean response) {
        InitializerIFace serviceInitializer = (InitializerIFace)service;
        String pool = (String)serviceInitializer.getConfig().getAttribute("POOL");
        SourceBean statement = (SourceBean)serviceInitializer.getConfig().getAttribute("QUERIES.INSERT_QUERY");
        RequestContextIFace serviceRequestContext = (RequestContextIFace)service;
        Boolean isOK = (Boolean)QueryExecutor.executeQuery(serviceRequestContext.getRequestContainer(), serviceRequestContext.getResponseContainer(),
            pool, statement, "INSERT");
        EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
        if ((isOK != null) && isOK.booleanValue()) {
            engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10007"));
            return true;
        } // if ((isOK != null) && isOK.booleanValue())
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
            "DefaultDetailModule::insert: errore inserimento riga");
        engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING, "10008"));
        restoreFields(service, request, response);
        return false;
    } // public static boolean insert(ServiceIFace service, SourceBean request,
    // SourceBean response)

    /**
         * Questo metodo viene invocato quando viene eseguita una richiesta di visualizzazione dei dati di un
         * dettaglio.
         * @param request sourceBean contenente i parametri della richiesta.
         * @param response sourceBean di risposta.
         */
    public static boolean select(ServiceIFace service, SourceBean request, SourceBean response) {
        InitializerIFace serviceInitializer = (InitializerIFace)service;
        String pool = (String)serviceInitializer.getConfig().getAttribute("POOL");
        SourceBean statement = (SourceBean)serviceInitializer.getConfig().getAttribute("QUERIES.SELECT_QUERY");
        RequestContextIFace serviceRequestContext = (RequestContextIFace)service;
        SourceBean rowsSourceBean = (SourceBean)QueryExecutor.executeQuery(serviceRequestContext.getRequestContainer(),
            serviceRequestContext.getResponseContainer(), pool, statement, "SELECT");
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "DefaultDetailModule::select: rowsSourceBean", rowsSourceBean);
        Object rowObject = null;
        if (rowsSourceBean != null)
            rowObject = rowsSourceBean.getAttribute(DataRow.ROW_TAG);
        EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
        if (rowObject == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "DefaultDetailModule::select: nessuna riga selezionata");
            engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING, "10006"));
            return false;
        } // if (rowObject == null)
        if (!(rowObject instanceof SourceBean)) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "DefaultDetailModule::select: pi righe selezionate");
            engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING, "10006"));
            return false;
        } // if (!(rowObject instanceof SourceBean))
        try {
            response.setAttribute((SourceBean)rowObject);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "DefaultDetailModule::select: response.setAttribute((SourceBean)rowObject)", ex);
        } // catch (Exception ex)
        return true;
    } // public static boolean select(ServiceIFace service, SourceBean request,
    // SourceBean response)

    /**
         * Questo metodo viene invocato quando viene eseguita una richiesta di modifica dei dati nel
         * dettaglio.
         * @param request sourceBean contenente i parametri della richiesta.
         * @param response sourceBean di risposta.
         */
    public static boolean update(ServiceIFace service, SourceBean request, SourceBean response) {
        InitializerIFace serviceInitializer = (InitializerIFace)service;
        String pool = (String)serviceInitializer.getConfig().getAttribute("POOL");
        SourceBean statement = (SourceBean)serviceInitializer.getConfig().getAttribute("QUERIES.UPDATE_QUERY");
        RequestContextIFace serviceRequestContext = (RequestContextIFace)service;
        Boolean isOK = (Boolean)QueryExecutor.executeQuery(serviceRequestContext.getRequestContainer(), serviceRequestContext.getResponseContainer(),
            pool, statement, "UPDATE");
        EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
        boolean success = false;
        if ((isOK != null) && isOK.booleanValue()) {
            engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10004"));
            success = true;
        } // if ((isOK != null) && isOK.booleanValue())
        else {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
                "DefaultDetailModule::update: errore aggiornamento riga");
            engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING, "10005"));
        } // if ((isOK != null) && isOK.booleanValue()) else
        restoreFields(service, request, response);
        return success;
    } // public static boolean update(ServiceIFace service, SourceBean request,

    // SourceBean response)
    public static String getMessage(SourceBean request) {
        return (String)request.getAttribute("MESSAGE");
    } // public static String getMessage(SourceBean request)

    /**
         * Questo metodo ripropone in response i dati presenti nella richiesta.
         * @param request sourceBean contenente i parametri della richiesta.
         * @param response sourceBean di risposta.
         */
    public static void restoreFields(ServiceIFace service, SourceBean request, SourceBean response) {
        InitializerIFace serviceInitializer = (InitializerIFace)service;
        try {
            SourceBean row = new SourceBean("ROW");
            List fields = serviceInitializer.getConfig().getAttributeAsList("FIELDS.FIELD");
            for (int i = 0; i < fields.size(); i++) {
                SourceBean field = (SourceBean)fields.get(i);
                String fieldName = (String)field.getAttribute("NAME");
                String fieldValue = (String)request.getAttribute(fieldName);
                row.setAttribute(fieldName, fieldValue);
            } // for (int i = 0; i < fields.size(); i++)
            response.setAttribute(row);
        } // try
        catch (SourceBeanException ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DefaultDetailModule::restoreFields:", ex);
        } // catch (Exception ex)
    } // public static void restoreFields(ServiceIFace service, SourceBean
    // request, SourceBean response)
} // public class DelegatedDetailService
