/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.service.list.smart.impl;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.dispatching.service.ServiceIFace;
import it.eng.spago.dispatching.service.list.smart.IFaceSmartListService;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.paginator.smart.IFaceListProvider;
import it.eng.spago.paginator.smart.IFacePageProvider;
import it.eng.spago.paginator.smart.IFaceRowHandler;
import it.eng.spago.paginator.smart.IFaceRowProvider;
import it.eng.spago.tracing.TracerSingleton;

public class DelegatedSmartListService {
    public static final String LIST_PAGE = "LIST_PAGE";
    public static final String LIST_FIRST = "LIST_FIRST";
    public static final String LIST_PREV = "LIST_PREV";
    public static final String LIST_NEXT = "LIST_NEXT";
    public static final String LIST_LAST = "LIST_LAST";
    public static final String LIST_CURRENT = "LIST_CURRENT";
    public static final String LIST_NOCACHE = "LIST_NOCACHE";
    public static final String LIST_DELETE = "LIST_DELETE";

    public DelegatedSmartListService() {
        super();
    } // public DelegatedSmartListService()

    public static void init(ServiceIFace service, SourceBean config) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "DelegatedSmartListService::init: invocato");
        IFaceSmartListService listService = (IFaceSmartListService) service;
        try {
            String rowProviderClass = (String) config.getAttribute("ROW_PROVIDER.CLASS");
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "DelegatedSmartListService::init: rowHandlerClass [" + rowProviderClass + "]");
            IFaceRowProvider rowProvider = (IFaceRowProvider) Class.forName(rowProviderClass).newInstance();
            InitializerIFace rowProviderInitializer = (InitializerIFace) rowProvider;
            SourceBean rowProviderConfig = (SourceBean) config.getAttribute("ROW_PROVIDER.CONFIG");
            rowProviderInitializer.init(rowProviderConfig);
            String pageProviderClass = (String) config.getAttribute("PAGE_PROVIDER.CLASS");
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "DelegatedSmartListService::init: pageProviderClass [" + pageProviderClass + "]");
            IFacePageProvider pageProvider = (IFacePageProvider) Class.forName(pageProviderClass).newInstance();
            pageProvider.setRowProvider(rowProvider);
            InitializerIFace pageProviderInitializer = (InitializerIFace) pageProvider;
            SourceBean pageProviderConfig = (SourceBean) config.getAttribute("PAGE_PROVIDER.CONFIG");
            pageProviderInitializer.init(pageProviderConfig);
            String listProviderClass = (String) config.getAttribute("LIST_PROVIDER.CLASS");
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "DelegatedSmartListService::init: listProviderClass [" + listProviderClass + "]");
            IFaceListProvider listProvider = (IFaceListProvider) Class.forName(listProviderClass).newInstance();
            listProvider.setPageProvider(pageProvider);
            InitializerIFace listProviderInitializer = (InitializerIFace) listProvider;
            SourceBean listProviderConfig = (SourceBean) config.getAttribute("LIST_PROVIDER.CONFIG");
            listProviderInitializer.init(listProviderConfig);
            listService.setList(listProvider);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DelegatedSmartListService::init: errore inizializzazione paginatore",
                ex);
            listService.setList(null);
        } // catch (Exception ex)
    } // public static void init(ServiceIFace service, SourceBean config)

    public static void service(ServiceIFace service, SourceBean request, SourceBean response) throws Exception {
        IFaceSmartListService listService = (IFaceSmartListService) service;
        IFaceListProvider listProvider = listService.getList();
        if (listProvider == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DelegatedSmartListService::service: listProvider nullo !");
            return;
        } // if (listProvider == null)
        IFacePageProvider pageProvider = listProvider.getPageProvider();
        if (pageProvider == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DelegatedSmartListService::service: pageProvider nullo !");
            return;
        } // if (pageProvider == null)
        IFaceRowProvider rowProvider = pageProvider.getRowProvider();
        if (rowProvider == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DelegatedSmartListService::service: rowProvider nullo !");
            return;
        } // if (rowProvider == null)
        RequestContextIFace serviceRequestContext = (RequestContextIFace) service;
        RequestContextIFace listProviderRequestContext = (RequestContextIFace) listProvider;
        listProviderRequestContext.setRequestContext(serviceRequestContext);
        String message = getMessage(request);
        if ((message == null) || message.equalsIgnoreCase("BEGIN"))
            message = LIST_FIRST;
        String list_nocache = (String) request.getAttribute(LIST_NOCACHE);
        if (list_nocache == null)
            list_nocache = "FALSE";
        if (message.equalsIgnoreCase(LIST_DELETE)) {
            listService.delete(request, response);
            message = LIST_PAGE;
            list_nocache = "TRUE";
        } // if (message.equalsIgnoreCase(LIST_DELETE))
        if (list_nocache.equalsIgnoreCase("TRUE"))
            listProvider.reload();
        int pagedListNumber = 1;
        if (message.equalsIgnoreCase(LIST_PAGE)) {
            String list_page = (String) request.getAttribute(LIST_PAGE);
            if (list_page == null)
                list_page = "1";
            try {
                pagedListNumber = Integer.parseInt(list_page);
            } // try
            catch (Exception ex) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "DelegatedSmartListService::service: Integer.parseInt(list_page)",
                    ex);
            } // catch (Exception ex) try
        } // if (message.equalsIgnoreCase(LIST_PAGE))
        else if (message.equalsIgnoreCase(LIST_FIRST))
            pagedListNumber = 1;
        else if (message.equalsIgnoreCase(LIST_PREV)) {
            pagedListNumber = listProvider.getCurrentPage() - 1;
            if (pagedListNumber < 1)
                pagedListNumber = 1;
        } // if (message.equalsIgnoreCase(LIST_PREV))
        else if (message.equalsIgnoreCase(LIST_NEXT))
            pagedListNumber = listProvider.getCurrentPage() + 1;
        else if (message.equalsIgnoreCase(LIST_LAST))
            pagedListNumber = IFaceListProvider.LAST;
        else if (message.equalsIgnoreCase(LIST_CURRENT))
            pagedListNumber = listProvider.getCurrentPage();
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DelegatedSmartListService::service: pagedListNumber richiesto [" + pagedListNumber + "]");
        listProvider.clearDynamicData();
        listService.callback(request, response, listProvider, pagedListNumber);
        SourceBean pagedList = (SourceBean) listProvider.getListPage(pagedListNumber);
        if (pagedList == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DelegatedSmartListService::service: pagedList nullo");
            return;
        } // if (pagedList == null)
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DelegatedSmartListService::service: pagedListNumber ritornato [" + listProvider.getCurrentPage() + "]");
        try {
            response.setAttribute(pagedList);
        } // try
        catch (SourceBeanException ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DelegatedSmartListService::service: response.setAttribute(pagedList)",
                ex);
        } // catch (SourceBeanException ex) try
    } // public static void service(ServiceIFace service, SourceBean request,

    // SourceBean response) throws Exception
    public static IFaceListProvider getList(ServiceIFace service, SourceBean request, SourceBean response) throws Exception {
        IFaceSmartListService listService = (IFaceSmartListService) service;
        IFaceListProvider listProvider = listService.getList();
        return listProvider;
    } // public static IFaceListProvider getList(ServiceIFace service,

    // SourceBean request, SourceBean response) throws Exception
    public static boolean delete(ServiceIFace service, SourceBean request, SourceBean response) {
        IFaceSmartListService listService = (IFaceSmartListService) service;
        IFaceListProvider listProvider = listService.getList();
        if (listProvider == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DelegatedSmartListService::delete: listProvider nullo !");
            return false;
        } // if (listProvider == null)
        IFacePageProvider pageProvider = listProvider.getPageProvider();
        if (pageProvider == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DelegatedSmartListService::delete: pageProvider nullo !");
            return false;
        } // if (pageProvider == null)
        IFaceRowProvider rowProvider = pageProvider.getRowProvider();
        if (rowProvider == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DelegatedSmartListService::delete: rowProvider nullo !");
            return false;
        } // if (rowProvider == null)
        if (!(rowProvider instanceof IFaceRowHandler)) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DelegatedSmartListService::delete: rowProvider non implementa IFaceRowHandler");
            return false;
        } // if (rowProvider == null)
        IFaceRowHandler rowHandler = (IFaceRowHandler) rowProvider;
        boolean isOK = rowHandler.deleteRow();
        RequestContextIFace serviceRequestContext = (RequestContextIFace) service;
        EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
        if (isOK)
            engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.INFORMATION, 10002));
        else {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.MAJOR,
                "DelegatedSmartListService::delete: errore cancellazione riga");
            engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING, 10003));
        } // if (isOK) else
        return isOK;
    } // public static boolean delete(ServiceIFace service, SourceBean request,

    // SourceBean response)
    public static String getMessage(SourceBean serviceRequest) {
        return (String) serviceRequest.getAttribute("MESSAGE");
    } // public static String getMessage(SourceBean serviceRequest)
} // public class DelegatedSmartListService
