package it.eng.spago.event;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.listeners.ISpagoListener;
import it.eng.spago.tracing.TracerSingleton;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author Zoppello
 * 
 * This class is an implementation of IEventNotifier
 * and is simple calls all event listeners registered
 * in spago in the file listeners.xml
 */
public class SynchronousEventNotifier implements IEventNotifier{

	public void notifyEvent(ISpagoEvent event, RequestContextIFace requestContext) {
		
        RequestContainer requestContainer = requestContext.getRequestContainer();
        ResponseContainer responseContainer = requestContext.getResponseContainer();
        List listenerClasses = getListenerClasses();
		
		Class listenerClass = null;
		ISpagoListener theListener = null;
		Iterator it = listenerClasses.iterator();
		while (it.hasNext()){
			listenerClass = (Class)it.next();
			try{
				
				theListener = (ISpagoListener)listenerClass.newInstance();
				
				theListener.onEvent(event, requestContainer, responseContainer);
				
			}catch(Exception e){
				TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.CRITICAL,
                        "SynchronousEventNotifier::notifyEvent: Exception",e);
			
			}
		}	
	}
	
	public List getListenerClasses(){
		
		List listenerClassList = new ArrayList();
		ApplicationContainer aApplicationContainer = ApplicationContainer.getInstance();
		
		if (aApplicationContainer.getAttribute("LISTENER_CLASS_LIST") != null){
			listenerClassList = (List)aApplicationContainer.getAttribute("LISTENER_CLASS_LIST");
		}else{
			// Leggo la documentazione  la salvo
			List listenerListSb = ConfigSingleton.getInstance().getAttributeAsList("EVENTS.EVENTS-LISTENER");
		
			Iterator it = listenerListSb.iterator();	
			String classname = null;
			Class listenerClass = null;
			while (it.hasNext()){
				classname = (String)((SourceBean)it.next()).getAttribute("listenerClass");
				try{
					listenerClass = Class.forName(classname);
					listenerClassList.add(listenerClass);
				}catch (Exception e) {
					TracerSingleton.log(
	                        Constants.NOME_MODULO,
	                        TracerSingleton.CRITICAL,
	                        "SynchronousEventNotifier::getListenerClasses: Exception",e);
				}
			}
			aApplicationContainer.setAttribute("LISTENER_CLASS_LIST", listenerClassList);
		}
		
		return listenerClassList;
	}


	
}
