/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.paginator.basic.impl;

import java.io.Serializable;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.paginator.basic.ListIFace;
import it.eng.spago.paginator.basic.PaginatorIFace;
import it.eng.spago.tracing.TracerSingleton;

public class GenericList implements ListIFace, Serializable {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    
    private static final String PAGED_LIST = "PAGED_LIST";
    private static final String STATIC_DATA = "STATIC_DATA";
    private static final String DYNAMIC_DATA = "DYNAMIC_DATA";
    private static final String PAGE_NUMBER = "PAGE_NUMBER";
    private static final String PAGES_NUMBER = "PAGES_NUMBER";
    private PaginatorIFace _paginator = null;
    private SourceBean _staticData = null;
    private SourceBean _dynamicData = null;
    private int _currentPage = 1;

    public GenericList() {
        try {
            _paginator = null;
            _staticData = new SourceBean(STATIC_DATA);
            _dynamicData = new SourceBean(DYNAMIC_DATA);
            _currentPage = 1;
        } // try
        catch (SourceBeanException ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "GenericList::GenericList:", ex);
        } // catch (SourceBeanException ex) try
    } // public GenericList()

    public PaginatorIFace getPaginator() {
        return _paginator;
    } // public PaginatorIFace getPaginator()

    public void setPaginator(PaginatorIFace paginator) {
        _paginator = paginator;
    } // public void setPaginator(PaginatorIFace paginator)

    public void addStaticData(SourceBean data) {
        try {
            _staticData.setAttribute(data);
        } // try
        catch (SourceBeanException ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "GenericList::addStaticData:", ex);
        } // catch (SourceBeanException ex) try
    } // public void addStaticData(SourceBean data)

    public void clearDynamicData() {
        _dynamicData.clearBean();
    } // public void clearDynamicData()

    public void addDynamicData(SourceBean data) {
        try {
            _dynamicData.setAttribute(data);
        } // try
        catch (SourceBeanException ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "GenericList::addDynamicData:", ex);
        } // catch (SourceBeanException ex) try
    } // public void addDynamicData(SourceBean data)

    public SourceBean getPagedList(int page) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "GenericList::getPagedList: parametro page [" + page + "]");
        SourceBean pagedList = null;
        try {
            pagedList = new SourceBean(PAGED_LIST);
            pagedList.setAttribute(_staticData);
            pagedList.setAttribute(_dynamicData);
            if ((_paginator == null) || (_paginator.pages() == 0)) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "GenericList::getPagedList: _paginator nullo");
                return pagedList;
            } // if ((_paginator == null) || (_paginator.pages() == 0))
            setCurrentPage(page);
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "GenericList::getPagedList: page calcolato [" + page + "]");
            SourceBean pageBean = _paginator.getPage(page);
            if (pageBean == null) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "GenericList::getPagedList: pageBean nullo");
                return pagedList;
            } // if (pageBean == null)
            pagedList.setAttribute(PAGE_NUMBER, String.valueOf(page));
            pagedList.setAttribute(PAGES_NUMBER, String.valueOf(_paginator.pages()));
            pagedList.setAttribute(pageBean);
        } // try
        catch (SourceBeanException ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "GenericList::getPagedList:", ex);
        } // catch (SourceBeanException ex) try
        return pagedList;
    } // public SourceBean getPagedList(int page)

    public int getCurrentPage() {
        if (_paginator == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "GenericList::getCurrentPage: _paginator nullo");
            return 0;
        } // if (_paginator == null)
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "GenericList::getCurrentPage: _currentPage [" + _currentPage + "]");
        return _currentPage;
    } // public int getCurrentPage()

    public void setCurrentPage(int page) {
        if (_paginator == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "GenericList::setCurrentPage: _paginator nullo");
            return;
        } // if (_paginator == null)
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "GenericList::setCurrentPage: page [" + page + "]");
        if (page < 1)
            page = 1;
        if (page > _paginator.pages())
            page = _paginator.pages();
        _currentPage = page;
    } // public void setCurrentPage(int page)

    public int getPrevPage() {
        if (_paginator == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "GenericList::getPrevPage: _paginator nullo");
            return 0;
        } // if (_paginator == null)
        if (_currentPage <= 1)
            return 1;
        _currentPage--;
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "GenericList::getPrevPage: _currentPage [" + _currentPage + "]");
        return _currentPage;
    } // public int getPrevPage()

    public int getNextPage() {
        if (_paginator == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "GenericList::getNextPage: _paginator nullo");
            return 0;
        } // if (_paginator == null)
        if (_currentPage >= _paginator.pages())
            return _paginator.pages();
        _currentPage++;
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "GenericList::getPrevPage: _currentPage [" + _currentPage + "]");
        return _currentPage;
    } // public int getNextPage()

    public int pages() {
        if (_paginator == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "GenericList::pages: _paginator nullo");
            return 0;
        } // if ((_paginator == null) || (_paginator.pages() == 0))
        return _paginator.pages();
    } // public int pages()
} // public class GenericList implements ListIFace, Serializable
