/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.paginator.basic.impl;

import java.io.Serializable;
import java.util.Vector;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.paginator.basic.PaginatorIFace;
import it.eng.spago.tracing.TracerSingleton;

public class GenericPaginator implements PaginatorIFace, Serializable {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    public GenericPaginator() {
        _rows = new Vector();
        _size = 10;
    } // public GenericPaginator()

    public void addRow(Object row) {
        if (row == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "GenericPaginator::addRow: row nullo");
            return;
        } // if (row == null)
        _rows.addElement(row);
    } // public void addRow(Object row)

    public int getPageSize() {
        return _size;
    } // public int getPageSize()

    public void setPageSize(int size) {
        if (size < 1) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "GenericPaginator::setPageSize: size [" + size + "] non valida");
            return;
        } // if (size < 1)
        _size = size;
    } // public void setPageSize(int size)

    public int rows() {
        return _rows.size();
    } // public int rows()

    public int pages() {
        if (_rows.size() == 0)
            return 0;
        return (_rows.size() - 1) / _size + 1;
    } // public int pages()

    public SourceBean getPage(int page) {
        SourceBean rows = null;
        try {
            rows = new SourceBean(ROWS);
            if ((page < 1) || (page > pages())) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "GenericPaginator::getPage: page [" + page + "] non valida");
                return rows;
            } // if ((page < 1) || (page > pages()))
            int startRow = (page - 1) * _size;
            int endRow = page * _size;
            if (endRow > _rows.size())
                endRow = _rows.size();
            for (int i = startRow; i < endRow; i++) {
                Object objectRow = _rows.elementAt(i);
                if (!(objectRow instanceof SourceBean)) {
                    SourceBean row = new SourceBean("ROW");
                    row.setAttribute("VALUE", objectRow);
                    objectRow = row;
                } // if (!(objectRow instanceof SourceBean))
                rows.setAttribute((SourceBean)objectRow);
            } // for (int i = startRow; i < endRow; i++)
        } // try
        catch (SourceBeanException ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "GenericPaginator::getPage:", ex);
        } // catch (SourceBeanException ex) try
        return rows;
    } // public SourceBean getPage(int page)

    public SourceBean getAll() {
        SourceBean rows = null;
        try {
            rows = new SourceBean(ROWS);
            for (int i = 0; i < _rows.size(); i++) {
                Object objectRow = _rows.elementAt(i);
                if (!(objectRow instanceof SourceBean)) {
                    SourceBean row = new SourceBean("ROW");
                    row.setAttribute("VALUE", objectRow);
                    objectRow = row;
                } // if (!(objectRow instanceof SourceBean))
                rows.setAttribute((SourceBean)objectRow);
            } // for (int i = startRow; i < endRow; i++)
        } // try
        catch (SourceBeanException ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "GenericPaginator::getAll:", ex);
        } // catch (SourceBeanException ex) try
        return rows;
    } // public SourceBean getAll()

    private Vector _rows = null;
    private int _size = 0;
    private static final String ROWS = "ROWS";
} // public class GenericPaginator implements PaginatorIFace, Serializable
