/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.paginator.smart;

/**
 * L'interfaccia <code>IFaceListProvider</code> definisce i metodi per la gestione
 * di una lista paginata.
 * @version 1.0, 15/03/2003
 */
public interface IFaceListProvider {
    public static final int LAST = -1;

    /**
     * Notifica al componente di ricaricare i dati dei quali fa cacheing.
     * <p>
     * @see AbstractListProvider#init(SourceBean)
     */
    void reload();

    /**
     * Restituisce il <em>page provider</em> associato.
     * @return <code>IFacePageProvider</code>
     * @see IFacePageProvider
     */
    IFacePageProvider getPageProvider();

    /**
     * Imposta il <em>page provider</em> associato.
     * @param pageProvider <code>IFacePageProvider</code>
     * @see IFacePageProvider
     */
    void setPageProvider(IFacePageProvider pageProvider);

    /**
     * Aggiunge un oggetto nella sezione STATIC_DATA associata a ciascuna
     * pagina pubblicata.
     * @param data <code>Object</code>
     */
    void addStaticData(Object data);

    /**
     * Aggiunge un oggetto nella sezione DYNAMIC_DATA associata a ciascuna
     * pagina pubblicata.
     * @param data <code>Object</code>
     */
    void addDynamicData(Object data);

    /**
     * Svuota la sezione DYNAMIC_DATA associata a ciascuna pagina pubblicata.
     */
    void clearDynamicData();

    /**
     * Ritorna un <code>Object</code> che rappresenta la pagina di indice
     * <em>page</em> richiesto.
     * @param page <code>int</code>
     * @return <code>Object</code>
     */
    Object getListPage(int i);

    /**
     * Ritorna il numero di pagina corrente.
     * @return <code>int</code>
     */
    int getCurrentPage();
} // public interface IFaceListProvider
