/**

 Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.paginator.smart.impl;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.paginator.smart.AbstractRowProvider;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;
import it.eng.spago.util.StringUtils;

import java.sql.Types;
import java.util.ArrayList;
import java.util.List;

/**
 * @author boccalon
 * 
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates. To enable and disable the creation of type
 * comments go to Window>Preferences>Java>Code Generation.
 */
public class DBFilteredRowProvider extends AbstractRowProvider {
	/**
     * 
     */
    private static final long serialVersionUID = 1L;

    private DataConnection _dataConnection = null;

	private SQLCommand _sqlCommand = null;

	private DataResult _dataResult = null;

	/**
	 * Costruisce un <code>DBRowProvider</code>. Questo costruttore &egrave;
	 * vuoto ed ogni azione di inizializzazione &egrave; demandata al metodo
	 * <code>init(SourceBean config)</code>.
	 * <p>
	 * 
	 * @see DBRowProvider#init(SourceBean)
	 */
	public DBFilteredRowProvider() {
		super();
		_dataConnection = null;
		_sqlCommand = null;
		_dataResult = null;
	} // public DBRowProvider()

	/**
	 * Questo metodo apre la connessione alla sorgente dati e recupera tutte le
	 * righe relative alla query configurata.
	 * <p>
	 * 
	 * @see DBRowProvider#init(SourceBean)
	 */
	public void open() {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"DBRowProvider::open: invocato");
		super.open();
		SourceBean config = getConfig();
		SourceBean query = (SourceBean) config.getAttribute("LIST_QUERY");
		String pool = (String) config.getAttribute("POOL");
		try {
			DataConnectionManager dataConnectionManager = DataConnectionManager
					.getInstance();
			_dataConnection = dataConnectionManager.getConnection(pool);
			String selectStatement = SQLStatements.getStatement(
					(String) query.getAttribute("STATEMENT")).toUpperCase();
			StringBuffer statement = new StringBuffer(selectStatement);
			ArrayList inputParameters = new ArrayList();

			// Elaboro i parametri normali, ossia quelli di tipo "PARAMETER"
			List parameters = query.getAttributeAsList("PARAMETER");
			for (int i = 0; i < parameters.size(); i++) {
				SourceBean parameter = (SourceBean) parameters.get(i);
				handleParameter(parameter, inputParameters, _dataConnection);
			} //for

			// Elaboro i parametri di filtro, ossia quelli di tipo
			// "FILTER_PARAMETER"
			List filterParameters = query
					.getAttributeAsList("FILTER_PARAMETER");
			String condizioneSql = (selectStatement.indexOf("WHERE") != -1) ? " AND "
					: " WHERE ";
			for (int i = 0; i < filterParameters.size(); i++) {
				SourceBean filterParameter = (SourceBean) filterParameters
						.get(i);

				if (handleFilterParameter(filterParameter, inputParameters,
						_dataConnection, statement, condizioneSql)) {
					condizioneSql = " AND ";
				}
			} //for

			// Elabora tutte le sezioni di tipo "GROUP_BY"
			SourceBean groupByParameters = (SourceBean) query
					.getAttribute("GROUP_BY");
			if (groupByParameters != null) {
				handleOrderByParameter(groupByParameters, statement,
						" GROUP BY ");
			}

			// Elabora tutte le sezioni di tipo "ORDER_BY"
			SourceBean orderByParameters = (SourceBean) query
					.getAttribute("ORDER_BY");
			if (orderByParameters != null) {
				handleOrderByParameter(orderByParameters, statement,
						" ORDER BY ");
			}

			_sqlCommand = _dataConnection.createSelectCommand(statement
					.toString());
			_dataResult = _sqlCommand.execute(inputParameters);
			ScrollableDataResult scrollableDataResult = (ScrollableDataResult) _dataResult
					.getDataObject();
			setRows(scrollableDataResult.getRowsNumber());
			setCurrentRow(0);
		} // try
		catch (Exception ex) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL, "DBRowProvider::executeQuery:",
					ex);
			Utils.releaseResources(_dataConnection, _sqlCommand, _dataResult);
			_dataConnection = null;
			_sqlCommand = null;
			_dataResult = null;
		} // catch (Exception ex)
	} // public void open()

	/**
	 * Questo metodo serve per posizionare il <em>cursore</em> di lettura alla
	 * riga individuata da <em>row</em>.
	 * <p>
	 * 
	 * @param row
	 *            <code>int</code> il numero della riga su cui posizionarsi.
	 */
	public void absolute(int row) {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"DBRowProvider::absolute: invocato");
		if (_dataResult == null) {
			TracerSingleton
					.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
							"DBRowProvider::absolute: row provider non aperto correttamente !");
			return;
		} // if (_dataResult == null)
		if (row < 1)
			row = 1;
		else if (row > rows())
			row = rows();
		setCurrentRow(row - 1);
		ScrollableDataResult scrollableDataResult = (ScrollableDataResult) _dataResult
				.getDataObject();
		try {
			scrollableDataResult.moveTo(row);
		} // try
		catch (EMFInternalError emfie) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL, "DBRowProvider::absolute:",
					(Exception) emfie);
		} // catch (EMFInternalError emfie)
	} // public void absolute(int row)

	/**
	 * Ritorna un <code>Object</code> che rappresenta la riga individuata da
	 * <em>row</em>.
	 * <p>
	 * 
	 * @return <code>Object</code> la riga individuata da <em>row</em>.
	 * @param row
	 *            <code>int</code> il numero della riga da ritornare.
	 */
	public Object getRow(int row) {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"DBRowProvider::getRow: invocato");
		if (_dataResult == null) {
			TracerSingleton
					.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
							"DBRowProvider::getRow: row provider non aperto correttamente !");
			return null;
		} // if (_dataResult == null)
		if (row < 1)
			row = 1;
		else if (row > rows())
			row = rows();
		setCurrentRow(row);
		ScrollableDataResult scrollableDataResult = (ScrollableDataResult) _dataResult
				.getDataObject();
		SourceBean rowBean = null;
		try {
			rowBean = scrollableDataResult.getDataRow(row).getSourceBean();
		} // try
		catch (EMFInternalError emfie) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL, "DBRowProvider::getRow:",
					(Exception) emfie);
		} // catch (EMFInternalError emfie)
		return rowBean;
	} // public Object getRow(int row)

	/**
	 * Ritorna un <code>Object</code> che rappresenta la riga successiva
	 * all'ultima recuperata o individuata con il comando <code>absolute(int)
	 * </code>.
	 * <p>
	 * 
	 * @return <code>Object</code> la riga individuata da <em>row</em>.
	 * @param row
	 *            <code>int</code> il numero della riga da ritornare.
	 */
	public Object getNextRow() {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"DBRowProvider::getNextRow: invocato");
		if (_dataResult == null) {
			TracerSingleton
					.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
							"DBRowProvider::getNextRow: row provider non aperto correttamente !");
			return null;
		} // if (_dataResult == null)
		ScrollableDataResult scrollableDataResult = (ScrollableDataResult) _dataResult
				.getDataObject();
		SourceBean rowBean = null;
		try {
			if (scrollableDataResult.hasRows()) {
				rowBean = scrollableDataResult.getDataRow().getSourceBean();
				setCurrentRow(getCurrentRow() + 1);
			} // if (scrollableDataResult.hasRows())
		} // try
		catch (EMFInternalError emfie) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL, "DBRowProvider::getNextRow:",
					(Exception) emfie);
		} // catch (EMFInternalError emfie)
		return rowBean;
	} // public Object getNextRow()

	/**
	 * Questo metodo chiude la connessione alla sorgente dati.
	 * <p>
	 * 
	 * @see DBRowProvider#init(SourceBean)
	 */
	public void close() {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"DBRowProvider::close: invocato");
		Utils.releaseResources(_dataConnection, _sqlCommand, _dataResult);
		_dataConnection = null;
		_sqlCommand = null;
		_dataResult = null;
		setRows(0);
		setCurrentRow(0);
	} // public void close()

	protected boolean handleParameter(SourceBean parameter,
			ArrayList inputParameters, DataConnection dataConnection) {

		return handleParameter(parameter, inputParameters, dataConnection,
				false, null, null);
	}

	protected boolean handleFilterParameter(SourceBean parameter,
			ArrayList inputParameters, DataConnection dataConnection,
			StringBuffer statement, String condizioneSql) {

		return handleParameter(parameter, inputParameters, dataConnection,
				true, statement, condizioneSql);
	}

	protected boolean handleParameter(SourceBean parameter,
			ArrayList inputParameters, DataConnection dataConnection,
			boolean isFilterParameter, StringBuffer statement,
			String condizioneSql) {

		boolean parameterUsed = false;
		// Lettura attributi di "tipo" del parametro
		String parameterType = (String) parameter.getAttribute("TYPE");
		String parameterValue = (String) parameter.getAttribute("VALUE");
		String parameterScope = (String) parameter.getAttribute("SCOPE");

		String inParameterValue = null;
		if (parameterType.equalsIgnoreCase("ABSOLUTE"))
			inParameterValue = parameterValue;
		else {
			Object parameterValueObject = ContextScooping.getScopedParameter(
					getRequestContainer(), getResponseContainer(),
					parameterValue, parameterScope, parameter);
			if (parameterValueObject != null)
				inParameterValue = parameterValueObject.toString();
		} //else
		if (inParameterValue == null)
			inParameterValue = "";

		if (!isFilterParameter) { // In caso di parametro "normale"

			inputParameters.add(dataConnection.createDataField("",
					Types.VARCHAR, inParameterValue));
			parameterUsed = true;
		} else { // In caso di parametro di filtro

			// Se il parametro di filtro HA VERAMENTE UN VALORE,
			if (!inParameterValue.equals("")) {

				// Lettura attributo di "sql" del parametro
				String sqlToAdd = (String) parameter.getAttribute("SQL");

				// Aggiungo il parametro tra i parametri di input dell'SQL, se
				// necessario
				int countParams = StringUtils.count(sqlToAdd, '?');
				if (countParams != 0) {
					// Aggiungo il parametro tante volte quanti sono i caratteri
					// '?' presenti nella stringa
					for (int i = 0; i < countParams; i++) {
						inputParameters.add(dataConnection.createDataField("",
								Types.VARCHAR, inParameterValue));
					}
					statement.append(condizioneSql);
				}

				// E aggiungo la parte di SQL allo statement (modifico
				// direttamente l'oggetto StringBuffer)
				statement.append(sqlToAdd);

				parameterUsed = true;

			} //if !inParameterValue
		} //else !isFilterParameter
		return parameterUsed;

	} //elaboraParametro

	protected void handleOrderByParameter(SourceBean parameter,
			StringBuffer statement, String condizioneSql) {

		// Lettura attributo di "sql" del parametro
		String sqlToAdd = (String) parameter.getAttribute("SQL");

		// Aggiungo la parte di SQL allo statement (modifico direttamente
		// l'oggetto StringBuffer)
		statement.append(condizioneSql);
		statement.append(sqlToAdd);

	} //elaboraParametroOrderBy

}
