/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.paginator.smart.impl;

import java.util.ArrayList;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.paginator.smart.AbstractListProvider;
import it.eng.spago.paginator.smart.IFaceListProvider;
import it.eng.spago.paginator.smart.IFacePageProvider;
import it.eng.spago.tracing.TracerSingleton;

/**
 * La classe <code>DefaultListProvider</code> &egrave;
 * l'implementazione di default di <code>AbstractListProvider</code>.
 * Questa implementazione consente di riutilizzare i fogli di stile scritti
 * per il precedente paginatore perch&eacute; mantiene la medesima
 * rappresentazione XML di ciascuna pagina.
 * @version 1.0, 15/03/2003
 * @see AbstractListProvider
 */
public class DefaultListProvider extends AbstractListProvider {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private static final String ROWS = "ROWS";
    private static final String STATIC_DATA = "STATIC_DATA";
    private static final String DYNAMIC_DATA = "DYNAMIC_DATA";
    private static final String PAGED_LIST = "PAGED_LIST";
    private static final String PAGES_NUMBER = "PAGES_NUMBER";
    private static final String PAGE_NUMBER = "PAGE_NUMBER";

    /**
     * Costruisce un <code>DefaultListProvider</code>. Questo costruttore
     * &egrave; vuoto ed ogni azione di inizializzazione &egrave; demandata
     * al metodo <code>init(SourceBean config)</code> in
     * <code>AbstractListProvider</code>.
     * <p>
     * @see AbstractListProvider#init(SourceBean)
     */
    public DefaultListProvider() {
        super();
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DefaultListProvider::DefaultListProvider: invocato");
    } // public DefaultListProvider()

    public void init(SourceBean config) {
        super.init(config);
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DefaultListProvider::init: invocato");
    } // public void init(SourceBean config)

    /**
     * Ritorna un <code>SourceBean</code> contenente tutti gli attributi utili
     * a rappresentare la pagina richiesta. La rappresentazione XML del
     * <code>SourceBean</code> &egrave; quella gi&agrave; usata dal precedente
     * paginatore. Questo consente di mantenere tutti i fogli di stile
     * gi&agrave; definiti.
     * <p>
     * @param page <code>int</code> numero della pagina richiesta. Se
     * <em>page</em> &egrave; non positivo viene tornata la prima pagina;
     * se <em>page</em> eccede il numero di pagine viene tornata l'ultima pagina
     * disponibile.
     * @see AbstractPageProvider#getPage(int)
     */
    public Object getListPage(int page) {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DefaultListProvider::getListPage: pagina richiesta ["
                + page
                + "]");
        SourceBean listData = null;
        try {
            listData = new SourceBean(PAGED_LIST);
            SourceBean staticData = new SourceBean(STATIC_DATA);
            ArrayList staticDataArrayList = new ArrayList(getStaticData());
            for (int i = 0; i < staticDataArrayList.size(); i++)
                staticData.setAttribute(
                    (SourceBean) staticDataArrayList.get(i));
            listData.setAttribute(staticData);
            SourceBean dynamicData = new SourceBean(DYNAMIC_DATA);
            ArrayList dynamicDataArrayList = new ArrayList(getDynamicData());
            for (int i = 0; i < dynamicDataArrayList.size(); i++)
                dynamicData.setAttribute(
                    (SourceBean) dynamicDataArrayList.get(i));
            listData.setAttribute(dynamicData);
            SourceBean pageData = new SourceBean(ROWS);
            listData.setAttribute(pageData);
            int currentPage = 0;
            int pages = 0;
            IFacePageProvider pageProvider = getPageProvider();
            if (pageProvider != null) {
                if (page == IFaceListProvider.LAST)
                    page = IFacePageProvider.LAST;
                ArrayList rows = new ArrayList(pageProvider.getPage(page));
                for (int i = 0; i < rows.size(); i++)
                    if (rows.get(i) instanceof SourceBean)
                        pageData.setAttribute((SourceBean) rows.get(i));
                    else
                        pageData.setAttribute("row", rows.get(i));
                currentPage = pageProvider.getCurrentPage();
                pages = pageProvider.pages();
            } // if (pageProvider != null)
            setCurrentPage(currentPage);
            listData.setAttribute(PAGE_NUMBER, String.valueOf(currentPage));
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "DefaultListProvider::getListPage: pagina ritornata ["
                    + currentPage
                    + "]");
            listData.setAttribute(PAGES_NUMBER, String.valueOf(pages));
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "DefaultListProvider::getListPage: numero pagine ["
                    + pages
                    + "]");
        } // try
        catch (SourceBeanException sbe) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DefaultListProvider::getListPage:",
                sbe);
        } // catch (SourceBeanException sbe)
        return listData;
    } // public Object getListPage(int page)
} // public abstract class DefaultListProvider extends AbstractListProvider
