/**

    Copyright 2004 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**/
package it.eng.spago.presentation;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.base.XMLObject;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.transcoding.TranscoderFactory;
import it.eng.spago.transcoding.TranscoderIFace;

import java.io.ByteArrayOutputStream;
import java.util.Hashtable;
import java.util.List;

import javax.xml.transform.Templates;
import javax.xml.transform.stream.StreamResult;

public class PresentationRendering {
	
	/**
	 * Perform the render of the response for non JSP/Servlet/Loop publishers.
	 * @param requestContext
	 * @param resources The presentation resources defined in configuration files
	 * @return The string to write on output stream
	 */
	public static String render(RequestContextIFace requestContext,
			List resources) {
		String isXMLRequest = (String)requestContext.getRequestContainer().getAttribute(Constants.HTTP_IS_XML_REQUEST);
		
		boolean isAjaxRequest = (isXMLRequest != null) && (isXMLRequest.equalsIgnoreCase("TRUE"));
		boolean noPublishersDefined = (resources == null) || (resources.size() == 0);
		if (isAjaxRequest && noPublishersDefined) {
			// Keep service response "as is": dont introduce Spago elements in response
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
					"PresentationRendering::render: resources nullo");
			// The produced XML is without inline entity and without indentation
			return requestContext.getServiceResponse().toXML(false, false);
		} // if ((resources == null) || (resources.size() == 0))
		
		return render(requestContext.getResponseContainer(), resources);
	}

	public static String render(ResponseContainer responseContainer,
			List resources) {
		SourceBean responseBean = null;
		try {
			responseBean = new SourceBean("RESPONSE");
			responseBean.setAttribute(responseContainer.getServiceResponse());
			responseBean.setAttribute(responseContainer.getErrorHandler()
					.getSourceBean());
		} // try
		catch (SourceBeanException sbe) {
			TracerSingleton
					.log(
							Constants.NOME_MODULO,
							TracerSingleton.CRITICAL,
							"PresentationRendering::render: errore creazione stream RESPONSE",
							sbe);
			return "";
		} // catch (SourceBeanException sbe)
		if ((resources == null) || (resources.size() == 0)) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
					"PresentationRendering::render: resources nullo");
			return toXML(responseBean);
		} // if ((resources == null) || (resources.size() == 0))
		// Modifica Monica 15/01/2004 - inizio
		// recupero il SourceBean della busta ITEM, contenente la busta CONFIG
		String stylesheetName = (String) ((SourceBean) resources.get(0))
				.getAttribute("resource");
		// Modifica Monica 15/01/2004 - fine

		if (stylesheetName == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
					"PresentationRendering::render: stylesheetName nullo");
			return toXML(responseBean);
		} // if (stylesheetName == null)
		ApplicationContainer application = ApplicationContainer.getInstance();
		Hashtable xslTemplates = (Hashtable) application
				.getAttribute("XSL_TEMPLATES");
		if (xslTemplates == null) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"PresentationRendering::render: xslTemplates nullo");
			return toXML(responseBean);
		} // if (xslTemplates == null)
		// Recupero dalla cache il file contrassegnato dalla label
		// stylesheetLabel
		StylesheetFile stylesheet = (StylesheetFile) xslTemplates
				.get(stylesheetName);
		if (stylesheet == null) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"PresentationRendering::render: template di stylesheetName ["
							+ stylesheetName + "] non trovato");
			return toXML(responseBean);
		} // if (stylesheet == null)
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"PresentationRendering::render: template di stylesheetName ["
						+ stylesheetName + "]");
		// Recupero la struttura XSL dall'oggetto StylesheetFile
		Templates template = stylesheet.getTemplate();
		ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		StreamResult streamResult = new StreamResult(byteArrayOutputStream);

		try {
			TranscoderIFace transcoder = TranscoderFactory
					.getTranscoder(responseContainer);
			transcoder.perform(responseBean, template, streamResult);
		} catch (EMFInternalError err) {
			TracerSingleton
					.log(
							Constants.NOME_MODULO,
							TracerSingleton.CRITICAL,
							"PresentationRendering::render: errore durante il rendering",
							(XMLObject) err);
			return toXML(responseBean);
		}

		String result = byteArrayOutputStream.toString();
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"PresentationRendering::render: risultato trasformazione\n"
						+ result);
		return result;
	} // public static String render(ResponseContainer responseContainer,
		// ArrayList resources)

	private static String toXML(SourceBean source) {
		return source.toXML(true, false);
	}
} // public class PresentationRendering
