/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.presentation;

import java.io.File;

import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;

import org.apache.xalan.processor.TransformerFactoryImpl;

import it.eng.spago.base.Constants;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

public class StylesheetFile {
    private String _fileName = null;
    private long _fileDate = 0;
    private Templates _template = null;

    public StylesheetFile() {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "StylesheetFile::StylesheetFile: invocato");
    } // public StylesheetFile()

    public void setFileName(String fileName) {
        _fileName = fileName;
    } // public void setFileName (String fileName)

    public Templates getTemplate() {
        if (_fileName == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "StylesheetFile::getTemplate: _fileName nullo");
            return null;
        } // if (_fileName == null)
        ConfigSingleton configure = ConfigSingleton.getInstance();
        //verifico se ricaricare gli xsl
        String reload = (String)configure.getAttribute("COMMON.XSL_RELOAD");
        if ((_template == null) || ((reload != null) && reload.equalsIgnoreCase("TRUE"))) {
            // Ricarico le pagine XSL con la data di registrazione diversa da quella caricata
            try {
                File stylesheet = new File(ConfigSingleton.getRootPath() + _fileName);
                long currentFileDate = stylesheet.lastModified();
                // se la data del file in cache  diversa da quello su file System
                // allora sostitusco l'oggetto in cache.
                if (_fileDate != currentFileDate)
                    loadStylesheet();
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "StylesheetFile::getTemplate:", ex);
            } // catch (Excpetion ex)
        } // if ((_template == null) || ((reload != null) && reload.equalsIgnoreCase("TRUE")))
        return _template;
    } // public Templates getTemplate()

    // Questo metodo mi permette di caricare un solo stylesheet
    public void loadStylesheet() {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "StylesheetFile::loadStylesheet: invocato");
        try {
            // nuova istanza per la lettura dei file XSL
            File stylesheet = new File(ConfigSingleton.getRootPath() + _fileName);
            _fileDate = stylesheet.lastModified();
            Source xslSource = new StreamSource(stylesheet);

            /*
            System.setProperty("javax.xml.transform.TransformerFactory", "org.apache.xalan.processor.TransformerFactoryImpl");
            TransformerFactory tFactory = TransformerFactory.newInstance();
            */

            TransformerFactory tFactory = new TransformerFactoryImpl();
            _template = tFactory.newTemplates(xslSource);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "FileStylesheet::loadStylesheet:", ex);
        } // catch (Excpetion ex) try
    } // public void loadStylesheet()
} // public class StylesheetFile
