/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.presentation;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;
/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           SourceBean ora restituisce ArrayList
* 										    
**/

/**
 * @author 
 */
public class StylesheetInitializer implements InitializerIFace {
    private SourceBean _config = null;

    public StylesheetInitializer() {
        super();
        _config = null;
    } // public StylesheetInitializer()

    public void init(SourceBean config) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "StylesheetInitializer::init: config", config);
        _config = config;
        ArrayList stylesheets = new ArrayList();
        ConfigSingleton configure = ConfigSingleton.getInstance();
        List publishers = configure.getAttributeAsList("PUBLISHERS.PUBLISHER");
        for (int i = 0; i < publishers.size(); i++) {
            SourceBean publisher = (SourceBean)publishers.get(i);
            List renderings = publisher.getFilteredSourceBeanAttributeAsList("RENDERING", "TYPE", "XSL");
            for (int j = 0; j < renderings.size(); j++) {
                SourceBean rendering = (SourceBean)renderings.get(j);
                List items = rendering.getAttributeAsList("RESOURCES.ITEM");
                for (int k = 0; k < items.size(); k++) {
                    SourceBean item = (SourceBean)items.get(k);
                    stylesheets.add(item.getAttribute("RESOURCE"));
                } // for (int k = 0; k < items.size(); k++)
            } // for (int j = 0; j < renderings.size(); j++)
        } // for (int i = 0; i < publishers.size(); i++)
        Hashtable xslTemplates = new Hashtable();
        for (int i = 0; i < stylesheets.size(); i++) {
            String stylesheet = (String)stylesheets.get(i);
            if (!xslTemplates.containsKey(stylesheet)) {
                StylesheetFile stylesheetFile = new StylesheetFile();
                stylesheetFile.setFileName(stylesheet);
                if (stylesheetFile.getTemplate() == null)
                    TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                        "StylesheetInitializer::init: errore caricamento stylesheet [" + stylesheet + "]");
                else
                    TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                        "StylesheetInitializer::init: stylesheet [" + stylesheet + "] caricato");
                xslTemplates.put(stylesheet, stylesheetFile);
            } // if (!xslTemplates.containsKey(stylesheet))
        } // for (int i = 0; i < stylesheets.size(); i++)
        ApplicationContainer.getInstance().setAttribute("XSL_TEMPLATES", xslTemplates);
    } // public void init(SourceBean config)

    public SourceBean getConfig() {
        return _config;
    } // public SourceBean getConfig()
} // public class StylesheetInitializer implements InitializerIFace
