/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.security;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
 /**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           SourceBean ora restituisce ArrayList
* 										    
**/
/**
 * @author zoppello
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */

public class SecurityConfigurationSingleton {
    private static SecurityConfigurationSingleton _instance = null;
    private List actionDifferentFromDefaultConfiguration = null;
    private List pageModuleDifferentFromDefaultConfiguration = null;
    public boolean isSecurityGloballyEnabled = false;

    private SecurityConfigurationSingleton() {
        SourceBean globalSecuritySettings = (SourceBean)ConfigSingleton.getInstance().getAttribute("SECURITY.GLOBAL_SETTING");
        if (globalSecuritySettings == null)
            this.isSecurityGloballyEnabled = false;
        else if (globalSecuritySettings.getCharacters().equalsIgnoreCase("ENABLED"))
            this.isSecurityGloballyEnabled = true;
        else
            this.isSecurityGloballyEnabled = false;
        actionDifferentFromDefaultConfiguration = new ArrayList();
        List actions = ConfigSingleton.getInstance().getAttributeAsList("SECURITY.ACTIONS_EXCLUDED_FROM_GLOBAL.ACTION");
        Iterator actionsIterator = actions.iterator();
        while (actionsIterator.hasNext()) {
            SourceBean temp = (SourceBean)actionsIterator.next();
            String actionName = (String)temp.getAttribute("NAME");
            actionDifferentFromDefaultConfiguration.add(actionName.toUpperCase());
        } // while (actionsIterator.hasNext())
        pageModuleDifferentFromDefaultConfiguration = new ArrayList();
        List pageModules = ConfigSingleton.getInstance().getAttributeAsList("SECURITY.PAGE_MODULES_EXCLUDED_FROM_GLOBAL.PAGE_MODULE");
        Iterator pageModulesIterator = pageModules.iterator();
        while (pageModulesIterator.hasNext()) {
            SourceBean temp = (SourceBean)pageModulesIterator.next();
            String pageName = (String)temp.getAttribute("PAGENAME");
            String moduleName = (String)temp.getAttribute("MODULENAME");
            String pageModuleIdentifier = "PAGE[" + pageName + "]" + "MODULE[" + moduleName + "]";
            pageModuleDifferentFromDefaultConfiguration.add(pageModuleIdentifier.toUpperCase());
        } // while (pageModulesIterator.hasNext())
    } // private SecurityConfigurationSingleton()

    public static SecurityConfigurationSingleton getInstance() {
        if (_instance == null) {
            synchronized(SecurityConfigurationSingleton.class) {
                if (_instance == null)
                    _instance = new SecurityConfigurationSingleton();
            } // synchronized(SecurityConfigurationSingleton.class)
        } // if (_instance == null)
        return _instance;
    } // public static SecurityConfigurationSingleton getInstance()

    private boolean isActionDifferentFromDefault(String actionName) {
        return actionDifferentFromDefaultConfiguration.contains(actionName.toUpperCase());
    } // private boolean isActionDifferentFromDefault(String actionName)

    private boolean isPageModuleDifferentFromDefault(String pageName, String moduleName) {
        String pageModuleIdentifier = "PAGE[" + pageName + "]" + "MODULE[" + moduleName + "]";
        return pageModuleDifferentFromDefaultConfiguration.contains(pageModuleIdentifier.toUpperCase());
    } // private boolean isPageModuleDifferentFromDefault(String pageName, String moduleName)

    public boolean isActionChecked(String actionName) {
        if (isSecurityGloballyEnabled)
            if (isActionDifferentFromDefault(actionName))
                return false;
            else
                return true;
        else if (isActionDifferentFromDefault(actionName))
            return true;
        else
            return false;
    } // public boolean isActionChecked(String actionName)

    public boolean isPageModuleChecked(String pageName, String moduleName) {
        if (isSecurityGloballyEnabled)
            if (isPageModuleDifferentFromDefault(pageName, moduleName))
                return false;
            else
                return true;
        else if (isPageModuleDifferentFromDefault(pageName, moduleName))
            return true;
        else
            return false;
    } // public boolean isPageModuleChecked(String pageName, String moduleName)
} // public class SecurityConfigurationSingleton
