/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.transcoding;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;

import java.io.StringReader;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamSource;

/**
 * Questa classe effettua la transcodifica: data una stringa contente XML,
 * applica un XSL(gi processato, ovvero un'istanza di Templates) e produce
 * HTML, WML etc
 * Esempio di utilizzo della classe:
 *       Transcoder transcoder = new Transcoder();
 *       //out pu essere un'istanza di PrintWriter o OutputStream(vedere doc Xalan)
 *       Result result         = new StreamResult(out);
 *       transcoder.setXSL(template);
 *       transcoder.perform(list.perform(), result);
 *
 * @author Daniela Butano
 * @version 1.0
 */
public class Transcoder implements TranscoderIFace {

    public Transcoder() {
        super();
    } // private Transcoder()

    public void perform(SourceBean toRender, Templates xslTemplate, Result out) {
        String xml = toRender.toXML(true, false);
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "Transcoder::perform: xml\n", toRender);
        
        Source xmlSource = new StreamSource(new StringReader(xml));
        try {
            Transformer transformer = xslTemplate.newTransformer();
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "Transcoder::perform: transformer [" + transformer + "]");

            transformer.transform(xmlSource, out);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR, "Transcoder::perform: ", ex);
        } // catch (Exception ex)
    } // public void perform(SourceBean toRender, Templates xslTemplate, Result out)

} // public class Transcoder
