/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.transcoding;

import it.eng.spago.base.Constants;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;

/**
 * Factory used to create the transcoders.
 *
 */
public class TranscoderFactory {

    public static TranscoderIFace getTranscoder(ResponseContainer responseContainer) throws EMFInternalError {
        
        ConfigSingleton configure = ConfigSingleton.getInstance();
        String transcoderClass = (String)configure.getAttribute("TRANSCODERS.TRANSCODER.CLASS");
        
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "TranscoderFactory::getTranscoder: transcoderClass [" + transcoderClass + "]");
        
        TranscoderIFace transcoder = null;
        try {
            transcoder = (TranscoderIFace)Class.forName(transcoderClass).newInstance();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "TranscoderFactory::getTranscoder: classe del transcoder non valida", ex);
            throw new EMFInternalError(EMFErrorSeverity.ERROR, "classe del transcoder [" + transcoderClass + "] non valida");
        } // catch (Exception ex) try
        return transcoder;
    }
}
