/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.util;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spago.tracing.TracerSingleton;

import java.util.Iterator;

/**
 * DATE CONTRIBUTOR/DEVELOPER NOTE 13-12-2004 Butano SourceBean ora restituisce
 * ArrayList
 * 
 */

/**
 * @author
 */
public abstract class ContextScooping {

    /**
     * Retrieve a parameter from any available context.
     * @param requestContext The request context
     * @param paramConfig SourceBean containing configuration information for parameter
     * @return The value of the parameter
     */
    public static Object getScopedParameter(final RequestContextIFace requestContext,
            final SourceBean paramConfig) {
        if (requestContext == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.MINOR,
                "ContextScooping::getScopedParameter: requestContext nullo");
            return null;
        }
        if (paramConfig == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.MINOR,
                "ContextScooping::getScopedParameter: paramConfig nullo");
            return null;
        }
        String parameterName = (String) paramConfig.getAttribute("NAME");
        String parameterScope = (String) paramConfig.getAttribute("SCOPE");
        return getScopedParameter(requestContext.getRequestContainer(),
                requestContext.getResponseContainer(), parameterName,
                parameterScope, paramConfig);
    }

    /**
     * Retrieve a parameter from any available context.
     * @param requestContainer Current RequestContainer
     * @param responseContainer Current ResponseContainer
     * @param parameterName The name of the parameter
     * @param parameterScope The scope of the parameter
     * @return The value of the parameter
     */
    public static Object getScopedParameter(RequestContainer requestContainer,
            ResponseContainer responseContainer, String parameterName, String parameterScope) {
        
        return getScopedParameter(requestContainer,
                responseContainer, parameterName,
                parameterScope, null);
        
    }

    public static Object getScopedParameter(final RequestContainer requestContainer,
            final ResponseContainer responseContainer, final String parameterName,
            final String parameterScope, final SourceBean paramConfig) {
        Object parameterValue = null;
        if (parameterScope == null)
            return parameterValue;
        if (parameterScope.equalsIgnoreCase("CONFIG"))
            parameterValue = ConfigSingleton.getInstance().getAttribute(parameterName);
        else if (parameterScope.equalsIgnoreCase("USER")) {
            IEngUserProfile userProfile = (IEngUserProfile) requestContainer.getSessionContainer()
                    .getPermanentContainer().getAttribute(IEngUserProfile.ENG_USER_PROFILE);
            if (userProfile == null)
                return parameterValue;
            try {
                parameterValue = (String) userProfile.getUserAttribute(parameterName);
            } // try
            catch (EMFInternalError ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                        "ContextScooping::getScopedParameter: userProfile.getUserAttribute("
                                + parameterName + ")", (Exception) ex);
            } // catch (EntityException ex) try
        } // if (parameterScope.equalsIgnoreCase("USER"))
        else if (parameterScope.equalsIgnoreCase("ADAPTER_REQUEST"))
            parameterValue = requestContainer.getAttribute(parameterName);
        else if (parameterScope.equalsIgnoreCase(Constants.SERVICE_REQUEST))
            parameterValue = requestContainer.getServiceRequest().getAttribute(parameterName);
        else if (parameterScope.equalsIgnoreCase("SESSION"))
            parameterValue = requestContainer.getSessionContainer().getAttribute(parameterName);
        else if (parameterScope.equalsIgnoreCase("APPLICATION"))
            parameterValue = ApplicationContainer.getInstance().getAttribute(parameterName);
        else if (parameterScope.equalsIgnoreCase("ADAPTER_RESPONSE"))
            parameterValue = responseContainer.getAttribute(parameterName);
        else if (parameterScope.equalsIgnoreCase(Constants.SERVICE_RESPONSE))
            parameterValue = responseContainer.getServiceResponse().getAttribute(parameterName);
        // Modifiche Monica 16/01/2004 - inizio
        // gestione dello scope="ERROR"
        else if (parameterScope.equalsIgnoreCase("ERROR")) {
            EMFErrorHandler errorHandler = responseContainer.getErrorHandler();
            parameterValue = handleErrorScope(parameterName, errorHandler, paramConfig);
        } // else if (parameterScope.equalsIgnoreCase("ERROR"))
        // Modifiche Monica 16/01/2004 - fine
        return parameterValue;
    }

    /**
     * @param parameterName
     * @param errorHandler
     * @return true if an error with the specified conditions, exists
     */
    private static Object handleErrorScope(final String parameterName,
            final EMFErrorHandler errorHandler, final SourceBean paramConfig) {
        Object parameterValue = null;
        String severity = null;
        String category = (String) paramConfig.getAttribute("CATEGORY");
        
        // se attributo name = null si cercano tutti gli errori
        if (parameterName == null || parameterName.equals("")) {
            if (!errorHandler.isOK())
                parameterValue = Boolean.TRUE;
        } // if (parameterName == null)
        // se attributo name vale "AF_"+severity si cercano tutti gli errori
        // con tale severity
        else if (parameterName.equalsIgnoreCase("AF_INFORMATION")) {
            severity = EMFErrorSeverity.INFORMATION;
        } // if (parameterName.equalsIgnoreCase("AF_INFORMATION"))
        else if (parameterName.equalsIgnoreCase("AF_WARNING")) {
            severity = EMFErrorSeverity.WARNING;
        } // if (parameterName.equalsIgnoreCase("AF_WARNING"))
        else if (parameterName.equalsIgnoreCase("AF_ERROR")) {
            severity = EMFErrorSeverity.ERROR;
        } // if (parameterName.equalsIgnoreCase("AF_ERROR"))
        else if (parameterName.equalsIgnoreCase("AF_BLOCKING")) {
            severity = EMFErrorSeverity.BLOCKING;
        } // if (parameterName.equalsIgnoreCase("AF_BLOCKING"))
        // se la stringa contenuta nell'attributo name non corrisponde a
        // nessuna delle stringhe precedenti
        //  un codice di errore di tipo UserError
        else {
            // ciclo su Collection ottenuta da getErrors , che ritorna tutti
            // gli errori
            Iterator iterator = errorHandler.getErrors().iterator();
            while (iterator.hasNext()) {
                Object error = iterator.next();
                if (error instanceof EMFUserError) {
                    EMFUserError userError = (EMFUserError) error;
                    if (parameterName.equalsIgnoreCase(userError.getErrorCode())) {
                        parameterValue = Boolean.TRUE;
                        break;
                    } // if(parameterName.equalsIgnoreCase(String.valueOf(userError.getCode())))
                } // if(iterator.next() instanceof EMFUserError)
            } // while (iterator.hasNext())
        } // else

        // Check for severity
        if (severity != null) {
            if (!errorHandler.isOKBySeverity(severity))
                parameterValue = Boolean.TRUE;
        }
        
        // Check for category, only if previous conditions are not satisfied
        if ((parameterValue == null) && (category != null) && !category.equals("")) {
            if (!errorHandler.isOKByCategory(category))
                parameterValue = Boolean.TRUE;
        }
        
        return parameterValue;
    }
} // public abstract class ContextScooping
