/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.fieldvalidators;

import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.validation.EMFValidationError;
import it.eng.spago.validation.impl.SyntaxChecker;

import java.util.List;
import java.util.Map;

/**
 * @author zoppello
 *
 * This class checks if a field is a valid decimal number and perform additional checks on
 * the exact number of decimals after decimal separator and on the maximum number of decimal
 * after the decimal separator
 * 
 * This implementation need that this validator should be configured with parameters regarding language
 * and country, with this information the notion of "decimal separator" will depend on Locale object
 * istantiate using this information
 * 
 * For Example:
 * 
 * <FIELD-VALIDATOR fieldType="IMPORTO" fieldValidatorClass="it.eng.spago.validation.fieldvalidators.DecimalValidator">
 *		<CONFIG>
 *			<LANGUAGE language="it"/>
 *			<COUNTRY country="IT"/>
 *		</CONFIG>
 *	</FIELD-VALIDATOR>
 *
 *	If the config section the Italian Locale will be used
 *
 *  This Validator will convert the original value in a java.lang.Double object if it succeed and  
 *  this will be putted  in TYPED_SERVICE_REQUEST with an alias if this is configured in validation.xml file
 *  
 */
public class DecimalValidator extends NumericValidator {
	
    private static final long serialVersionUID = 1L;
    public static final String ERROR_MAXDECIMALS_IDENTIFIER = "ERROR_CODE_MAX_DECIMALS";
    public static final String ERROR_REQUIRED_DECIMALS_IDENTIFIER = "ERROR_CODE_REQUIRED_DECIMALS";

    public static final String ERROR_MAX_DECIMALS = "10114";
    public static final String ERROR_REQUIRED_DECIMALS = "10111";
    
    public void handleDecimals(SourceBean serviceRequest, String fieldName, String value, Map fieldValidationParams, char decimalSeparator, List params) throws EMFValidationError {

        // Si tratta di un decimale valido in accordo con il locale
        int maxDecimals = ((Integer)fieldValidationParams.get(SyntaxChecker.PARAM_MAX_DECIMALS)).intValue();
        int decimals = ((Integer)fieldValidationParams.get(SyntaxChecker.PARAM_DECIMALS)).intValue();
    
        int pos = value.indexOf(decimalSeparator);
        String cifre = "";
        if(pos!=-1)
            cifre = value.substring(pos+1);

        // Check if the number of decimals exceeds the declared maximum number
        if((decimals==-1) && (maxDecimals!=-1) && (cifre.length()>maxDecimals))
        {
            params.add(String.valueOf(maxDecimals));                            
            throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName, getParameter(ERROR_MAXDECIMALS_IDENTIFIER, ERROR_MAX_DECIMALS, fieldValidationParams), params);
        }   
        
        if ((decimals!=-1) && (cifre.length()!=decimals))
        {
            params.add(String.valueOf(decimals));   
            throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName, getParameter(ERROR_REQUIRED_DECIMALS_IDENTIFIER, ERROR_REQUIRED_DECIMALS, fieldValidationParams), params);

        }   
        
        
    }
}
