/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.impl;

import it.eng.spago.base.Constants;
import it.eng.spago.base.NullValue;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.validation.EMFValidationError;
import it.eng.spago.validation.FieldValidatorFactory;
import it.eng.spago.validation.FieldValidatorIFace;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.validator.GenericValidator;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           COntrollo su campo alfa numerico
* 										    
**/

public class SyntaxChecker 
{
	public static final String FIELD_GENERIC = "GENERIC";
	public static final String FIELD_FISCALCODE = "FISCALCODE";
	public static final String FIELD_EMAIL = "EMAIL";
	public static final String FIELD_URL = "URL";
	public static final String FIELD_DATE = "DATE";
	public static final String FIELD_NUMERIC = "NUMERIC";	
	public static final String FIELD_ALFANUMERIC = "ALFANUMERIC";
	public static final String FIELD_LETTERSTRING = "LETTERSTRING";
	public static final String FIELD_REGULAR_EXPRESSION = "REGEXP";
	public static final String FIELD_IMPORTO = "DECIMAL";
	public static final String FIELD_NAME = "NAME";
	
	public static final String FIELD_GENERIC_DEPRECATED = "1";
	public static final String FIELD_FISCALCODE_DEPRECATED = "2";
	public static final String FIELD_EMAIL_DEPRECATED = "3";
	public static final String FIELD_URL_DEPRECATED = "4";
	public static final String FIELD_DATE_DEPRECATED = "5";
	public static final String FIELD_NUMERIC_DEPRECATED = "6";	
	public static final String FIELD_ALFANUMERIC_DEPRECATED = "7";
	public static final String FIELD_LETTERSTRING_DEPRECATED = "8";
	public static final String FIELD_REGULAR_EXPRESSION_DEPRECATED = "10";	
	public static final String FIELD_IMPORTO_DEPRECATED = "11";
	public static final String FIELD_NAME_DEPRECATED = "12";
	
	
	//Codici Di Errore
	public static final String ERRORE_CAMPO_OBBLIGATORIO = "10100";
	public static final String ERRORE_CODICE_CONTROLLO = "10112";
	public static final String ERRORE_MAXLENGTH = "10117";		
	
	public static final String DEFAULT_RANGE = "";
	public static final int DEFAULT_CIFREDECIMALI = -1;
	public static final int DEFAULT_MAXCIFREDECIMALI = -1;
	
	
	//Name of the parameters passed in Map to fieldValidators	
	public static final String PARAM_REGXEP = "PARAM_REGXEP";
	public static final String PARAM_MAX_DECIMALS = "PARAM_MAX_DECIMALS";
	public static final String PARAM_DECIMALS = "PARAM_DECIMALS";
	public static final String PARAM_FIELD_ALIAS = "PARAM_FIELD_ALIAS";
	

	
    public static final String STRICT_MANDATORY_ATTRIBUTE = "STRICTMANDATORY";
    public static final String MANDATORY_ATTRIBUTE = "MANDATORY";
	public static final String DECIMALS_ATTRIBUTE = "DECIMALS";
	public static final String MAXLENGTH_ATTRIBUTE = "MAXLENGTH";
	public static final String MAX_DECIMALS_ATTRIBUTE = "MAXDECIMALS";
	public static final String REG_EXP_ATTRIBUTE = "REGEXP";
	public static final String MIN_VALUE_ATTRIBUTE = "MIN_VALUE";
    public static final String MAX_VALUE_ATTRIBUTE = "MAX_VALUE";
    public static final String ENUM_ATTRIBUTE = "ENUM";
    
	
	/**
	 * This function serves to conform previous validation syntax to the new syntax, infact
	 * in previous version field type were expressed by numbers code this function only checks if field type
	 * is written with old syntax an convert it in the string with the STRING code
	 * @param fieldType The field type that arrive to check() method
	 * @return the eventually converted fieldType
	 */
	private static String retroCompatibilityConversion(String fieldType){
		if (fieldType.equalsIgnoreCase(FIELD_FISCALCODE_DEPRECATED)){
			return FIELD_FISCALCODE;
		}else if (fieldType.equalsIgnoreCase(FIELD_EMAIL_DEPRECATED)){
			return FIELD_EMAIL;
		}else if (fieldType.equalsIgnoreCase(FIELD_URL_DEPRECATED)){
				return FIELD_URL;
		}else if (fieldType.equalsIgnoreCase(FIELD_DATE_DEPRECATED)){
			return FIELD_DATE;
		}else if (fieldType.equalsIgnoreCase(FIELD_NUMERIC_DEPRECATED)){	
			return FIELD_NUMERIC;
		}else if (fieldType.equalsIgnoreCase(FIELD_ALFANUMERIC_DEPRECATED)){
			return FIELD_ALFANUMERIC;
		}else if (fieldType.equalsIgnoreCase(FIELD_LETTERSTRING_DEPRECATED)){
			return FIELD_LETTERSTRING;
		}else if (fieldType.equalsIgnoreCase(FIELD_REGULAR_EXPRESSION_DEPRECATED)){
			return FIELD_REGULAR_EXPRESSION;
		}else if (fieldType.equalsIgnoreCase(FIELD_IMPORTO_DEPRECATED)){
			return FIELD_IMPORTO;
		}else if (fieldType.equalsIgnoreCase(FIELD_NAME_DEPRECATED)){
			return FIELD_NAME;
		}else if (fieldType.equalsIgnoreCase(FIELD_GENERIC_DEPRECATED)){
			return FIELD_GENERIC;
		}else{
			return fieldType;
		}
	}
    
    /**
     * Searches for the validation attribute in different scopes: first in the field validation params (validation.xml),
     * then in field validator params (fieldvalidators.xml)
     * @param attributeName Name of validation attribute
     * @param fieldValidationParams Attributes of validation.xml
     * @param validatorConfig Configuration of field validator
     * @return The value of validation attribute
     */
    private static Object getValidationAttribute(final String attributeName, final Map fieldValidationParams, final SourceBean validatorConfig) {
        // Try first searching in the field configuration
        Object result = fieldValidationParams.get(attributeName);
        
        if ((result == null) && (validatorConfig != null)) {
            // If not found, search in the configuration of validator
            result = validatorConfig.getAttribute(attributeName);
        }
        
        return result;
    }
	/**
	 * Method check. Verifica la validit di un campo di input.
	 * @param mandatory true se  il campo  obbligatorio, false altrimenti.
	 * @param tipoCampo tipo del campo, deve essere uno dei valori CAMPO_CODICE_FISCALE..CAMPO_DATA
	 * @param value valore da verificare
	 * @return boolean true se il campo supera i controlli formali, false altrimenti
	 */
	public static boolean check(RequestContextIFace requestContext,
					String fieldType, 
					String value,
					String fieldName,
					String fieldAliasAfterValidation,
					Map fieldValidationParams){
        
        SourceBean serviceRequest = requestContext.getServiceRequest();
        EMFErrorHandler errorHandler = requestContext.getErrorHandler();
        
        // Converti i tipi deprecati nei tipi normal
        fieldType = retroCompatibilityConversion(fieldType);

        FieldValidatorIFace fieldValidator = (FieldValidatorIFace) FieldValidatorFactory.getFieldValidator(fieldType);
        SourceBean validatorConfig = null;
        if (fieldValidator != null) {
            validatorConfig = fieldValidator.getConfig();
        } else {
            List errorParams = new ArrayList(1);
            errorParams.add(String.valueOf(fieldType));
            errorHandler.addError(new EMFValidationError(EMFErrorSeverity.ERROR, fieldName,ERRORE_CODICE_CONTROLLO, errorParams));
            return false;
        }
        
        String mandatory = (String) getValidationAttribute(MANDATORY_ATTRIBUTE, fieldValidationParams, validatorConfig);
        String strictMandatory = (String) getValidationAttribute(STRICT_MANDATORY_ATTRIBUTE, fieldValidationParams, validatorConfig);
        boolean bMandatory = (mandatory != null && mandatory.equalsIgnoreCase("TRUE"));
        boolean bStrictMandatory = (strictMandatory != null && strictMandatory.equalsIgnoreCase("TRUE"));
        String cifreDecimali = (String) getValidationAttribute(DECIMALS_ATTRIBUTE, fieldValidationParams, validatorConfig);

		int nCifreDecimali = (cifreDecimali != null && !cifreDecimali
				.equals("")) ? Integer.parseInt(cifreDecimali)
				: -1;

		String maxLength = (String) getValidationAttribute(MAXLENGTH_ATTRIBUTE, fieldValidationParams, validatorConfig);

		int nMaxLength = (maxLength != null && !maxLength
				.equals("")) ? Integer.parseInt(maxLength) : -1;

		String maxCifreDecimali = (String) getValidationAttribute(MAX_DECIMALS_ATTRIBUTE, fieldValidationParams, validatorConfig);
		String regularExpression = (String) getValidationAttribute(REG_EXP_ATTRIBUTE, fieldValidationParams, validatorConfig);

		int nMaxCifreDecimali = (maxCifreDecimali != null && !maxCifreDecimali
				.equals("")) ? Integer
				.parseInt(maxCifreDecimali) : -1;
				
		TracerSingleton.log(Constants.NOME_MODULO,
				TracerSingleton.INFORMATION, "bMandatory="
						+ mandatory + ", tipoControllo="
						+ fieldType + ", value="
						+ value + ", nMaxLength="
						+ nMaxLength + ", fieldAliasAfterValidation="
						+ fieldAliasAfterValidation);
		boolean risultato = true;
		
        // Preparo la lista dei parametri in caso d'errore
        List params = new ArrayList(2);
        params.add(fieldName);
        params.add(getUserFieldName(fieldName, requestContext));

        // Controllo campo obbligatorio: questo controllo viene effettuato anche se il campo non
        // arriva nel form
        if (bStrictMandatory && GenericValidator.isBlankOrNull(value))
        {
            errorHandler.addError(new EMFValidationError(EMFErrorSeverity.ERROR, fieldName,ERRORE_CAMPO_OBBLIGATORIO, params)); 
            risultato = false;
        }
        
        // Controllo campo obbligatorio: questo controllo viene effettuato solo se il campo
        // arriva nel form
		if (bMandatory && (value != null) && (value.length() == 0))
		{
			errorHandler.addError(new EMFValidationError(EMFErrorSeverity.ERROR, fieldName,ERRORE_CAMPO_OBBLIGATORIO, params));	
			risultato = false;
		}

		//Controllo maxlength
		if (risultato && nMaxLength!=-1  
			&& (value!=null && value.length()>nMaxLength) )
		{
			errorHandler.addError(new EMFValidationError(EMFErrorSeverity.ERROR, fieldName,ERRORE_MAXLENGTH, params));
			risultato = false;
		}

		if (risultato)	
		{
            if (fieldValidator instanceof RequestContextIFace) {       
                ((RequestContextIFace)fieldValidator).setRequestContext(requestContext); 
            }
            
            // Rimappo o tipizzo alcuni attributi
            fieldValidationParams.put(PARAM_REGXEP,regularExpression);
            fieldValidationParams.put(PARAM_MAX_DECIMALS, new Integer(nMaxCifreDecimali));
            fieldValidationParams.put(PARAM_DECIMALS, new Integer(nCifreDecimali));
            
            if (fieldAliasAfterValidation == null) {
            	// If not alias specified, use original name for typed value
            	fieldAliasAfterValidation = fieldName;
            }
            fieldValidationParams.put(PARAM_FIELD_ALIAS, fieldAliasAfterValidation);

            if (!GenericValidator.isBlankOrNull(value))    
            {
                try{
                    fieldValidator.validateField(serviceRequest, fieldName, value, fieldValidationParams);
                }catch(EMFValidationError emfve){
                    risultato = false;
                    errorHandler.addError(emfve);
                }
            } else if (value == null) {
                // Handle default value
                try{
                    fieldValidator.setDefault(serviceRequest, fieldName, fieldValidationParams);
                } catch(EMFValidationError emfve) {
                    risultato = false;
                    errorHandler.addError(emfve);
                }
            } 
            else { /* value != null but it's the empty string */
                // Handle default value
                try{
                    if (!fieldValidator.setDefault(serviceRequest, fieldName, fieldValidationParams)) {
                    	// Set the value to null, no default configured
                    	ValidationServices.getTypedServiceRequest(serviceRequest).setAttribute(fieldAliasAfterValidation, new NullValue());
                    }
                } catch(EMFValidationError emfve) {
                    risultato = false;
                    errorHandler.addError(emfve);
                } catch(Exception ex) {
                	risultato = false;
                	errorHandler.addError(new EMFInternalError(EMFErrorSeverity.ERROR, ex));
                }
            }
			
		}
		
		return risultato;
	}	

    /**
     * Return the field name used in error messages.
     * @param fieldName Name of the field in the form.
     * @return Field name used in error messages.
     */
    public static String getUserFieldName(final String fieldName, final RequestContextIFace requestContext) {
        return MessageBundle.getServiceMessage(fieldName, requestContext.getResponseContainer());
    }
    
}
