/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.distribute;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.coordinator.AbstractCoordinator;
import it.eng.spago.dispatching.ejbchannel.SessionFacadeEJBLocalProxy;
import it.eng.spago.tracing.TracerSingleton;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

public class DistributeCoordinator extends AbstractCoordinator {

    private static final long serialVersionUID = 1L;

    private static final String SESSION_FACADE_EJB_PROXY = "SESSION_FACADE_EJB_PROXY";

    public DistributeCoordinator(final String businessType, final String businessName,
            final String publisherName) {
        super(businessType, businessName, publisherName);
    } // public DistributeCoordinator(String businessType, String
        // businessName)

    public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "DistributeCoordinator::service: invocato");
        Monitor monitor = MonitorFactory.start("controller.coordinator.distribute");
        try {
            SessionContainer sessionContainer = getRequestContainer().getSessionContainer();
            SessionFacadeEJBLocalProxy sessionFacadeEJBProxy = (SessionFacadeEJBLocalProxy) sessionContainer
                    .getAttribute(SESSION_FACADE_EJB_PROXY);
            if (sessionFacadeEJBProxy == null) {
                synchronized (sessionContainer) {
                    sessionFacadeEJBProxy = (SessionFacadeEJBLocalProxy) sessionContainer
                            .getAttribute(SESSION_FACADE_EJB_PROXY);
                    if (sessionFacadeEJBProxy == null) {
                        sessionFacadeEJBProxy = new SessionFacadeEJBLocalProxy(null);
                        
                        // Store a proxy to the EJB in session, to avoid a lookup if
                        // the EJB is used more than once
                        sessionContainer.setAttribute(SESSION_FACADE_EJB_PROXY,
                                sessionFacadeEJBProxy);
                    } // if (sessionFacadeEJBProxy == null)
                } // synchronized (sessionContainer)
            } // if (sessionFacadeEJBProxy == null)
            synchronized (sessionFacadeEJBProxy) {
                sessionFacadeEJBProxy.service(this);
            } // synchronized (sessionFacadeEJBProxy)
        } // try
        catch (Exception ex) {
            throw ex;
        } // catch (Exception ex)
        finally {
            monitor.stop();
        } // finally
    } // public void service(SourceBean serviceRequest, SourceBean
        // serviceResponse) throws Exception
} // public class DistributeCoordinator extends AbstractCoordinator
