/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.distribute;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.coordinator.AbstractCoordinator;
import it.eng.spago.dispatching.ejbchannel.RemoteSessionFacadeEJBProxy;
import it.eng.spago.tracing.TracerSingleton;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

public class RemoteDistributeCoordinator extends AbstractCoordinator {

    private static final long serialVersionUID = 1L;
    private static final String REMOTE_SESSION_FACADE_EJB_PROXY =
        "REMOTE_SESSION_FACADE_EJB_PROXY";
    /**
     * Costruttore 
     * @param businessType
     * @param businessName
     */
    public RemoteDistributeCoordinator(final String businessType, final String businessName, final String publisherName) {
        super(businessType, businessName, publisherName);
    } 

    /**
     * Metodo che esegue il servizio sull'EJB remoto utilizzando
     * la classe RemoteSessionFacadeEJBProxy
     * @param serviceRequest SB di richiesta
     * @param serviceResponse SB di risposta 
     */
    public void service(SourceBean serviceRequest, SourceBean serviceResponse)
        throws Exception {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "RemoteDistributeCoordinator::service: invocato");
        Monitor monitor =
            MonitorFactory.start("controller.coordinator.remotedistribute");
        try {
            SessionContainer sessionContainer =
                getRequestContainer().getSessionContainer();
            RemoteSessionFacadeEJBProxy eJBProxy =
                (RemoteSessionFacadeEJBProxy) sessionContainer.getAttribute(
                		REMOTE_SESSION_FACADE_EJB_PROXY);
            if (eJBProxy == null) {
                synchronized (sessionContainer) {
                	eJBProxy =
                        (RemoteSessionFacadeEJBProxy)sessionContainer.getAttribute(REMOTE_SESSION_FACADE_EJB_PROXY);
                    if (eJBProxy == null) {
                    	eJBProxy =
                            new RemoteSessionFacadeEJBProxy();
                        sessionContainer.setAttribute(
                        		REMOTE_SESSION_FACADE_EJB_PROXY,
								eJBProxy);
                    } 
                } 
            } 
            synchronized (eJBProxy) {
            	eJBProxy.service(this);
            } 
        } 
        catch (Exception ex) {
            throw ex;
        } 
        finally {
            monitor.stop();
        } 
    } 
} 
