/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.ejbchannel;

import java.io.Serializable;
import java.util.Hashtable;

import javax.ejb.Handle;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

public class AdapterEJBProxy implements Serializable {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private String _jndi = null;
    private transient AdapterEJB _adapterEJB = null;
    private Handle _adapterEJBHandle = null;
    private Hashtable _environment = null;

    public AdapterEJBProxy() {
        _jndi = "ejb/spago/AdapterEJBHome";
    } // public AdapterEJBProxy()

    public AdapterEJBProxy(Hashtable environment) {
        this();
        _environment = environment;
    } // public AdapterEJBProxy(Hashtable environment)

    public void setJndi(String jndi) {
        _jndi = jndi;
    } // public void setJndi(String jndi)

    public String getJndi() {
        return _jndi;
    } // public String getJndi()

    private AdapterEJB getRemote() throws Exception {
        if (_adapterEJB == null)
            if (_adapterEJBHandle != null)
                _adapterEJB = (AdapterEJB) _adapterEJBHandle.getEJBObject();
            else if (_jndi != null) {
                Context ctx = (_environment == null) ? new InitialContext() : new InitialContext(_environment);
                AdapterEJBHome adapterEJBHome =
                    (AdapterEJBHome) PortableRemoteObject.narrow(
                        ctx.lookup(_jndi),
                        AdapterEJBHome.class);
                _adapterEJB = adapterEJBHome.create();
                _adapterEJBHandle = _adapterEJB.getHandle();
            } // if (_jndi != null)
        return _adapterEJB;
    } // private AdapterEJB getRemote() throws Exception

    public String service(String request) throws Exception {
        AdapterEJB adapterEJB = getRemote();
        if (adapterEJB == null)
            throw new Exception("A JNDI must be specified via AdapterEJBProxy.setJndi(jndi).");
        return adapterEJB.service(request);
    } // public String service(String request)
} // public class AdapterEJBProxy implements Serializable
