/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.ejbchannel;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.dispatching.coordinator.CoordinatorIFace;
import it.eng.spago.dispatching.coordinator.DispatcherManager;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.tracing.TracerSingleton;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.transaction.SystemException;
import javax.transaction.UserTransaction;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

public class RemoteSessionFacadeEJBBean implements
		RemoteSessionFacadeEJBBusiness, SessionBean {

	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    private SessionContext _ejbContext = null;
	/**
	 * Costruttore statico per la configurazione N.B. Attualmente prende il
	 * riferimento al PATH per la directory di configurazione dalle variabili di
	 * systema
	 */
	/*static {
		InitializerManager.init();
	}*/

	/**
	 * Metodo che esegue il servizio e restituisce il request context
	 * @param requestContext
	 *            Request context
	 */
	public RequestContextIFace service(RequestContextIFace requestContext) {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"RemoteSessionFacadeEJBBean::service: invocato");
		Monitor monitor = MonitorFactory
				.start("controller.adapter.ejb.remote.session-facade");
		RequestContainer requestContainer = requestContext
				.getRequestContainer();
		ResponseContainer responseContainer = requestContext
				.getResponseContainer();
		CoordinatorIFace coordinator = null;
		UserTransaction userTx = null;
		try {
			requestContainer.setRequestDistributed(true);
			coordinator = DispatcherManager.getCoordinator(requestContext);
			((RequestContextIFace) coordinator)
					.setRequestContext(requestContext);
			userTx = _ejbContext.getUserTransaction();
			RequestContainer.setRequestContainer(requestContainer);
			userTx.begin();
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
					"RemoteSessionFacadeEJBBean::service: begin transaction");
			coordinator.service(requestContainer.getServiceRequest(),
					responseContainer.getServiceResponse());
			if (requestContext.getErrorHandler().isOKBySeverity(
					EMFErrorSeverity.BLOCKING)) {
				userTx.commit();
				TracerSingleton
						.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
								"RemoteSessionFacadeEJBBean::service: commit transaction");
			} else {
				userTx.rollback();
				TracerSingleton
						.log(
								Constants.NOME_MODULO,
								TracerSingleton.CRITICAL,
								"RemoteSessionFacadeEJBBean::service: errore BLOCKING nel servizio, rolling back transaction !");
			}
		} catch (Exception ex) {
			if (userTx != null) {
				try {
					userTx.rollback();
				} catch (IllegalStateException e) {
					e.printStackTrace();
				} catch (SecurityException e) {
					e.printStackTrace();
				} catch (SystemException e) {
					e.printStackTrace();
				}
				TracerSingleton
						.log(
								Constants.NOME_MODULO,
								TracerSingleton.CRITICAL,
								"RemoteSessionFacadeEJBBean::service: errore nel servizio, rolling back transaction !");
			}
		} finally {
			((RequestContextIFace) coordinator).setRequestContext(null);
			requestContainer.setRequestDistributed(false);
			monitor.stop();
		}

		return requestContext;
	}

	/**
	 * @return Returns the _ejbContext.
	 */
	public SessionContext getSessionContext() {
		return _ejbContext;
	}

	/**
	 * @param context
	 *            The _ejbContext to set.
	 */
	public void setSessionContext(SessionContext context) {
		_ejbContext = context;
	}

	public void ejbActivate() {
	}

	public void ejbCreate() throws CreateException {

	}

	public void ejbPassivate() {
	}

	public void ejbRemove() {
	}
}
