/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.ejbchannel;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.tracing.TracerSingleton;

import java.io.Serializable;

import javax.naming.Context;
import javax.naming.InitialContext;

public class SessionFacadeEJBLocalProxy implements Serializable {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private String _jndi = null;

    public SessionFacadeEJBLocalProxy(String jndi) {
        _jndi = jndi;
        if (_jndi == null) {
            ConfigSingleton configure = ConfigSingleton.getInstance();
            SourceBean proxy =
                (SourceBean) configure.getFilteredSourceBeanAttribute(
                    "PROXIES.PROXY",
                    "NAME",
                    "SESSION_FACADE_EJB_LOCAL_PROXY");
            if (proxy != null)
                _jndi = (String) proxy.getAttribute("jndi");
        } // if (_jndi == null)
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "SessionFacadeEJBLocalProxy::SessionFacadeEJBLocalProxy: _jndi ["
                + _jndi
                + "]");
    } // public SessionFacadeEJBLocalProxy(String jndi)

    public void service(RequestContextIFace request) throws Exception {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "SessionFacadeEJBLocalProxy::service: invocato");
        SessionFacadeEJBLocal sessionFacadeEJBLocal = null;
        try {
            Context ctx = new InitialContext();
            SessionFacadeEJBLocalHome sessionFacadeEJBLocalHome =
                (SessionFacadeEJBLocalHome) ctx.lookup(_jndi);
            sessionFacadeEJBLocal = sessionFacadeEJBLocalHome.create();
            sessionFacadeEJBLocal.service(request);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "SessionFacadeEJBLocalProxy::service:",
                ex);
        } // catch (Exception ex)
        finally {
            if (sessionFacadeEJBLocal != null)
                sessionFacadeEJBLocal.remove();
        } // finally
    } // public void service(RequestContextIFace request) throws Exception
} // public class SessionFacadeEJBLocalProxy implements Serializable
