/**

Copyright 2004 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

This file is part of Spago.

Spago is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
any later version.

Spago is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Spago; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**/
package it.eng.spago.dispatching.jmschannel;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.coordinator.CoordinatorIFace;
import it.eng.spago.dispatching.coordinator.DispatcherManager;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.presentation.PresentationRendering;
import it.eng.spago.presentation.Publisher;
import it.eng.spago.tracing.TracerSingleton;

import java.util.ArrayList;

import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSession;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.transaction.UserTransaction;


public class JmsAdapterMDB implements MessageDrivenBean, MessageListener {

    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    private MessageDrivenContext context = null;

    private QueueConnection conn = null;

    private QueueSession session = null;

	/**
	 * 
	 */
	public JmsAdapterMDB() {
		super();

	}

	public void ejbRemove() throws EJBException {

		context = null;
        try {
            if (session != null)
                session.close();
            if (conn != null)
                conn.close();
        } catch (JMSException e) {
            
        }
	}

	/**
	 * Set the associated context. The container calls this method 
	 * after the instance creation.  <br>
	 * 
	 * The enterprise bean instance should store the reference to the context 
	 * object in an instance variable. <br>
	 * 
	 * This method is called with no transaction context.
	 * 
	 * @param newContext A MessageDrivenContext interface for the instance. 
	 * 
	 * @throws EJBException Thrown by the method to indicate a failure caused by a system-level error.
	 */
	public void setMessageDrivenContext(MessageDrivenContext newContext)
		throws EJBException {
		context = newContext;
	}


	public void onMessage(Message message) {
	    // read xml from Message
		if ( message instanceof TextMessage){
			
			TextMessage txtMessage = (TextMessage) message;
	        try {
	        	// Create new Source Bean with request XML
				SourceBean serviceRequest = SourceBean.fromXMLString(txtMessage.getText());
				
	            RequestContainer requestContainer = new RequestContainer();
	            requestContainer.setSessionContainer(new SessionContainer(true));
	            requestContainer.setChannelType(Constants.JMS_CHANNEL);
	            requestContainer.setServiceRequest(serviceRequest);  
	            
	            ResponseContainer responseContainer = new ResponseContainer();
	            EMFErrorHandler emfErrorHandler = new EMFErrorHandler();
	            responseContainer.setErrorHandler(emfErrorHandler);
	            
	            // Create new response Source Bean
	            SourceBean serviceResponse = new SourceBean(Constants.SERVICE_RESPONSE);
	            responseContainer.setServiceResponse(serviceResponse);
	            
	            RequestContextIFace requestContext =
	                new DefaultRequestContext(requestContainer, responseContainer);

	            requestContainer.setRequestDistributed(true);
	            CoordinatorIFace coordinator =
	                DispatcherManager.getCoordinator(requestContext);
	            Exception serviceException = null;            
	            if (coordinator == null) {
	                serviceException = new Exception("Coordinatore non trovato");
	                emfErrorHandler.addError(
	                    new EMFInternalError(
	                        EMFErrorSeverity.ERROR,
	                        "Coordinatore non trovato !"));
	            } else {
	                ((RequestContextIFace) coordinator).setRequestContext(
	                        requestContext);
                    responseContainer.setBusinessType(coordinator.getBusinessType());
                    responseContainer.setBusinessName(coordinator.getBusinessName());
                    responseContainer.setPublisherName(coordinator.getPublisherName());

	                UserTransaction userTx = context.getUserTransaction();
	                    RequestContainer.setRequestContainer(requestContainer);
	                    try {
	                        //
	                    	userTx.begin();
	                        // execute Business Code
	                        coordinator.service(serviceRequest, serviceResponse);
	                        if (emfErrorHandler.isOKBySeverity(EMFErrorSeverity.BLOCKING)) {
	                            userTx.commit();
	                        }else {
	                            userTx.rollback();
	                        } 
	                    } 
	                    catch (Exception ex) {
	                    	TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,ex.getMessage(),ex);

	                    	userTx.rollback();
	                        serviceException = ex;
	                        emfErrorHandler.addError(
	                            new EMFInternalError(EMFErrorSeverity.ERROR, ex));
	                    }
	                    RequestContainer.delRequestContainer();
	                    ((RequestContextIFace) coordinator).setRequestContext(null);
	            }
	            
	            requestContainer.setRequestDistributed(false);
	            // retrive the publisher, if exists
	            String publisherType = null;
	            ArrayList resources = null;
	            Publisher publisher =
	                Publisher.getPublisher(requestContext,
	                    serviceException);
	            if (publisher == null) {
	                publisherType = Constants.NOTHING_PUBLISHER_TYPE;
	                resources = null;
	            } 
	            else {
	                publisherType = publisher.getType();
	                if ((publisherType == null)
	                    || (publisherType.equalsIgnoreCase(Constants.NOTHING_PUBLISHER_TYPE))) {
	                    publisherType = Constants.NOTHING_PUBLISHER_TYPE;
	                    resources = null;
	                } 
	                else {
	                    resources = publisher.getResources();
	                    if ((resources == null) || (resources.size() == 0)) {
	                        publisherType = Constants.NOTHING_PUBLISHER_TYPE;
	                        resources = null;
	                    } 
	                }
	            }

	            try {
                    String response =
	                    PresentationRendering.render(responseContainer, resources);
                    TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.DEBUG,response);
	            } 
	            catch (Exception ex) {
	            	TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,ex.getMessage(),ex);
	            	throw ex;
	            }
	         
			} catch (Exception e) {
				TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,e.getMessage(),e);
			} 	
			
		}else {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"THE ADAPTER ACEPT ONLY TextMessage");
		}
	}

	/**
	 * An ejbCreate method as required by the EJB specification.
	 * 
	 * The container calls the instance?s <code>ejbCreate</code> method 
	 * immediately after instantiation.
	 * 
	 * @ejb.create-method
	 */
	public void ejbCreate() {
        
        
        try {
            InitialContext iniCtx = new InitialContext();
            Object tmp = iniCtx.lookup("java:comp/env/jms/QCF");
            QueueConnectionFactory qcf = (QueueConnectionFactory) tmp;
            conn = qcf.createQueueConnection();
            session = conn.createQueueSession(false, QueueSession.AUTO_ACKNOWLEDGE);
            conn.start();
        } catch (Exception e) {
            
            throw new EJBException("Failed to init JmsAdapterMDB", e);
        }		
	}

}