/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.configuration;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.init.InitializerManager;

import java.io.InputStream;
import java.util.Enumeration;
import java.util.Properties;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import org.xml.sax.InputSource;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 07-12-2004		  Butano	   Output sulla console dipende da STARTUP_CONSOLE
* 13-12-2004              Butano   Se afRootPath == null chiamta a getResourcesAsStream()										    
**/

/**
 * @author 
 */
public class ConfigServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    /**
     * Oggetto che permette di leggere la configurazione tramite il contesto della servlet, indispensabile se
     * l'applicazione viene rilasciata sotto forma di WAR che non viene espanso.
     */
    class StreamCreatorConfiguration implements IConfigurationCreator {
        
        public SourceBean createConfiguration(String configName) throws SourceBeanException {
            ServletContext context = getServletConfig().getServletContext();            
            InputStream resourcesStream = context.getResourceAsStream(configName);
            return SourceBean.fromXMLStream(new InputSource(resourcesStream));
        }

    }
        
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        String afRootPath = config.getInitParameter("AF_ROOT_PATH");   
        String afConfigFile = config.getInitParameter("AF_CONFIG_FILE");
        ConfigSingleton.setConfigFileName(afConfigFile);
        
        if (afRootPath == null) {
            ConfigSingleton.setConfigurationCreation(new StreamCreatorConfiguration());
        	ConfigSingleton.setRootPath(config.getServletContext().getRealPath(""));
        } 
        else {
            ConfigSingleton.setConfigurationCreation(new FileCreatorConfiguration());
            ConfigSingleton.setRootPath(afRootPath);
        }
        
        //Print system properties
        String startupConsoleStr =
            (String) ConfigSingleton.getInstance().getAttribute("COMMON.startup_console");                       
        if ((startupConsoleStr == null)
                || (!startupConsoleStr.equalsIgnoreCase("FALSE"))) {
	        System.out.println("ConfigServlet::init: AF_ROOT_PATH = " + afRootPath); 
	        System.out.println("ConfigServlet::init: AF_CONFIG_FILE = " + afConfigFile);        
	        Properties props = System.getProperties();
	        Enumeration propNames = props.propertyNames();
	        while (propNames.hasMoreElements()) {
	            String name = (String)propNames.nextElement();
	            System.out.println("ConfigServlet::init: " + name + " [" + System.getProperty(name) + "]");
	        } 
        } //end Print system properties  
        
        InitializerManager.init();
    } // public void init(ServletConfig config) throws ServletException
    
} // public class ConfigServlet extends HttpServlet
