/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.httpchannel;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.tracing.TracerSingleton;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;

public abstract class AbstractBaseServlet
    extends HttpServlet{
   
    public RequestContainer getRequestContainer(HttpServletRequest request) {
        RequestContainer requestContainer =
            RequestContainerAccess.getRequestContainer(request);
        if (requestContainer == null)
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "AbstractBaseServlet::getRequestContainer: requestContainer nullo");
        return requestContainer;
    } // public RequestContainer getRequestContainer(HttpServletRequest request)

    public SourceBean getServiceRequest(HttpServletRequest request) {
        RequestContainer requestContainer = getRequestContainer(request);
        if (requestContainer == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "AbstractBaseServlet::getServiceRequest: requestContainer nullo");
            return null;
        } // if (requestContainer == null)
        SourceBean serviceRequest = requestContainer.getServiceRequest();
        if (serviceRequest == null)
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "AbstractBaseServlet::getServiceRequest: serviceRequest nullo");
        return serviceRequest;
    } // public SourceBean getServiceRequest(HttpServletRequest request)

    public ResponseContainer getResponseContainer(HttpServletRequest request) {
        ResponseContainer responseContainer =
            ResponseContainerAccess.getResponseContainer(request);
        if (responseContainer == null)
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "AbstractBaseServlet::getResponseContainer: responseContainer nullo");
        return responseContainer;
    } // public ResponseContainer getResponseContainer(HttpServletRequest request)

    public SourceBean getServiceResponse(HttpServletRequest request) {
        ResponseContainer responseContainer = getResponseContainer(request);
        if (responseContainer == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "AbstractHttpJspPage::getServiceResponse: responseContainer nullo");
            return null;
        } // if (responseContainer == null)
        SourceBean serviceResponse = responseContainer.getServiceResponse();
        if (serviceResponse == null)
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "AbstractBaseServlet::getServiceResponse: serviceResponse nullo");
        return serviceResponse;
    } // public SourceBean getServiceResponse(HttpServletRequest request)

    public EMFErrorHandler getErrorHandler(HttpServletRequest request) {
        ResponseContainer responseContainer = getResponseContainer(request);
        if (responseContainer == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "AbstractBaseServlet::getErrorHandler: responseContainer nullo");
            return null;
        } // if (responseContainer == null)
        EMFErrorHandler errorHandler = responseContainer.getErrorHandler();
        if (errorHandler == null)
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "AbstractBaseServlet::getErrorHandler: errorHandler nullo");
        return errorHandler;
    } // public EMFErrorHandler getErrorHandler(HttpServletRequest request)
} // public abstract AbstractHttpJspPage extends HttpServlet implements HttpJspPage
