/**

    Copyright 2004 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**/
package it.eng.spago.dispatching.httpchannel;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.cache.CacheResponseHandler;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.action.NavigationErrorUtility;
import it.eng.spago.dispatching.action.SessionExpiredUtility;
import it.eng.spago.dispatching.coordinator.CoordinatorIFace;
import it.eng.spago.dispatching.coordinator.DispatcherManager;
import it.eng.spago.dispatching.httpchannel.upload.IUploadHandler;
import it.eng.spago.dispatching.httpchannel.upload.UploadFactory;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.navigation.NavigationException;
import it.eng.spago.navigation.Navigator;
import it.eng.spago.presentation.PresentationRendering;
import it.eng.spago.presentation.Publisher;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.Serializer;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.fileupload.servlet.ServletRequestContext;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

public class AdapterHTTP extends HttpServlet {

    private static final long serialVersionUID = 1L;

    public static final String NEW_SESSION = "NEW_SESSION";

    public static final String REQUEST_CONTAINER = "REQUEST_CONTAINER";

    public static final String HTTP_CONTENT_TYPE = "text/html";

    public static final String WAP_CONTENT_TYPE = "text/vnd.wap.wml";

    public static final String XML_CONTENT_TYPE = "text/xml";

    public static final String HTTP_REQUEST_AUTH_TYPE = "HTTP_REQUEST_AUTH_TYPE";

    public static final String HTTP_REQUEST_CHARACTER_ENCODING = "HTTP_REQUEST_CHARACTER_ENCODING";

    public static final String HTTP_REQUEST_CONTENT_LENGTH = "HTTP_REQUEST_CONTENT_LENGTH";

    public static final String HTTP_REQUEST_CONTENT_TYPE = "HTTP_REQUEST_CONTENT_TYPE";

    public static final String HTTP_REQUEST_CONTEXT_PATH = "HTTP_REQUEST_CONTEXT_PATH";

    public static final String HTTP_REQUEST_METHOD = "HTTP_REQUEST_METHOD";

    public static final String HTTP_REQUEST_PATH_INFO = "HTTP_REQUEST_PATH_INFO";

    public static final String HTTP_REQUEST_PATH_TRANSLATED = "HTTP_REQUEST_PATH_TRANSLATED";

    public static final String HTTP_REQUEST_PROTOCOL = "HTTP_REQUEST_PROTOCOL";

    public static final String HTTP_REQUEST_QUERY_STRING = "HTTP_REQUEST_QUERY_STRING";

    public static final String HTTP_REQUEST_REMOTE_ADDR = "HTTP_REQUEST_REMOTE_ADDR";

    public static final String HTTP_REQUEST_REMOTE_HOST = "HTTP_REQUEST_REMOTE_HOST";

    public static final String HTTP_REQUEST_REMOTE_USER = "HTTP_REQUEST_REMOTE_USER";

    public static final String HTTP_REQUEST_REQUESTED_SESSION_ID = "HTTP_REQUEST_REQUESTED_SESSION_ID";

    public static final String HTTP_REQUEST_REQUEST_URI = "HTTP_REQUEST_REQUEST_URI";

    public static final String HTTP_REQUEST_SCHEME = "HTTP_REQUEST_SCHEME";

    public static final String HTTP_REQUEST_SERVER_NAME = "HTTP_REQUEST_SERVER_NAME";

    public static final String HTTP_REQUEST_SERVER_PORT = "HTTP_REQUEST_SERVER_PORT";

    public static final String HTTP_REQUEST_SERVLET_PATH = "HTTP_REQUEST_SERVLET_PATH";

    public static final String HTTP_REQUEST_USER_PRINCIPAL = "HTTP_REQUEST_USER_PRINCIPAL";

    public static final String HTTP_REQUEST_REQUESTED_SESSION_ID_FROM_COOKIE = "HTTP_REQUEST_REQUESTED_SESSION_ID_FROM_COOKIE";

    public static final String HTTP_REQUEST_REQUESTED_SESSION_ID_FROM_URL = "HTTP_REQUEST_REQUESTED_SESSION_ID_FROM_URL";

    public static final String HTTP_REQUEST_REQUESTED_SESSION_ID_VALID = "HTTP_REQUEST_REQUESTED_SESSION_ID_VALID";

    public static final String HTTP_REQUEST_SECURE = "HTTP_REQUEST_SECURE";

    public static final String HTTP_ACCEPT_HEADER = "ACCEPT";

    public static final String WAP_MIME_TYPE = "vnd.wap";

    public static final String HTTP_SESSION_ID = "HTTP_SESSION_ID";
    
    public static final String HTTP_REQUESTED_WITH = "x-requested-with";
        
    public static final String XMLHTTPREQUEST = "XMLHttpRequest";

    private static final String SERVLET_PUBLISHER_TYPE = "SERVLET";

    private static final String JSP_PUBLISHER_TYPE = "JSP";

    private static final String FORWARD_PUBLISHING_MODE = "FORWARD";

    private static final String SENDREDIRECT_PUBLISHING_MODE = "SENDREDIRECT";

    private static final String SERIALIZE_SESSION_ATTRIBUTE = "COMMON.SERIALIZE_SESSION";

    // Atributo della configurazione che indica se serializzare il contenuto
    // della sessione
    private boolean serializeSession = false;

    
    private void handleQueryStringField(HttpServletRequest request,SourceBean serviceReq,String queryStringFieldName) throws SourceBeanException {
    	
    	String queryString = queryStringFieldName.substring(queryStringFieldName.indexOf("{")+1, queryStringFieldName.indexOf("}"));
    	
    	StringTokenizer st = new StringTokenizer(queryString, "&", false);
    	
    	String parameterToken = null;
    	String parameterName = null;
    	String parameterValue = null;
    	while (st.hasMoreTokens()){
    		parameterToken = st.nextToken();
    		parameterName = parameterToken.substring(0, parameterToken.indexOf("="));
    		parameterValue = parameterToken.substring(parameterToken.indexOf("=")+1);
    		if (serviceReq.containsAttribute(parameterName)){
    			serviceReq.updAttribute(parameterName, parameterValue);
    		} else {
    			serviceReq.setAttribute(parameterName, parameterValue);
    		}
    	}
    	
    }
    
    /**
     * Make the service name available in request also if the service was invoked with the
     * .action or .page URL
     * @param requestContainer Current RequestContainer
     * @param serviceRequest Current request
     */
    private void handleServiceName(SourceBean serviceRequest, final RequestContainer requestContainer) {
    	String path = ((String)requestContainer.getAttribute(HTTP_REQUEST_SERVLET_PATH)).toUpperCase();
    	try {
            if (path.endsWith(Constants.ACTION_URL_PATTERN)) {
            	serviceRequest.setAttribute(Constants.ACTION_NAME, path.substring(1, path.length() - Constants.ACTION_URL_PATTERN.length()));
            } else if (path.endsWith(Constants.PAGE_URL_PATTERN)) {
            	serviceRequest.setAttribute(Constants.PAGE, path.substring(1, path.length() - Constants.PAGE_URL_PATTERN.length()));
            }
    	} catch (SourceBeanException ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "AdapterHTTP::handleServiceName: ", ex);
    	}
    }
    
    private void handleSuspendResume(SourceBean serviceRequest, RequestContainer requestContainer) 
    	throws SourceBeanException {
        String deleteSuspendResumeId = (String)serviceRequest.getAttribute("DELETE_SUSPEND_RESUME_ID"); 
        if ((deleteSuspendResumeId != null) && (deleteSuspendResumeId.trim().length() > 0)){
        	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
					"AdapterHTTP::service: DELETE_SUSPEND_RESUME_ID ["+deleteSuspendResumeId+"] FOUND IN SERVICE REQUEST : DELETE SUSPEND RESUME CONTAINERS");
        	SessionContainer aPermanentContainer = requestContainer.getSessionContainer().getPermanentContainer();
        	SourceBean suspendedResumeContainers = (SourceBean)aPermanentContainer.getAttribute(deleteSuspendResumeId);
        	/*
        	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
					"AdapterHTTP::service: DELETE_SUSPEND_RESUME_ID Container to delete" + suspendedResumeContainers);
        	*/
        	if (suspendedResumeContainers != null){
        		aPermanentContainer.delAttribute(deleteSuspendResumeId);
        	}
        	/*
        	suspendedResumeContainers = (SourceBean)aPermanentContainer.getAttribute(deleteSuspendResumeId);
        	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
					"AdapterHTTP::service: DELETE_SUSPEND_RESUME_ID Container to delete" + suspendedResumeContainers);
        	*/
        }	
        // Suspend Service Request
        String suspendResumeId = (String)serviceRequest.getAttribute("SUSPEND_RESUME_ID"); 
        if ((suspendResumeId != null) && (suspendResumeId.trim().length() > 0)){
        	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
					"AdapterHTTP::service: SUSPEND_RESUME_ID ["+suspendResumeId+"] FOUND IN SERVICE REQUEST SUSPEND CURRENT SERVICE REQUEST");
        	SessionContainer aPermanentContainer = requestContainer.getSessionContainer().getPermanentContainer();
        	SourceBean suspendedResumeContainers = (SourceBean)aPermanentContainer.getAttribute(suspendResumeId);
            
            if (suspendedResumeContainers == null) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                        "AdapterHTTP::service: SUSPENDED-RESUME-CONTAINER NOT FOUND");
            } else if (suspendedResumeContainers.getAttribute(Constants.SERVICE_REQUEST) != null){
        		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
        				"AdapterHTTP::service: SUSPENDED-RESUME-CONTAINER CONTAINS ALREADY A SERVICE REQUEST OVERWRITE IT");
        		suspendedResumeContainers.updAttribute(serviceRequest);
        	} else {
        		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
        				"AdapterHTTP::service: SUSPENDED-RESUME-CONTAINER CONTAINS ALREADY A SERVICE REQUEST WRITE IT");
        		suspendedResumeContainers.setAttribute(serviceRequest);
        	}
        }
    }
    
    private void handleMultipartForm(HttpServletRequest request, RequestContextIFace requestContext) 
    	throws Exception{
    	SourceBean serviceRequest = requestContext.getServiceRequest();
    	
    	// Create a factory for disk-based file items
    	FileItemFactory factory = new DiskFileItemFactory();
    	
        // Create a new file upload handler
    	ServletFileUpload upload = new ServletFileUpload(factory);

        // Parse the request
        List fileItems = upload.parseRequest(request);
        Iterator iter = fileItems.iterator();
        while (iter.hasNext()) {
            FileItem item = (FileItem) iter.next();

            if (item.isFormField()) {
                String name = item.getFieldName();
                String value = item.getString();
                serviceRequest.setAttribute(name, value);
            } else {
                processFileField(item, requestContext);
            }
        }
    }
    
    private void handleSimpleForm(HttpServletRequest request, RequestContextIFace requestContext)
    	throws SourceBeanException{
    	SourceBean serviceRequest = requestContext.getServiceRequest();
        Enumeration names = request.getParameterNames();
        while (names.hasMoreElements()) {
            String parameterName = (String) names.nextElement();
            
            if (parameterName.startsWith("QUERY_STRING")){
            	handleQueryStringField(request, serviceRequest, parameterName);
            }else{
            	String[] parameterValues = request.getParameterValues(parameterName);
            	if (parameterValues != null)
            		for (int i = 0; i < parameterValues.length; i++)
            			serviceRequest.setAttribute(parameterName, parameterValues[i]);
            }
        } // while (names.hasMoreElements())
    }
    
    public void service(HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "AdapterHTTP::service: invocato");
        Monitor monitor = null;
        try {
            SourceBean serviceRequest = null;
            EMFErrorHandler emfErrorHandler = null;
            
            RequestContainer requestContainer = new RequestContainer();
            RequestContainer.setRequestContainer(requestContainer);
            ResponseContainer responseContainer = new ResponseContainer();
            RequestContextIFace requestContext = new DefaultRequestContext(requestContainer,
                    responseContainer);
            ResponseContainer loopbackResponseContainer = ResponseContainerAccess
                    .getResponseContainer(request);
            if (loopbackResponseContainer != null) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                        "AdapterHTTP::service: loop-back rilevato");
                serviceRequest = loopbackResponseContainer.getLoopbackServiceRequest();
                if (serviceRequest == null)
                    serviceRequest = new SourceBean(Constants.SERVICE_REQUEST);
                else {
                    Object newServiceRequest = serviceRequest
                            .getAttribute(Constants.SERVICE_REQUEST);
                    if ((newServiceRequest != null) && (newServiceRequest instanceof SourceBean))
                        serviceRequest = (SourceBean) newServiceRequest;
                } // if (serviceRequest == null)
                requestContainer.setServiceRequest(serviceRequest);
                emfErrorHandler = loopbackResponseContainer.getErrorHandler();
                if (emfErrorHandler == null)
                    emfErrorHandler = new EMFErrorHandler();
            } // if (loopbackResponseContainer != null)
            else {
                monitor = MonitorFactory.start("controller.adapter.http");
                serviceRequest = new SourceBean(Constants.SERVICE_REQUEST);
                requestContainer.setServiceRequest(serviceRequest);

                // Get header parameter before parsing the request
                setHttpRequestData(request, requestContainer);
                
                // Check if the service was invoked with the .action or .page URL
                handleServiceName(serviceRequest, requestContainer);
                
                boolean isMultipart = ServletFileUpload.isMultipartContent(new ServletRequestContext(request));
                if (isMultipart) {
                	handleMultipartForm(request, requestContext);
                } else {
                	handleSimpleForm(request, requestContext);
                }

                emfErrorHandler = new EMFErrorHandler();
            } // if (loopbackResponseContainer != null) else

            HttpSession session = request.getSession(true);
            checkSession(session, requestContext);
            
            String channelType = Constants.HTTP_CHANNEL;
            String channelTypeParameter = (String) (serviceRequest
                    .getAttribute(Constants.CHANNEL_TYPE));
            String channelTypeHeader = (String) (requestContainer.getAttribute(HTTP_ACCEPT_HEADER));
            if (((channelTypeParameter != null) && channelTypeParameter
                    .equalsIgnoreCase(Constants.WAP_CHANNEL))
                    || ((channelTypeHeader != null) && (channelTypeHeader.indexOf(WAP_MIME_TYPE) != -1)))
                channelType = Constants.WAP_CHANNEL;
            requestContainer.setChannelType(channelType);
            requestContainer.setInternalRequest(request);
            requestContainer.setInternalResponse(response);
            requestContainer.setAdapterConfig(getServletConfig());
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "AdapterHTTP::service: requestContainer", requestContainer);
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "AdapterHTTP::service: sessionContainer", requestContainer
                            .getSessionContainer());

            responseContainer.setErrorHandler(emfErrorHandler);
            SourceBean serviceResponse = new SourceBean(Constants.SERVICE_RESPONSE);
            responseContainer.setServiceResponse(serviceResponse);
            try {
                Navigator.checkNavigation(requestContainer);
            } // try
            catch (NavigationException ne) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                        "AdapterHTTP::service: ", ne);
                requestContainer.setServiceRequest(NavigationErrorUtility
                        .getNavigationErrorServiceRequest());
            } // catch (NavigationException ne)
            
            // Refresh service request because Navigator services can changed it
            serviceRequest = requestContainer.getServiceRequest();
            
            // Suspend/Resume service
            handleSuspendResume(serviceRequest, requestContainer);
            
            CoordinatorIFace coordinator = DispatcherManager.getCoordinator(requestContext);
            Exception serviceException = null;
            if (coordinator == null) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                        "AdapterHTTP::service: coordinator nullo !");
                serviceException = new Exception("Coordinatore non trovato");
                emfErrorHandler.addError(new EMFInternalError(EMFErrorSeverity.ERROR,
                        "Coordinatore non trovato !"));
            } // if (coordinator == null)
            else {
                ((RequestContextIFace) coordinator).setRequestContext(requestContext);
                responseContainer.setBusinessType(coordinator.getBusinessType());
                responseContainer.setBusinessName(coordinator.getBusinessName());
                responseContainer.setPublisherName(coordinator.getPublisherName());
                try {
                    coordinator.service(serviceRequest, serviceResponse);
                } // try
                catch (Exception ex) {
                    TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                            "AdapterHTTP::service:", ex);
                    serviceException = ex;
                    emfErrorHandler.addError(new EMFInternalError(EMFErrorSeverity.ERROR, ex));
                } // catch (Exception ex)
                ((RequestContextIFace) coordinator).setRequestContext(null);
            } // if (coordinator == null) else
            requestContainer.setInternalRequest(null);
            requestContainer.setInternalResponse(null);
            requestContainer.setAdapterConfig(null);
            // nel caso in cui sia attiva la persistenza della sessione
            // forza la scrittura sul database
            synchronized (session) {
                session.setAttribute(REQUEST_CONTAINER, session.getAttribute(REQUEST_CONTAINER));
            } // synchronized (session)
            
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "AdapterHTTP::service: responseContainer", responseContainer);
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "AdapterHTTP::service: sessionContainer", requestContainer
                            .getSessionContainer());

            if (serializeSession) {
                TracerSingleton
                        .log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                                "AdapterHTTP::service: sessionContainer size ["
                                        + Serializer.serialize(requestContainer
                                                .getSessionContainer()).length + "]");
            }

            render(requestContext, request, response, session, serviceException);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "AdapterHTTP::service: ", ex);
        } // catch (Excpetion ex) try
        finally {
            RequestContainer.delRequestContainer();
            if (monitor != null)
                monitor.stop();
        } // finally
    } // public void service(HttpServletRequest request, HttpServletResponse
    // response) throws IOException, ServletException

    private void processFileField(final FileItem item, RequestContextIFace requestContext)
            throws Exception {
        String uploadManagerName = (String)ConfigSingleton.getInstance().getAttribute("UPLOAD.UPLOAD-MANAGER.NAME");
        if (uploadManagerName == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "AdapterHTTP::processFileField: metodo di upload non selezionato");
        }
        
        IUploadHandler uploadHandler = UploadFactory.getHandler(uploadManagerName);
        if (uploadHandler instanceof RequestContextIFace) {
            ((RequestContextIFace) uploadHandler).setRequestContext(requestContext);
        }
        uploadHandler.upload(item);
    }
    
    private void checkSession(HttpSession session, RequestContextIFace requestContext) {
        boolean isRequestedSessionIdValid = true;
        RequestContainer requestContainer = requestContext.getRequestContainer();
        SourceBean serviceRequest = requestContainer.getServiceRequest();
        
        if (session.isNew()) {
            String newSessionString = (String) (serviceRequest.getAttribute(NEW_SESSION));
            isRequestedSessionIdValid = ((newSessionString != null) && (newSessionString
                    .equalsIgnoreCase("TRUE")));
        } // if (session.isNew())
        synchronized (session) {
            RequestContainer parentRequestContainer = (RequestContainer) session
                    .getAttribute(REQUEST_CONTAINER);
            if (!Navigator.isNavigatorEnabled()) {
                if (parentRequestContainer == null)
                    requestContainer.setSessionContainer(new SessionContainer(true));
                else
                    requestContainer.setSessionContainer(parentRequestContainer
                            .getSessionContainer());
            }
            else {
                if (parentRequestContainer == null)
                    requestContainer.setSessionContainer(new SessionContainer(true));
                else {
                    requestContainer.setSessionContainer(new SessionContainer(false));
                    requestContainer.setParent(parentRequestContainer);
                } // if (parentRequestContainer == null) else
            } // if (!Navigator.isNavigatorEnabled())
            session.setAttribute(REQUEST_CONTAINER, requestContainer);
        } // synchronized (session)
        if (!isRequestedSessionIdValid) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "AdapterHTTP::service: sessione scaduta !");
            SessionExpiredUtility.setSessionExpiredAction(serviceRequest);
        } // if (!isRequestedSessionIdValid)
    }

    private void render(RequestContextIFace requestContext, HttpServletRequest request,
            HttpServletResponse response, HttpSession session, Exception serviceException)
            throws Exception {
        RequestContainer requestContainer = requestContext.getRequestContainer();
        ResponseContainer responseContainer = requestContext.getResponseContainer();

       
        Boolean isHttpResponseFreezed = (Boolean) responseContainer
                .getAttribute(Constants.HTTP_RESPONSE_FREEZED);
        if ((isHttpResponseFreezed == null) || (!isHttpResponseFreezed.booleanValue())) {
            String publisherType = null;
            ArrayList resources = null;
            Publisher publisher = Publisher.getPublisher(requestContext,
                    serviceException);
            if (publisher == null) {
                publisherType = Constants.NOTHING_PUBLISHER_TYPE;
                resources = null;
            } // if (publisher == null)
            else {
                publisherType = publisher.getType();
                if (publisherType.equalsIgnoreCase(Constants.NOTHING_PUBLISHER_TYPE))
                    resources = null;
                else if (publisherType.equalsIgnoreCase(Constants.LOOP_PUBLISHER_TYPE)) {
                    resources = new ArrayList();
                    // Modifica Monica 15/01/2004 - inizio
                    // costruito SourceBean per busta ITEM contenente
                    // l'attributo resource ="/servlet/AdapterHTTP"
                    SourceBean item = new SourceBean("ITEM");
                    item.setAttribute("prog", "0");
                    item.setAttribute("resource", "/servlet/AdapterHTTP");
                    resources.add(item);
                    // Modifica Monica 15/01/2004 - fine
                } // if
                // (publisherType.equalsIgnoreCase(AF_PUBLISHER_TYPE))
                else {
                    resources = publisher.getResources();
                    if (resources.size() == 0) {
                        publisherType = Constants.NOTHING_PUBLISHER_TYPE;
                        resources = null;
                    } // if (resources.size() == 0)
                } // if
                // (publisherType.equalsIgnoreCase(Constants.LOOP_PUBLISHER_TYPE))
                // else
            } // if (publisher == null) else

            // Memorizzo in sessione il nome dell'ultimo publisher utilizzato
            String publisherName = (publisher != null) ? publisher.getName() : null;
            Publisher.setLastPublisherName(publisherName);

            if (publisherType.equalsIgnoreCase(Constants.LOOP_PUBLISHER_TYPE)
                    || publisherType.equalsIgnoreCase(SERVLET_PUBLISHER_TYPE)
                    || publisherType.equalsIgnoreCase(JSP_PUBLISHER_TYPE)) {
                // Modifica Monica 15/01/2004 - inizio
                // recupero il SourceBean della busta ITEM, contenente la
                // busta CONFIG
            	
            	CacheResponseHandler.service(requestContainer, responseContainer, publisherName);
                String resource = (String) ((SourceBean) resources.get(0)).getAttribute("resource");
                // Modifica Monica 15/01/2004 - fine
                String publisherMode = publisher.getMode();
                if (publisherMode == null) {
                    TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                            "AdapterHTTP::service: publisherMode nullo, default [FORWARD]");
                    publisherMode = FORWARD_PUBLISHING_MODE;
                } // if (publisherMode == null)
                String requestContainerName = Constants.REQUEST_CONTAINER
                        + requestContainer.hashCode();
                String responseContainerName = Constants.RESPONSE_CONTAINER
                        + responseContainer.hashCode();
                Router router = null;
                if (publisherMode.equalsIgnoreCase(SENDREDIRECT_PUBLISHING_MODE)) {
                    session.setAttribute(requestContainerName, requestContainer);
                    session.setAttribute(responseContainerName, responseContainer);
                    router = new Router(resource, false);
                } // if
                // (publisherMode.equalsIgnoreCase(SENDREDIRECT_PUBLISHING_MODE))
                else {
                    request.setAttribute(requestContainerName, requestContainer);
                    request.setAttribute(responseContainerName, responseContainer);
                    router = new Router(resource, true);
                } // if
                // (publisherMode.equalsIgnoreCase(SENDREDIRECT_PUBLISHING_MODE))
                // else
                router.setParameter(Constants.REQUEST_CONTAINER, requestContainerName);
                router.setParameter(Constants.RESPONSE_CONTAINER, responseContainerName);
                router.route(getServletContext(), request, response);
            } // if (publisherType.equalsIgnoreCase(AF_PUBLISHER_TYPE)
            // || publisherType.equalsIgnoreCase(SERVLET_PUBLISHER_TYPE) ||
            // publisherType.equalsIgnoreCase(JSP_PUBLISHER_TYPE))
            else {
                boolean isXMLRequest = false;

                // Check if the request is an AJAX request
                String requestedWith = (String)requestContainer.getAttribute(HTTP_REQUESTED_WITH);
                if ((requestedWith != null) && (requestedWith.equalsIgnoreCase(XMLHTTPREQUEST))) {
                	// This is probably an AJAX request
                    isXMLRequest = true;
                    requestContainer.setAttribute(Constants.HTTP_IS_XML_REQUEST, "TRUE");
                }

                if (requestContainer.getChannelType().equalsIgnoreCase(Constants.WAP_CHANNEL))
                    response.setContentType(WAP_CONTENT_TYPE);
                else if (isXMLRequest)
                	// For AJAX requests the response should be XML
                    response.setContentType(XML_CONTENT_TYPE);
                else
                	response.setContentType(HTTP_CONTENT_TYPE);
                
                Monitor renderingMonitor = null;
                if (publisher == null)
                    renderingMonitor = MonitorFactory.start("view."
                            + requestContainer.getChannelType().toLowerCase() + "."
                            + publisherType.toLowerCase());
                else
                    renderingMonitor = MonitorFactory
                            .start("view." + requestContainer.getChannelType().toLowerCase() + "."
                                    + publisherType.toLowerCase() + "."
                                    + publisherName.toLowerCase());
                try {
                    response.getWriter().print(
                            PresentationRendering.render(requestContext, resources));
                } // try
                catch (Exception ex) {
                    throw ex;
                } // catch (Exception ex)
                finally {
                    renderingMonitor.stop();
                } // finally
                response.getWriter().flush();
                return;
            } // if (publisherType.equalsIgnoreCase(AF_PUBLISHER_TYPE)
            // || publisherType.equalsIgnoreCase(SERVLET_PUBLISHER_TYPE) ||
            // publisherType.equalsIgnoreCase(JSP_PUBLISHER_TYPE)) else
        } // if ((isHttpResponseFreezed == null) ||
        // (!isHttpResponseFreezed.getBoolean()))
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "AdapterHTTP::service: http response congelata");
    }

    private void setHttpRequestData(HttpServletRequest request, RequestContainer requestContainer) {
        requestContainer.setAttribute(HTTP_REQUEST_AUTH_TYPE, request.getAuthType());
        requestContainer.setAttribute(HTTP_REQUEST_CHARACTER_ENCODING, String.valueOf(request
                .getCharacterEncoding()));
        requestContainer.setAttribute(HTTP_REQUEST_CONTENT_LENGTH, String.valueOf(request
                .getContentLength()));
        requestContainer.setAttribute(HTTP_REQUEST_CONTENT_TYPE, request.getContentType());
        requestContainer.setAttribute(HTTP_REQUEST_CONTEXT_PATH, request.getContextPath());
        requestContainer.setAttribute(HTTP_REQUEST_METHOD, request.getMethod());
        requestContainer.setAttribute(HTTP_REQUEST_PATH_INFO, request.getPathInfo());
        requestContainer.setAttribute(HTTP_REQUEST_PATH_TRANSLATED, request.getPathTranslated());
        requestContainer.setAttribute(HTTP_REQUEST_PROTOCOL, request.getProtocol());
        requestContainer.setAttribute(HTTP_REQUEST_QUERY_STRING, request.getQueryString());
        requestContainer.setAttribute(HTTP_REQUEST_REMOTE_ADDR, request.getRemoteAddr());
        requestContainer.setAttribute(HTTP_REQUEST_REMOTE_HOST, request.getRemoteHost());
        requestContainer.setAttribute(HTTP_REQUEST_REMOTE_USER, request.getRemoteUser());
        requestContainer.setAttribute(HTTP_REQUEST_REQUESTED_SESSION_ID, request
                .getRequestedSessionId());
        requestContainer.setAttribute(HTTP_REQUEST_REQUEST_URI, request.getRequestURI());
        requestContainer.setAttribute(HTTP_REQUEST_SCHEME, request.getScheme());
        requestContainer.setAttribute(HTTP_REQUEST_SERVER_NAME, request.getServerName());
        requestContainer.setAttribute(HTTP_REQUEST_SERVER_PORT, String.valueOf(request
                .getServerPort()));
        requestContainer.setAttribute(HTTP_REQUEST_SERVLET_PATH, request.getServletPath());
        if (request.getUserPrincipal() != null)
            requestContainer.setAttribute(HTTP_REQUEST_USER_PRINCIPAL, request.getUserPrincipal());
        requestContainer.setAttribute(HTTP_REQUEST_REQUESTED_SESSION_ID_FROM_COOKIE, String
                .valueOf(request.isRequestedSessionIdFromCookie()));
        requestContainer.setAttribute(HTTP_REQUEST_REQUESTED_SESSION_ID_FROM_URL, String
                .valueOf(request.isRequestedSessionIdFromURL()));
        requestContainer.setAttribute(HTTP_REQUEST_REQUESTED_SESSION_ID_VALID, String
                .valueOf(request.isRequestedSessionIdValid()));
        requestContainer.setAttribute(HTTP_REQUEST_SECURE, String.valueOf(request.isSecure()));
        Enumeration headerNames = request.getHeaderNames();
        while (headerNames.hasMoreElements()) {
            String headerName = (String) headerNames.nextElement();
            String headerValue = request.getHeader(headerName);
            requestContainer.setAttribute(headerName, headerValue);
        } // while (headerNames.hasMoreElements())
        requestContainer.setAttribute(HTTP_SESSION_ID, request.getSession().getId());
        requestContainer.setAttribute(Constants.HTTP_IS_XML_REQUEST, "FALSE");
    } // private void setHttpRequestData(HttpServletRequest request,

    // RequestContainer requestContainer)

    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.GenericServlet#init()
     */
    public void init() throws ServletException {
        super.init();

        String serializeSessionStr = (String) ConfigSingleton.getInstance().getAttribute(
                SERIALIZE_SESSION_ATTRIBUTE);
        if ((serializeSessionStr != null) && (serializeSessionStr.equalsIgnoreCase("TRUE"))) {
            serializeSession = true;
        }
    }

} // public class ActionServlet extends HttpServlet
