/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.soapchannel;

import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;

import org.apache.soap.Constants;
import org.apache.soap.Fault;
import org.apache.soap.SOAPException;
import org.apache.soap.rpc.Call;
import org.apache.soap.rpc.Parameter;
import org.apache.soap.rpc.Response;
import org.apache.soap.transport.http.SOAPHTTPConnection;

public class AdapterSOAPProxy {
    private Method _setTcpNoDelayMethod = null;
    private Call _call = null;
    private URL _url = null;

    public AdapterSOAPProxy() {
        super();
        try {
            _setTcpNoDelayMethod =
                SOAPHTTPConnection.class.getMethod(
                    "setTcpNoDelay",
                    new Class[] { Boolean.class });
        } // try
        catch (Exception ex) {
            ex.printStackTrace();
        } // catch (Exception ex) try
        _call = createCall();
        _url = null;
    } // public AdapterSOAPProxy()

    public synchronized void setEndPoint(URL url) {
        _url = url;
    } // public synchronized void setEndPoint(URL url)

    public synchronized URL getEndPoint() throws MalformedURLException {
        return getURL();
    } // public synchronized URL getEndPoint() throws MalformedURLException

    private URL getURL() throws MalformedURLException {
        if (_url == null)
            _url = new URL("http://localhost:8080/AFDemoWeb/servlet/rpcrouter");
        return _url;
    } // private URL getURL() throws MalformedURLException

    public synchronized String service(String request) throws Exception {
        String targetObjectURI =
            "http://tempuri.org/it.eng.spago.dispatching.soapchannel.AdapterSOAP";
        String SOAPActionURI = "";
        if (getURL() == null) {
            throw new SOAPException(
                Constants.FAULT_CODE_CLIENT,
                "A URL must be specified via AdapterSOAPProxy.setEndPoint(URL).");
        } // if (getURL() == null)
        _call.setMethodName("service");
        _call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
        _call.setTargetObjectURI(targetObjectURI);
        Vector params = new Vector();
        Parameter requestParam =
            new Parameter(
                "request",
                String.class,
                request,
                Constants.NS_URI_SOAP_ENC);
        params.addElement(requestParam);
        _call.setParams(params);
        Response resp = _call.invoke(getURL(), SOAPActionURI);
        if (resp.generatedFault()) {
            Fault fault = resp.getFault();
            _call.setFullTargetObjectURI(targetObjectURI);
            throw new SOAPException(
                fault.getFaultCode(),
                fault.getFaultString());
        } // if (resp.generatedFault())
        else {
            Parameter refValue = resp.getReturnValue();
            return ((String) refValue.getValue());
        } // if (resp.generatedFault()) else
    } // public synchronized String service(String request) throws Exception

    protected Call createCall() {
        SOAPHTTPConnection soapHTTPConnection = new SOAPHTTPConnection();
        if (_setTcpNoDelayMethod != null) {
            try {
                _setTcpNoDelayMethod.invoke(
                    soapHTTPConnection,
                    new Object[] { Boolean.TRUE });
            } // try
            catch (Exception ex) {
                ex.printStackTrace();
            } // catch (Exception ex) try
        } // if (setTcpNoDelayMethod != null)
        Call call = new Call();
        call.setSOAPTransport(soapHTTPConnection);
        /*SOAPMappingRegistry smr = */call.getSOAPMappingRegistry();
        return call;
    } // protected Call createCall()
} // public class AdapterSOAPProxy
