/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.tags;

import java.util.Vector;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.navigation.LabelID;
import it.eng.spago.navigation.Navigator;
import it.eng.spago.tracing.TracerSingleton;

public class ComboNavigationToolbarTag extends TagSupport {

    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    public static final String NAVIGATOR_SERVICE = "NAVIGATOR_SERVICE";

    public int doStartTag() throws JspException {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DefaultNavigationToolbarTag::doStartTag:: invocato");
        if (pageContext == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "ComboNavigationToolbarTag::doStartTag:: pageContext nullo");
            throw new JspException("pageContext nullo");
        } // if (_httpRequest == null)
        HttpServletRequest httpRequest = null;
        try {
            httpRequest = (HttpServletRequest) pageContext.getRequest();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DefaultNavigationToolbarTag::doStartTag::",
                ex);
            throw new JspException(ex.getMessage());
        } // catch (Exception ex)
        if (httpRequest == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "ComboNavigationToolbarTag::doStartTag:: httpRequest nullo");
            throw new JspException("httpRequest nullo");
        } // if (_httpRequest == null)
        ResponseContainer responseContainer =
            ResponseContainerAccess.getResponseContainer(httpRequest);
        if (responseContainer == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "ComboNavigationToolbarTag::doStartTag:: responseContainer nullo");
            throw new JspException("responseContainer nullo");
        } // if (responseContainer == null)
        StringBuffer output = new StringBuffer();

        RequestContainer requestContainer =
            RequestContainerAccess.getRequestContainer(httpRequest);
        generateForm(output, requestContainer, responseContainer);
        try {
            pageContext.getOut().print(output.toString());
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "ComboNavigationToolbarTag",
                ex);
            throw new JspException(ex.getMessage());
        } // catch (Exception ex)
        return SKIP_BODY;
    } // public int doStartTag() throws JspException

    public int doEndTag() throws JspException {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "ComboNavigationToolbarTag::doEndTag:: invocato");
        return super.doEndTag();
    } // public int doEndTag() throws JspException

    protected void generateForm(
        StringBuffer output,
        RequestContainer requestContainer,
        ResponseContainer responseContainer)
        throws JspException {
        generateJavaScript(output, requestContainer, responseContainer);
        output.append("<form name=\"navigation_combo\">\n");
        generateCombo(output, requestContainer, responseContainer);
        output.append("</form>\n");
    } //protected void generateForm(StringBuffer output, RequestContainer requestContainer, ResponseContainer responseContainer) throws JspException      

    protected void generateJavaScript(
        StringBuffer output,
        RequestContainer requestContainer,
        ResponseContainer responseContainer)
        throws JspException {
        output.append(
            "<SCRIPT language=\"Javascript\" type=\"text/javascript\"> \n");
        output.append("function goTo() { \n");
        output.append(
            "  window.location = document.navigation_combo.pagina.options[document.navigation_combo.pagina.selectedIndex].value; \n");
        output.append("} \n");
        output.append("</SCRIPT> \n");
    } //protected void generateJavaScript(StringBuffer output, RequestContainer requestContainer, ResponseContainer responseContainer) throws JspException 

    protected void generateCombo(
        StringBuffer output,
        RequestContainer requestContainer,
        ResponseContainer responseContainer)
        throws JspException {
        Vector ordered = new Vector();
        RequestContainer container = requestContainer;
        if (container != null)
            container = (RequestContainer) container.getParent();
        while (container != null) {
            LabelID labelID =
                (LabelID) container.getAttribute(NAVIGATOR_SERVICE);
            if (labelID != null) {
                ordered.addElement(labelID);
            } //if (label != null)
            container = (RequestContainer) container.getParent();
        } // while (container != null)
        if (ordered.size() != 0) {
            output.append("<TABLE><tr><td>\n");
            output.append("<select name =\"pagina\" onSelect=\"goTo();\">\n");
            //scorro vettore in ordine dalla pi recente alla pi vecchia e creo html
            for (int i = 0; i < ordered.size(); i++) {
                LabelID label = (LabelID) ordered.elementAt(i);
                output.append(
                    " <option value =\"AdapterHTTP?"
                        + Navigator.NAVIGATOR_BACK_TO_SERVICE_ID
                        + "=");
                output.append(label.getId());
                output.append("&NEW_SESSION=TRUE\">");
                output.append(label.getLabel());
                output.append(" </option>\n");
            } //for(int i = 0; i < ordered.size(); i++)
            output.append("</select>");
            output.append(
                "<input type=\"button\" value=\"Go\" onClick=\"goTo();\"></p>");
        } //if(ordered.size() != 0)
        output.append("</td></tr></TABLE>\n");
    } // protected void generateToolbar(StringBuffer output, SourceBean _layout, RequestContainer requestContainer,
} //public class ComboNavigationToolbarTag extends TagSupport
