/**

Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

This file is part of Spago.

Spago is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
any later version.

Spago is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Spago; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.tags;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.dispatching.TokenHandler;
import it.eng.spago.tracing.TracerSingleton;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * @author Boccalon
 */
public class GenerateTokenTag extends TagSupport {

	/**
     * 
     */
    private static final long serialVersionUID = 1L;

    /**
	 * Constructor for GenerateTokenTag.
	 */
	public GenerateTokenTag() {
		super();
	}

    public int doStartTag() throws JspException {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "GenerateTokenTag::doStartTag:: invocato");
        HttpServletRequest httpRequest = (HttpServletRequest)pageContext.getRequest();
        RequestContainer requestContainer = RequestContainerAccess.getRequestContainer(httpRequest);
        if (requestContainer == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "GenerateTokenTag::doStartTag:: requestContainer nullo");
            return SKIP_BODY;
        } // if (_content == null)

        String token = (String)requestContainer.getSessionContainer().getPermanentContainer().getAttribute(TokenHandler.TRANSACTION_TOKEN_KEY);
        if (token == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "GenerateTokenTag::doStartTag:: token nullo");
            return SKIP_BODY;
        } // if (_content == null)

        StringBuffer _htmlStream = new StringBuffer();
        _htmlStream.append("<input type=\"hidden\" name=\"");
        _htmlStream.append(TokenHandler.REQUEST_TOKEN_KEY);
        _htmlStream.append("\" value=\"");        
        _htmlStream.append(token);
        _htmlStream.append("\">");
        
        try {
            pageContext.getOut().print(_htmlStream);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "GenerateTokenTag::doStartTag:: Impossibile inviare lo stream");
            throw new JspException("Impossibile inviare lo stream");
        } // catch (Exception ex)
        return SKIP_BODY;
    } // public int doStartTag() throws JspException

    /**
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag() throws JspException {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "GenerateTokenTag::doEndTag:: invocato");
        return super.doEndTag();
    } // public int doEndTag() throws JspException

}
