/**

Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

This file is part of Spago.

Spago is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
any later version.

Spago is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Spago; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/

package it.eng.spago.tags;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.tracing.TracerSingleton;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * 
 * This tag take an id as input. It verify if there are container suspended with a given id.
 * If there are suspended containers it merge with new ones otherwise provide to suspend the current service response
 * 
 * @author Zoppello
 */
public class SuspendResumeTag extends TagSupport {
	
    private static final long serialVersionUID = 1L;

    private String id = null;
	public SuspendResumeTag() {
		super();
	}
	
	public int doStartTag() throws JspException {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
                "SuspendResumeTag::doStartTag: BEGIN");
        
        HttpServletRequest httpRequest = (HttpServletRequest)pageContext.getRequest();
        
        RequestContainer requestContainer = RequestContainerAccess.getRequestContainer(httpRequest);
        
        ResponseContainer responseContainer = ResponseContainerAccess.getResponseContainer(httpRequest);
        
        SessionContainer sessionContainer = requestContainer.getSessionContainer();
        
        SessionContainer permanentContainer = sessionContainer.getPermanentContainer();
        
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
        			"SuspendResumeTag::doStartTag: Container Retrieved");
        
        SourceBean suspendResume = (SourceBean)permanentContainer.getAttribute(id); 
        
        SourceBean currentServiceRequest = responseContainer.getServiceResponse();
        
        SourceBean currentServiceResponse = responseContainer.getServiceResponse();
        try{
        if (( suspendResume != null)) {
        	
        	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
        			"SuspendResumeTag::doStartTag: SUSPEND-RESUME WITH ["+id+"]  FOUND ");
        	
        	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
        			"SuspendResumeTag::doStartTag: ******** START RESUME PROCESS ********* ");
        	
        	// -- Devo fare un resume
        	
        	SourceBean resumedServiceRequest = (SourceBean)suspendResume.getAttribute("SERVICE_REQUEST");
        	
        	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
        			"SuspendResumeTag::doStartTag: Resumed Service Request is" ,resumedServiceRequest);
        	SourceBean resumedServiceResponse = (SourceBean)suspendResume.getAttribute("SERVICE_RESPONSE");
        
        	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
         			"SuspendResumeTag::doStartTag: Resumed Service Response is" ,resumedServiceResponse);
        	List listAttributes = null;
        	SourceBeanAttribute sba = null;
        	if (resumedServiceResponse != null){
        		listAttributes = resumedServiceResponse.getContainedAttributes();
            	
            	
            	for (int i=0; i < listAttributes.size(); i++){
            		sba = (SourceBeanAttribute)listAttributes.get(i);
            		String key = sba.getKey();
            		if (currentServiceResponse.getAttribute(key) == null){
            			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
                     			"SuspendResumeTag::doStartTag: Resuming Attribute ["+key+"] from Resumed Service Response to current service response");
            			currentServiceResponse.setAttribute(key, sba.getValue());
            		}else{
            			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
                     			"SuspendResumeTag::doStartTag: !! CANNOT RESUME Attribute ["+key+"]  this attribute is already contained in Current Service Response" );
            		}
            	}
        	}
        	if (resumedServiceRequest != null){
        		
        		listAttributes = resumedServiceRequest.getContainedAttributes();
            	for (int i=0; i < listAttributes.size(); i++){
            		sba = (SourceBeanAttribute)listAttributes.get(i);
            		String key = sba.getKey();
            		if (currentServiceRequest.getAttribute(key) == null){
            			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
                     			"SuspendResumeTag::doStartTag: Resuming Attribute ["+key+"]  from Suspended Service Request to current service request");
            			currentServiceRequest.setAttribute(key, sba.getValue());
            		}else{
            			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
                     			"SuspendResumeTag::doStartTag: !! CANNOT RESUME Attribute ["+key+"]  this attribute is already contained in Current Service Request" );
            		}
            	}
        	}
        	
        	
        } else {
        		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
        			"SuspendResumeTag::doStartTag: ******** START SUSPEND PROCESS ********* ");
        		SourceBean suspendedResumeContainers = new SourceBean("CONTAINERS");
        		suspendedResumeContainers.setAttribute(currentServiceResponse);
        		
        		permanentContainer.setAttribute(id, suspendedResumeContainers);
        
        }
        }catch (SourceBeanException sbe) {
        	
        	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                     "SuspendResumeTag::doStartTag: ", sbe);
    		
        	throw new JspException(sbe);
		
    	}
        return SKIP_BODY;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}
	
	
}
